package cn.estsh.i3plus.core.api.iservice.base;

import io.swagger.annotations.ApiOperation;

/**
 * @Description : 系统启动加载服务
 * @Reference :
 * @Author : Adair Peng
 * @CreateDate : 2019-01-11 10:02
 * @Modify:
 **/
public interface ISystemInitService {

    /**
     * 系统初始化加载
     */
    @ApiOperation(value = "初始化加载所有数据",notes = "初始化加载所有数据")
    void putAndLoadAll();

    /**
     * 加载系统配置
     */
    @ApiOperation(value = "加载系统配置",notes = "加载系统配置")
    void putAndLoadSysConfig();

    /**
     * 加载系统字典
     */
    @ApiOperation(value = "加载字典数据",notes = "加载字典数据")
    void putAndLoadSysDictionary();

    /**
     * 加载系统语言信息
     */
    @ApiOperation(value = "加载语言数据",notes = "加载语言数据")
    void putAndLoadSysLocaleLanguage();

    @ApiOperation(value = "加载权限资源",notes = "加载权限资源")
    void putAndLoadSysMenu();

    /**
     * 从缓存获取对象数据
     * @param key       Redis Key
     * @param dataType 数据类型
     * @return
     */
    @ApiOperation(value = "获取缓存中的数据",notes = "获取缓存中的数据")
    Object getDataFromCache(String key,Class dataType);

    @ApiOperation(value = "重新加载Elasticsearch 客户端信息")
    void reloadElasticSearchClient();

    @ApiOperation(value = "对象版本记录信息")
    void reloadPojoVersionPlan();

    @ApiOperation(value = "定时任务日志等级")
    void reloadTaskPlanLogLevel();

    @ApiOperation(value = "定时任务日志等级")
    void reloadBarcodeRule();

    @ApiOperation(value = "重启短信状态同步线程")
    void createSmsSyncThread();
}
