package cn.estsh.i3plus.core.api.iservice.base;

import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.AdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.SaAdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import io.swagger.annotations.ApiOperation;

import javax.servlet.http.HttpServletRequest;

/**
 * @Description : 用户登陆方法，平台统一登陆
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-11 16:59
 * @Modify:
 **/
public interface ISystemLoginService {

    /**
     * 普通用户登录
     *
     * @param loginName    账号
     * @param loginPwd     密码
     * @param languageCode 登录语言
     * @return 用户会话
     */
    @ApiOperation(value = "用户登录", notes = "普通用户登录")
    SessionUser queryUserLogin(String loginName, String loginPwd, String languageCode);

    /**
     * 管理员登录
     *
     * @param loginName    账号
     * @param loginPwd     密码
     * @param languageCode 登录语言
     * @return 用户会话
     */
    @ApiOperation(value = "用户登录", notes = "管理用户登录")
    SessionUser queryAdminLogin(String loginName, String loginPwd, String languageCode);

    /**
     * 用户信息分装
     *
     * @param authenticationToken
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装普通用户信息")
    SessionUser queryCheckUserLogin(UserToken authenticationToken);

    /**
     * 管理员登录
     *
     * @param authenticationToken 登录信息
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装管理员信息")
    SessionUser queryCheckAdminLogin(AdminToken authenticationToken);

    /**
     * 超级管理员登录
     *
     * @param authenticationToken 登录信息
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装超级管理员信息")
    SessionUser queryCheckSaAdminLogin(SaAdminToken authenticationToken);
    /**
     * E9用户单点登录
     *
     * @param userToken 用户token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装普通用户信息")
    SessionUser queryCheckE9UserLogin(UserToken userToken);
    /**
     * E9Admin单点登录
     *
     * @param adminToken admin token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装管理员信息")
    SessionUser queryCheckE9AdminLogin(AdminToken adminToken);

    /**
     * E9 SA单点登录
     * @param saAdminToken sa token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装超级管理员信息")
    SessionUser queryCheckE9SaAdminLogin(SaAdminToken saAdminToken);

    /**
     * Name用户单点登录
     *
     * @param userToken 用户token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装普通用户信息")
    SessionUser queryCheckNameUserLogin(UserToken userToken);
    /**
     * NameAdmin单点登录
     *
     * @param adminToken admin token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装管理员信息")
    SessionUser queryCheckNameAdminLogin(AdminToken adminToken);

    /**
     * Name SA单点登录
     * @param saAdminToken sa token
     * @return 用户会话
     */
    @ApiOperation(value = "封装用户", notes = "封装超级管理员信息")
    SessionUser queryCheckNameSaAdminLogin(SaAdminToken saAdminToken);

    /**
     * 记录登录错误
     *
     * @param loginName 账号
     * @param sessionId Session ID
     * @return 用户会话
     */
    @ApiOperation(value = "登录密码错误", notes = "登录密码错误记录错误次数功能")
    Integer doLoginPasswordError(String loginName, String sessionId);

    /**
     * 获取登录的平台
     *
     * @param request http请求
     * @return 平台枚举
     */
    CommonEnumUtil.LOG_LOGIN_PLATFORM getLoginPlatform(HttpServletRequest request);
}
