package cn.estsh.i3plus.core.apiservice.auth.realm;

import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseThreadLocal;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.AdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.SaAdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.model.platform.RoleRealmModel;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.pam.ModularRealmAuthenticator;
import org.apache.shiro.realm.Realm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Collection;
import java.util.Map;

/**
 * @Description : 验证规则模块封装
 * @Reference :
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 18:37
 * @Modify:
 **/
public class DefaultModularRealm extends ModularRealmAuthenticator {
    private static final Logger LOGGER = LoggerFactory.getLogger(DefaultModularRealm.class);

    private Map<String, RoleRealmModel> definedRealms;

    public DefaultModularRealm(Map<String, RoleRealmModel> definedRealms) {
        this.definedRealms = definedRealms;
    }

    @Override
    protected AuthenticationInfo doMultiRealmAuthentication(Collection<Realm> realms, AuthenticationToken token) {
        return super.doMultiRealmAuthentication(realms, token);
    }

    @Override
    protected AuthenticationInfo doAuthenticate(AuthenticationToken token) throws AuthenticationException {
        Realm realm;
        RoleRealmModel roleRealmModel;
        String realmStrategy = (String) BaseThreadLocal.getData(PlatformConstWords.AUTH_LOGIN_STRATEGY);

        if (StringUtils.isEmpty(realmStrategy)) {
            roleRealmModel = this.definedRealms.get(ImppEnumUtil.AUTH_LOGIN_STRATEGY.ACCOUNT.getCode());
        } else {
            roleRealmModel = this.definedRealms.get(realmStrategy);
        }

        realm = chooseRealmByUserType(token, roleRealmModel);

        return this.doSingleRealmAuthentication(realm, token);
    }

    private Realm chooseRealmByUserType(AuthenticationToken token, RoleRealmModel roleRealmModel) {
        Realm realm;
        if (token instanceof SaAdminToken) {
            //超级管理员
            realm = roleRealmModel.getSaRealm();
        } else if (token instanceof AdminToken) {
            //管理员
            realm = roleRealmModel.getAdminRealm();
        } else if (token instanceof UserToken) {
            //用户
            realm = roleRealmModel.getUserRealm();
        } else {
            throw new AuthenticationException("不存在TOKEN，请联系管理员（" + token + "）");
        }
        return realm;
    }

    @Override
    protected AuthenticationInfo doSingleRealmAuthentication(Realm realm, AuthenticationToken token) {
        if (!realm.supports(token)) {
            throw new AuthenticationException("TOKEN类型异常，请联系管理员");
        } else {
            return realm.getAuthenticationInfo(token);
        }
    }
}
