package cn.estsh.i3plus.core.apiservice.auth.realm.strategy.name;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * @Description : 验证规则
 * @Reference :-
 * @Author : alwaysfrin
 * @CreateDate : 2018-10-13 14:04
 * @Modify:
 **/
public class NameUserAuthRealm extends AuthorizingRealm {
    public static final Logger LOGGER = LoggerFactory.getLogger(NameUserAuthRealm.class);

    @Autowired
    private ISystemLoginService systemLoginService;

    public NameUserAuthRealm() {
        //添加支持的token
        this.setAuthenticationTokenClass(UserToken.class);
    }

    //权限验证
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        return new SimpleAuthorizationInfo();
    }

    //令牌确认
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        try {
            //管理员令牌
            SessionUser sessionUser = systemLoginService.queryCheckNameUserLogin((UserToken) authenticationToken);
            LOGGER.info("用户令牌验证：{}", sessionUser);
            //todo:shiro 用法可能有问题, SimpleAuthenticationInfo这里的参数应该为 user信息或者username, 密码, realm name,这里没传密码传的是用户名
            //没有用shiro的验证方式，走的是自己的密码验证逻辑
            return new SimpleAuthenticationInfo(sessionUser, ((UserToken) authenticationToken).getLoginName(), this.getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new AuthenticationException(e.getMessage());
        }
    }
}
