package cn.estsh.i3plus.core.apiservice.configuration;

import cn.estsh.i3plus.pojo.platform.bean.CoreDataSource;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import java.sql.*;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2022/1/7 11:43
 * @Modify:
 **/
@Slf4j
@Data
public class CoreJdbcTemplate {
    private CoreDataSource source;

    private NamedParameterJdbcTemplate coreJdbcTemplate;

    public NamedParameterJdbcTemplate getNamedParameterJdbcTemplate() {
        return coreJdbcTemplate;
    }

    /**
     * 获取数据库连接
     *
     * @return
     */
    public Connection getJdbcTempConnection() {
        Connection conn;
        try {
            conn = coreJdbcTemplate.getJdbcTemplate().getDataSource().getConnection();
        } catch (SQLException e) {
            log.error("wmsJdbcTemplate获取数据源连接失败:{}", e.getMessage());
            throw ImppExceptionBuilder.newInstance().setErrorDetail("wmsJdbcTemplate获取数据源连接失败:" + e.getMessage()).build();
        }
        return conn;
    }

    /**
     * 开启事务，可以在使用时自己开启
     *
     * @param conn
     */
    public void beginTransaction(Connection conn) {
        try {
            if (conn != null) {
                conn.setAutoCommit(false);
            }
        } catch (SQLException e) {
            log.error("【开启事务connection】开启事务异常Error Message:{}", e.getMessage());
            throw ImppExceptionBuilder.newInstance().setErrorDetail("【开启事务connection】开启事务异常Error Message:" + e.getMessage()).build();
        }
    }

    /**
     * 提交事务
     *
     * @param conn
     */
    public void commit(Connection conn) {
        try {
            if (conn != null) {
                conn.commit();
            }
        } catch (SQLException e) {
            log.error("【提交事务connection】提交事务异常Error Message:{}", e.getMessage());
            throw ImppExceptionBuilder.newInstance().setErrorDetail("【开启事务connection】开启事务异常Error Message:" + e.getMessage()).build();
        }
    }

    /**
     * 清除连接
     *
     * @param conn
     * @param pstmt
     * @param stmt
     * @param resultSet
     */
    public static void clean(Connection conn, PreparedStatement pstmt, Statement stmt, ResultSet resultSet) {
        try {
            if (resultSet != null && !resultSet.isClosed()) {
                resultSet.close();
            }
        } catch (SQLException e) {
            log.error("【数据库连接 Connection】 关闭连接异常 Error Message:{}", e.getMessage());
        }
        try {
            if (stmt != null && !stmt.isClosed()) {
                stmt.close();
            }
        } catch (SQLException e) {
            log.error("【数据库连接 Connection】 关闭连接异常 Error Message:{}", e.getMessage());
        }
        try {
            if (pstmt != null && !pstmt.isClosed()) {
                pstmt.close();
            }
        } catch (SQLException e) {
            log.error("【数据库连接 Connection】 关闭连接异常 Error Message:{}", e.getMessage());
        }
        try {
            if (conn != null && !conn.isClosed()) {
                log.info("【数据库连接 Connection】 关闭连接");
                conn.close();
            }
        } catch (SQLException e) {
            log.error("【数据库连接 Connection】 关闭连接异常 Error Message:{}", e.getMessage());
        }
    }


    /**
     * 获取数据源isRead or Transaction
     * @param isRead
     * @return
     * @throws SQLException
     */
    public Connection getJdbcTempConnection(boolean isRead) throws SQLException {
        Connection conn;
        if(isRead){
            log.info("【数据库】 事物提示：ReadOnly ");
            conn = coreJdbcTemplate.getJdbcTemplate().getDataSource().getConnection();
            conn.setReadOnly(true);

            log.info("【表单数据源切换，只读：{}】", conn);
        }else{
            log.info("【数据库】 事物提示：beginTransaction ");
            conn = coreJdbcTemplate.getJdbcTemplate().getDataSource().getConnection();
            //开启事务
            beginTransaction(conn);
            log.info("【表单数据源切换，只写：{}】", conn);
        }
        return conn;
    }
}
