package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.IBackstageService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemInitService;
import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.license.ImppLicense;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.license.ImppLicenseTool;
import cn.estsh.impp.framework.boot.license.serviceimpl.ImppLicenseDecoder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;

/**
 * @Description : 后台服务
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-06-19 下午2:32
 * @Modify:
 **/
@RestController
@Api(tags = "后台运维服务")
@RequestMapping("/impp/white/backstage")
public class BackstageController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(BackstageController.class);

    @Autowired
    private ISystemInitService systemInitService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private IBackstageService backstageService;

    @GetMapping(value = "/redis-put-all")
    @ApiOperation(value = "重新加载所有缓存数据",notes = "重新加载所有缓存数据")
    public ResultBean redisPutAll(){
        try {
            systemInitService.putAndLoadAll();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-pojo-version-plan")
    @ApiOperation(value = "重新加载所有缓存数据",notes = "重新加载所有缓存数据")
    public ResultBean redisPutPojoVersionPlan(){
        try {
            systemInitService.reloadPojoVersionPlan();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-language")
    @ApiOperation(value = "重新加载语言数据",notes = "重新加载语言数据")
    public ResultBean redisPutSystemLanguage(){
        try {
            systemInitService.putAndLoadSysLocaleLanguage();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-config")
    @ApiOperation(value = "重新加载系统配置数据",notes = "重新加载系统配置数据")
    public ResultBean redisPutSystemConfig(){
        try {
            systemInitService.putAndLoadSysConfig();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/redis-put-system-dictionary")
    @ApiOperation(value = "重新加载系统字典数据",notes = "重新加载系统字典数据")
    public ResultBean redisPutSystemDictionary(){
        try {
            systemInitService.putAndLoadSysDictionary();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/reload-license")
    @ApiOperation(value = "重新加载授权数据",notes = "重新加载授权数据")
    public ResultBean reloadLicense(){
        try {
            ImppLicenseTool.cloneLicense();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(ImppLicenseTool.getLicense());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/reset/task-log-level")
    @ApiOperation(value = "重置定时任务日志等级",notes = "重置定时任务日志等级")
    public ResultBean restTaskLogLevel(){
        try {
            systemInitService.reloadTaskPlanLogLevel();
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            return ResultBean.fail(busExcep);
        }catch(Exception e){
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/license")
    @ApiOperation(value = "更新授权信息", notes = "更新授权信息")
    public ResultBean updateLicense(@RequestBody String content){
        try {
            ImppLicense license = ImppLicenseDecoder.getLicenseDecoder(content);
            if (Objects.nonNull(license)) {
                ImppLicenseTool.updateImppLicense(license);
            } else {
                return ResultBean.fail("错误的授权信息");
            }
            return ResultBean.success("操作成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/license")
    @ApiOperation(value = "获取授权信息", notes = "获取授权信息")
    public ResultBean getLicense(){
        try {
            ImppLicense license = ImppLicenseTool.getLicense();
            //icense.getMaturityDateTime() 可能从lockEndTime和licenseEndDateTime字段获取值
            license.setMaturityDateTime(license.getMaturityDateTime());
            ResultBean result = ResultBean.success("查询成功").setResultObject(license);
            return result;
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }



    @GetMapping(value = "/update-clean-menu-soft-type")
    @ApiOperation(value = "更新并清理功能软件模块数据", notes = "更新并清理功能软件模块数据")
    public ResultBean updateCleanMenuSoftType(){
        try {
            backstageService.updateCleanMenuSoftType();
            return ResultBean.success("操作成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/update-clean-ref-role-menu")
    @ApiOperation(value = "更新并清理角色功能数据", notes = "更新并清理角色功能数据")
    public ResultBean updateRefRoleMenu(){
        try {
            backstageService.updateRefRoleMenu();
            return ResultBean.success("操作成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

}
