package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.IDataSeparatorService;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorRule;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2021/12/3 16:38
 * @Modify:
 **/
@Api(tags = "通用冷热数据分离管理")
@RestController
@RequestMapping(PlatformConstWords.BASE_URL + "/separator")
public class DataSeparatorController {

    @Autowired
    private IDataSeparatorService dataSeparatorService;

    @GetMapping("/query")
    @ApiOperation(notes = "根据条件查询数据分离规则", value = "根据条件查询数据分离规则")
    public ResultBean query(DataSeparatorRule dataSeparatorRule, Pager pager) {
        try {
            ListPager<DataSeparatorRule> result = dataSeparatorService.query(dataSeparatorRule, pager);
            return ResultBean.success("false").setListPager(result).setMsg("查询成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/add")
    @ApiOperation(notes = "添加数据分离规则", value = "添加数据分离规则")
    public ResultBean add(@RequestBody DataSeparatorRule dataSeparatorRule) {
        try {
            dataSeparatorService.addRule(dataSeparatorRule);
            return ResultBean.success("添加成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }

    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(notes = "逻辑删除数据分离规则", value = "逻辑删除数据分离规则")
    public ResultBean delete(@PathVariable Long id) {
        try {
            dataSeparatorService.deleteRule(id);
            return ResultBean.success("删除成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping("/update")
    @ApiOperation(notes = "更新数据分离规则", value = "更新数据分离规则")
    public ResultBean update(@RequestBody DataSeparatorRule dataSeparatorRule) {
        try {
            dataSeparatorService.update(dataSeparatorRule);
            return ResultBean.success("修改成功");
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
