package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ICoreDataSourceService;
import cn.estsh.i3plus.core.apiservice.configuration.CoreJdbcTemplateConfig;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.WmsEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.CoreDataSource;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2022/1/7 10:37
 * @Modify:
 **/

@Api(value = "冷热数据分离数据源管理", tags = "冷热数据分离数据源管理")
@RestController
@RequestMapping(PlatformConstWords.BASE_URL + "/datasource")
public class DataSourceController {


    @Autowired
    private ICoreDataSourceService datasourceService;

    @Autowired
    private CoreJdbcTemplateConfig jdbcTemplateConfig;

    /**
     * 添加数据源
     *
     * @return
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加数据源", notes = "添加数据源")
    public ResultBean addDataSource(@RequestBody CoreDataSource coreDatasource) {
        try {
            ValidatorBean.beginValid(coreDatasource)
                    .numberCheck("sourcePort", coreDatasource.getSourcePort())
                    .notNull("sourceCode", coreDatasource.getSourceCode())
                    .notNull("sourceName", coreDatasource.getSourceName())
                    .notNull("sourceHost", coreDatasource.getSourceHost())
                    .notNull("sourceUserName", coreDatasource.getSourceUserName())
                    .notNull("sourcePassword", coreDatasource.getSourcePassword())
                    .notNull("sourceDataBaseName", coreDatasource.getSourceDataBaseName());
            ConvertBean.modelInitialize(coreDatasource, AuthUtil.getSessionUser());
            datasourceService.checkDataSourceOnly(coreDatasource);
            coreDatasource = datasourceService.saveDataSource(coreDatasource);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(coreDatasource);
        } catch (ImppBusiException exception) {
            return ResultBean.fail(exception).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除数据源
     *
     * @return
     */
    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "删除数据源", notes = "删除数据源")
    public ResultBean deleteDataSource(@PathVariable("id") Long id) {
        try {
            ValidatorBean.checkNotNull(id, "数据对象id不能为空");
            datasourceService.delete(id);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException exception) {
            return ResultBean.fail(exception).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 更新数据源
     *
     * @return
     */
    @PutMapping("/update")
    @ApiOperation(value = "修改数据源", notes = "修改数据源")
    public ResultBean updateDataSource(@RequestBody CoreDataSource coreDataSource) {
        try {

            ValidatorBean.beginValid(coreDataSource)
                    .notNull("id", coreDataSource.getId())
                    .notNull("sourceName", coreDataSource.getSourceName());

            ConvertBean.modelUpdate(coreDataSource, AuthUtil.getSessionUser());
            datasourceService.updateDataObject(coreDataSource);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询数据源
     *
     * @return
     */
    @PostMapping("/query")
    @ApiOperation(value = "查询数据源", notes = "查询数据源")
    public ResultBean list(CoreDataSource cusDatasource, Pager pager) {
        try {
            ListPager<CoreDataSource> listPager = datasourceService.queryDataObjectByPager(cusDatasource, pager);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setListPager(listPager);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PutMapping("/connection/{id}")
    @ApiOperation(value = "数据源链接", notes = "数据源连接测试")
    public ResultBean connectionDataSource(@PathVariable("id") Long id) {
        try {
            ValidatorBean.checkNotNull(id, "数据对象id不能为空");
            CoreDataSource datasource = datasourceService.getCoreDatasourceObject(id);
            ValidatorBean.checkNotNull(datasource, "不存在的数据源信息");
            boolean isConn = jdbcTemplateConfig.checkConnection(datasource);
            if (isConn) {
                if (WmsEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue() != datasource.getSourceStatus()) {
                    datasource.setSourceStatus(WmsEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue());
                    datasourceService.updateDataObject(datasource);
                }
                return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setMsg("连接成功");
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.FAIL.getCode()).setMsg("连接失败");
        } catch (ImppBusiException busiException) {
            return ResultBean.fail(busiException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/list")
    @ApiOperation(value = "获取所有数据源的code和id", notes = "获取所有数据源的code和id")
    public ResultBean listAll() {
        try {
            List<CoreDataSource> list = datasourceService.list();
            return ResultBean.success().setResultList(list);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }

    }

    @GetMapping("/data-source-type")
    @ApiOperation(value = "数据源类型", notes = "数据源类型")
    public ResultBean getDataSourceType() {
        return new ResultBean(true, "操作成功",
                LocaleUtils.getEnumLocaleResValuesToList(CommonEnumUtil.DATA_SOURCE_TYPE.values()));
    }
}
