package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.busi.ISysRefUserExternalService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserExternal;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.sdk.webchat.work.aes.AesException;
import cn.estsh.i3plus.sdk.webchat.work.enumutil.WorkWeChatEnumUtil;
import cn.estsh.i3plus.sdk.webchat.work.pojo.MsgTaskCardEvent;
import cn.estsh.i3plus.sdk.webchat.work.pojo.WorkWeChatUser;
import cn.estsh.i3plus.sdk.webchat.work.util.WorkWeChatTool;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.StringReader;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-09-02 19:08
 * @Modify:
 **/
@RestController
@Api(tags = "外部系统对接")
@RequestMapping("/impp/external-system")
public class ExternalSystemController {

    public static final Logger LOGGER = LoggerFactory.getLogger(ExternalSystemController.class);

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRefUserExternalService sysRefUserExternalService;

    /**
     * rabbitMQ队列
     */
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @GetMapping(value = "/enterprise-wechat/msg-callback")
    @ApiOperation(value = "消息回调测试", notes = "消息回调测试")
    public String test(@RequestParam(name = "msg_signature") String msgSignature, String timestamp,
                       String nonce, String echostr) throws AesException {
        return new WorkWeChatTool().msgCallbackVerify(msgSignature, timestamp, nonce, echostr);
    }

    @PostMapping(value = "/enterprise-wechat/msg-callback")
    @ApiOperation(value = "消息回调", notes = "消息回调")
    public ResultBean listSysLocaleLanguage(@RequestParam(name = "msg_signature") String msgSignature, String timestamp,
                                            String nonce, @RequestBody String msgStr) {
        try {
            WorkWeChatTool workWeChatTool = new WorkWeChatTool();
            MsgTaskCardEvent msgTaskCardEvent = workWeChatTool.msgCallbackDecrypt(msgSignature, timestamp, nonce, msgStr);
            rabbitTemplate.convertAndSend(PlatformConstWords.QUEUE_IMPP_WORK_WECHAT_MSG_CALLBACK, JSON.toJSONString(msgTaskCardEvent));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(msgTaskCardEvent);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/enterprise-wechat/bind-user-ref")
    @ApiOperation(value = "绑定用户关系", notes = "绑定用户关系")
    public ResultBean workWeChatUserBind() {
        try {
            WorkWeChatTool workWeChatTool = new WorkWeChatTool();

            // 获取所有微信用户，并按照配置的绑定数据转为map
            WorkWeChatEnumUtil.WECHAT_USER_BIND_PROPERTY weChatUserBindProperty = WorkWeChatEnumUtil.WECHAT_USER_BIND_PROPERTY
                    .valueOfEnum(RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_WORK_WECHAT_BIND_USER_PROPERTY));
            List<WorkWeChatUser> weChatUserList = workWeChatTool.getAllDeptUserListDetail();
            Map<String, WorkWeChatUser> weChatUserMap = weChatUserList.stream()
                    .collect(
                            Collectors.toMap(
                                    weChatUserBindProperty::getUserProperty,
                                    Function.identity(),
                                    (existing, replacement) -> existing)
                    );

            // 查询所有用户并根据配置属性进行匹配创建关系数据
            ImppEnumUtil.USER_BIND_PROPERTY userBindProperty = ImppEnumUtil.USER_BIND_PROPERTY
                    .valueOfEnum(
                            RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_WORK_WECHAT_BIND_IMPP_USER_PROPERTY)
                    );
            List<SysUser> sysUserList = sysUserService.list();
            for (SysUser sysUser : sysUserList) {
                String propertyValue = getUserProperty(userBindProperty, sysUser);
                if (propertyValue != null && weChatUserMap.containsKey(propertyValue)) {
                    String webChatUserId = weChatUserMap.get(propertyValue).getUserId();
                    SysRefUserExternal sysRefUserExternal =
                            sysRefUserExternalService.getSysRefUserExternalByRefAndUserId(
                                    ImppEnumUtil.USER_EXTERNAL_REF.WORK_WECHAT.getValue(), sysUser.getId()
                            );

                    // 判断是否存在如果存在则修改，不存在则新增
                    if (sysRefUserExternal == null) {
                        sysRefUserExternal = new SysRefUserExternal();
                        sysRefUserExternal.setUserId(sysUser.getId());
                        sysRefUserExternal.setExternalRef(ImppEnumUtil.USER_EXTERNAL_REF.WORK_WECHAT.getValue());
                        sysRefUserExternal.setRefValue(webChatUserId);
                        sysRefUserExternalService.insert(sysRefUserExternal);
                    } else if (!webChatUserId.equals(sysRefUserExternal.getRefValue())) {
                        sysRefUserExternal.setRefValue(webChatUserId);
                        sysRefUserExternalService.update(sysRefUserExternal);
                    }
                }
            }

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    private String getUserProperty(ImppEnumUtil.USER_BIND_PROPERTY userBindProperty, SysUser sysUser) {
        String userBindPropertyValue = null;
        switch (userBindProperty) {
            case USER_LOGIN_NAME:
                userBindPropertyValue = sysUser.getUserLoginName();
                break;
            case USER_NAME:
                userBindPropertyValue = sysUser.getUserName();
                break;
            case USER_EMP_NO:
                userBindPropertyValue = sysUser.getUserEmpNo();
                break;
            //FIXME 松下取消邮箱校验
//            case EMAIL:
//                userBindPropertyValue =sysUser.getUserEmail();
//                break;
            case USER_PHONE:
                userBindPropertyValue = sysUser.getUserPhone();
                break;
            case USER_WECHAT_NO:
                userBindPropertyValue = sysUser.getUserWeChatNo();
                break;
        }
        return userBindPropertyValue;
    }

    public static void main(String[] args) throws JAXBException {
        String xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
                "\n" +
                "<xml>\n" +
                "  <ToUserName><![CDATA[wwe40a593e30ed3196]]></ToUserName>\n" +
                "  <FromUserName><![CDATA[WangYunHao]]></FromUserName>\n" +
                "  <MsgType><![CDATA[event]]></MsgType>\n" +
                "  <Event><![CDATA[taskcard_click]]></Event>\n" +
                "  <CreateTime>1599057699</CreateTime>\n" +
                "  <EventKey><![CDATA[key111]]></EventKey>\n" +
                "  <TaskId><![CDATA[7]]></TaskId>\n" +
                "  <AgentId>1000002</AgentId>\n" +
                "</xml>\n";

        JAXBContext context = JAXBContext.newInstance(MsgTaskCardEvent.class);
        Unmarshaller unmarshaller = context.createUnmarshaller();
        MsgTaskCardEvent boy2 = (MsgTaskCardEvent) unmarshaller.unmarshal(new StringReader(xml));
        System.out.println(boy2);
        System.out.println(JSON.toJSONString(boy2));
    }
}
