package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserPasswordService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.AbstractEnvironment;
import org.springframework.core.env.Environment;
import org.springframework.core.env.MapPropertySource;
import org.springframework.core.env.PropertySource;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 2019-07-09 下午5:57
 * @Modify:
 **/
@RestController
@Api(tags = "系统服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/system")
public class SystemController {

    private static final Logger LOGGER = LoggerFactory.getLogger(SystemController.class);

    @Autowired
    private Environment environment;

    @Resource(name = CommonConstWords.IMPP_REDIS_CORE)
    protected ImppRedis redisCore;

    @Autowired
    private ISysUserPasswordService userPasswordService;

    @GetMapping("/get-properties")
    @ApiOperation(value = "获取系统配置信息", notes = "获取系统配置信息")
    public ResultBean getProperties(){
        try {
            //条件验证
            Map<String, Object> map = getStringObjectMap();

            return ResultBean.success("操作成功")
                    .setResultMap(map)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    private Map<String, Object> getStringObjectMap() {
        String[] profiles = environment.getDefaultProfiles();
        Map<String, Object> map = new TreeMap<>();
        for(Iterator it = ((AbstractEnvironment) environment).getPropertySources().iterator(); it.hasNext(); ) {
            PropertySource propertySource = (PropertySource) it.next();
            if (propertySource instanceof MapPropertySource) {
                MapPropertySource source = (MapPropertySource) propertySource;
                for (String key : source.getPropertyNames()) {
                    map.put(key,source.getProperty(key));
                }
            }
        }
        return map;
    }

    @GetMapping("/cloud-properties/{id}")
    @ApiOperation(value = "获取系统配置信息", notes = "获取系统配置信息")
    public ResultBean getProperties(@PathVariable("id") Integer id){
        try {

            ValidatorBean.checkIsNumber(id, "平台类型不能为空");
            CommonEnumUtil.SOFT_TYPE softType = CommonEnumUtil.SOFT_TYPE.valueOf(id);
            ValidatorBean.checkIsNumber(id, "错误平台信息");

            String redisKey = PlatformConstWords.REDIS_SYSTEM_PROPERTIES+ ":" + softType.getCode()+ "*";
            Set<String> redisKeys = redisCore.getKeysSet(redisKey);
            TreeMap<String,Map<String,Object>> result = new TreeMap<>();

            LOGGER.info("系统配置信息 redisKey中:{}",redisKey);
            Object redisValue = redisCore.getObject(redisKey);
            if(redisKeys != null){
                redisKeys.forEach(key ->{
                    String configInfo = key.substring(redisKey.length(), key.length()).replace("_",":");
                    LOGGER.info("系统配置信息 IP-port:{}",configInfo);
                    Map<Object,Object> redisCacheMap = redisCore.getHashMap(key);

                    if(redisCacheMap != null && redisCacheMap.size() > 0){
                        redisCacheMap.forEach((k,v) ->{
                            if(k != null && StringUtils.isNotBlank(k.toString())){
                                Map<String, Object> map = result.get(k);
                                if(map == null){
                                    map = new HashMap<>();
                                }

                                map.put(configInfo,v);
                                result.put(k.toString(),map);
                            }
                        });
                    }
                });
            }

            LOGGER.info("系统配置信息 result中值size:{}",result.size());

            return ResultBean.success("操作成功")
                    .setResultMap(result)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
