package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.platbean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-18 下午3:50
 * @Modify:
 **/
@RestController
@Api(tags = "系统日志查询服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/log")
public class ImppLogController extends CoreBaseController {

    @Autowired
    private ISysLogService logService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name= CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;


    /**
     * 系统日志分页复杂查询
     *
     * @param logSystem 查询条件
     * @param pager     分页数据
     * @return 处理结果
     */
    @GetMapping("/system/query")
    @ApiOperation(value = "系统日志分页复杂查询", notes = "系统日志分页复杂查询")
    public ResultBean querySysLogSystemByPager(SysLogSystem logSystem, Pager pager) {
        try {
            ListPager logSystemList = logService.queryLogSystemPager(logSystem, pager);
            return ResultBean.success("操作成功").setListPager(logSystemList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 定时任务日志分页复杂查询
     *
     * @param sysLogTaskTime 查询日志
     * @param pager          分页数据
     * @return 处理结果
     */
    @GetMapping("/task-time/query")
    @ApiOperation(value = "定时任务日志分页复杂查询", notes = "定时任务日志分页复杂查询")
    public ResultBean querySysLogSystemByPager(SysLogTaskTime sysLogTaskTime, Pager pager) {
        try {
            ListPager logSystemList = logService.queryLogTaskTimePager(sysLogTaskTime, pager);
            return ResultBean.success("操作成功").setListPager(logSystemList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 操作日志复杂查询，分页，排序
     *
     * @param logOperate 操作日志
     * @param pager      分页数据
     * @return 处理结果
     */
    @GetMapping(value = "/operate/query")
    @ApiOperation(value = "操作日志复杂查询，分页，排序", notes = "操作日志复杂查询，分页，排序")
    public ResultBean querySysLogOperateByPager(SysLogOperate logOperate, Pager pager) {
        try {
            ListPager logSystemList = logService.queryLogOperatePager(logOperate, pager);
            return ResultBean.success("操作成功").setListPager(logSystemList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 异常日志复杂查询，分页，排序
     *
     * @param sysLogException 查询条件
     * @param pager           分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/exception/query")
    @ApiOperation(value = "异常日志复杂查询，分页，排序", notes = "异常日志复杂查询，分页，排序")
    public ResultBean querySysLogExceptionByPager(SysLogException sysLogException, Pager pager) {
        try {
            ListPager logExceptionList = logService.queryLogExceptionPager(sysLogException, pager);
            return ResultBean.success("查询成功").setListPager(logExceptionList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 异常日志复杂查询，分页，排序
     *
     * @param bean  查询条件
     * @param pager 分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/console/query")
    @ApiOperation(value = "异常日志复杂查询，分页，排序", notes = "异常日志复杂查询，分页，排序")
    public ResultBean querySysLogConsoleByPager(SysLogConsole bean, Pager pager) {
        try {
            ListPager logExceptionList = logService.queryLogConsolePager(bean, pager);
            return ResultBean.success("查询成功").setListPager(logExceptionList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 定时任务控制台日志，分页，排序
     *
     * @param bean 查询条件
     * @return 处理结果
     */
    @GetMapping(value = "/console/query-schedule")
    @ApiOperation(value = "定时任务控制台日志，分页，排序", notes = "定时任务控制台日志")
    public ResultBean querySchdeuleLogConsoleByPager(SysLogConsole bean) {
        try {
            ValidatorBean.beginValid(bean)
                    .notNull("logGroup", bean.getLogGroup());
            // 参数
            // 1. group = CommonConstWords.SCHEDULE_LOG:Gourp.Name
            // 2.

            bean.setLogGroup(CommonConstWords.SCHEDULE_LOG + ":" + bean.getLogGroup());

            // 开始时间默认为上一分钟
            if (StringUtils.isBlank(bean.getCreateDateTimeStart())) {
                Date lastMinute = TimeTool.timeCalc(new Date(), Calendar.MINUTE, -1);
                bean.setCreateDateTimeStart(TimeTool.pareDateToString(lastMinute));
            }

            // 结束时间默认为当前时间
            if (StringUtils.isBlank(bean.getCreateDateTimeEnd())) {
                bean.setCreateDateTimeEnd(TimeTool.getNowTime(true));
            }
            // 一次100条
            bean.setOrderByParam("createDatetime");
            bean.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.ASC.getValue());

            List logScheduleList = new ArrayList();

            Pager pager = new Pager();
            pager.setPageSize(100);
            pager.setCurrentPage(1);

            ListPager listPager;
            do {
                if (pager.getTotalRows() != 0) {
                    pager.next();
                }
                listPager = logService.queryLogConsolePager(bean, pager);
                if(listPager!= null && listPager.getObjectList()!= null) {
                    logScheduleList.addAll(listPager.getObjectList());
                    pager = listPager.getObjectPager();
                }else if(pager.hasNext()) {
                    pager.setCurrentPage(pager.getTotalPages());
                }
            } while (pager.hasNext());

            return ResultBean.success("查询成功").setResultList(logScheduleList).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/clean-log")
    @ApiOperation(value = "添加操作日志", notes = "添加操作日志，可用于国际化")
    public ResultBean logClean(String time) {
        try {
            int to = RedisCacheTool.getSysConfigIntVal(CommonConstWords.LOG_STORAGE_DAYS, CommonConstWords.LOG_STORAGE_DAYS_DEFAULT);
            Date thisTime = TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -2 - to);
            String startTime = TimeTool.getToday(thisTime) + " 00:00:00";
            String endTime = TimeTool.getToday(TimeTool.timeCalc(thisTime, Calendar.DAY_OF_MONTH, to)) + " 23:59:59";
            logService.actionCleanLog(startTime,endTime);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep).build();
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 操作日志导出
     * @param logOperate 查询条件
     * @param response
     * @throws Exception 异常
     */
    @GetMapping(value = "/operate/export")
    @ApiOperation(value = "操作日志导出", notes = "操作日志导出")
    public void exportSysLogUserLogin(SysLogOperate logOperate, HttpServletResponse response) throws Exception {
        File file = null;
        try {
            Pager pager = Pager.defaultPager();
            pager.setCurrentPage(0);
            pager.setTotalPages(1);
            List<SysLogOperate> list = new ArrayList();
            do {
                pager.next();
                ListPager listPager = logService.queryLogOperatePager(logOperate, pager);
                if (listPager.getObjectPager().getTotalRows() >= 10000) {
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                            .setErrorDetail("数据过大，导出数据建议在10000以内")
                            .build();
                }

                list.addAll(listPager.getObjectList());
                pager= listPager.getObjectPager();
            } while (pager.hasNext());

            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            Class logOperateClass =SysLogOperate.class;
            String fileName = "log-operate-" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, list, logOperateClass, (LinkedHashMap<String, String>) ExcelTool.getColName(logOperateClass));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName,true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(excle)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } finally {
            FileUtils.deleteQuietly(file);
        }
    }

}
