package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPojoVersionService;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.common.CloudPagerModel;
import cn.estsh.i3plus.pojo.platform.bean.SysPojoVersion;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-4-21 下午2:56
 * @Modify:
 **/
@RestController
@Api(tags = "对象历史记录")
@RequestMapping(CommonConstWords.BASE_URL_CORE + "/pojo-version")
public class SysPojoVersionController extends CrudBaseController<SysPojoVersion> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SysPojoVersionController.class);

    @Autowired
    private ISysPojoVersionService pojoVersionService;

    @Autowired
    private EntityManager entityManager;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Override
    public ICrudService getCrudService() {
        return pojoVersionService;
    }

    @PostMapping("/cloud-query")
    @ApiOperation(value = "查询数据", notes = "复杂查询，分页，排序")
    BaseResultBean<SysPojoVersion> queryPojoVersionByPagerCloud(@RequestBody CloudPagerModel<SysPojoVersion> model) {
        try {
            SysPojoVersion bean = packQueryPager(model.getObj());

            return ResultBean.success("操作成功")
                    .setListPager(getCrudService().findPager(bean, model.getPager()))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/export-user-operation")
    @ApiOperation(value = "导出用户操作数据", notes = "导出用户操作数据")
    BaseResultBean<SysPojoVersion> exportUserOperation(HttpServletResponse response, String userName, String startDate, String endDate) {
        File file =null;
        try {
            final String ZERO_TIME = " 00:00:00";
            startDate += ZERO_TIME;
            endDate += ZERO_TIME;

            List<SysPojoVersion> pojoVersions = pojoVersionService.findPojoVersionByUserBetweenTime(userName, startDate, endDate);
            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "user-operation-" + System.currentTimeMillis() + ".xls";

            file = new File(fileName);
            file.createNewFile();
            excelTool.exportData(file, pojoVersions, SysPojoVersion.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(SysPojoVersion.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(file)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("数据变更日志导出异常", e);
            }
            return ResultBean.success().setResultList(pojoVersions);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }finally {
            if(file!=null){
                FileUtils.deleteQuietly(file);
            }
        }
    }
}
