package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreMemTreeService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysPositionService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysPosition;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.base.service.ICrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 岗位信息
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-19 17:05
 * @Modify :
 **/
@RestController
@Api(tags = "岗位管理服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/position")
public class SysPositionController extends CrudBaseController<SysPosition> {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysPositionController.class);

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private ICoreMemTreeService coreMemTreeService;

    @Override
    public ICrudService getCrudService() {
        return positionService;
    }

    @Override
    public SysPosition validatorInsertBean(SysPosition bean) throws Exception {
        //条件验证
        ValidatorBean.beginValid(bean)
                .notNull("name", bean.getName())
                .notNull("positionCode", bean.getPositionCode());

        bean.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
        if (bean.getParentId() == null || bean.getParentId() <= 0) {
            bean.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            bean.setParentNameRdd("顶级岗位");
        }
        return bean;
    }

    @Override
    public SysPosition validatorUpdateBean(SysPosition bean) throws Exception {
        //条件验证
        ValidatorBean.beginValid(bean)
                .notNull("id", bean.getId())
                .notNull("name", bean.getName())
                .notNull("positionCode", bean.getPositionCode());
        return bean;
    }


    /**
     * 查询所有岗位
     * @return 处理结果
     */
    @GetMapping(value = "/list-user")
    @ApiOperation(value = "查询所有岗位", notes = "查询所有岗位")
    public ResultBean findSysPositionListUser(SysPosition position) {
        try {
            List<SysPosition> result = null;
            if(AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.USER)){
                position.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }
            result = positionService.findAllByBean(position);

            return ResultBean.success("操作成功").setResultList(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }


    @Override
    @ApiOperation(value = "查询岗位", notes = "复杂查询岗位带分页")
    public ResultBean queryPager(SysPosition bean, Pager pager) {
        try {
            ListPager<SysPosition> listPager = null;

            List<Long> idList = new ArrayList<>();
            if(bean.getParentId() != null){
                idList = coreMemTreeService.findSysPositionChildIdList(bean.getParentId());
            }

            if(!AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)){
                bean.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }

            listPager = positionService.querySysPosition(bean,idList, pager);
            return ResultBean.success("操作成功").setListPager(listPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }


    @ApiOperation(value = "微服务查询岗位", notes = "微服务根据岗位代码等值查询岗位")
    @GetMapping("/cloud-query/{positionCode}")
    public ResultBean<SysPosition> findPositionByCode(@PathVariable String positionCode) {
        String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
        try {
            SysPosition sysPosition = positionService.findPositionByPositionCodeAndOrganizeId(positionCode, organizeCode);
            return ResultBean.success("操作成功").setResultObject(sysPosition).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ResultBean.fail(e).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @Override
    @ApiOperation(value = "微服务添加新的职位", notes = "微服务添加新职位")
    @PostMapping("/cloud-insert")
    public ResultBean insertByCloud(@RequestBody SysPosition bean) {
        return this.insert(bean);
    }
}
