package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsSendRecordService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsSignService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsTemplateService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.model.common.ImppSmsContent;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsSendRecord;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsSign;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsTemplate;
import cn.estsh.i3plus.sdk.sms.base.SmsUtilFactory;
import cn.estsh.i3plus.sdk.sms.base.pojo.SmsContent;
import cn.estsh.i3plus.sdk.sms.base.pojo.SmsResult;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.CONFIG_SMS_SWITCH;

/**
 * @Description : 短信发送记录
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-12-14 20:25
 * @Modify:
 **/
@RestController
@Api(tags = "短信发送记录")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-sms-send-record")
public class SysSmsSendRecordController extends CrudBaseController<SysSmsSendRecord> {

    private static final Logger LOGGER = LoggerFactory.getLogger(SysSmsSendRecordController.class);

    @Autowired
    private ISysSmsSignService sysSmsSignService;

    @Autowired
    private ISysSmsSendRecordService sysSmsSendRecordService;

    @Autowired
    private ISysSmsTemplateService sysSmsTemplateService;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis resRedis;

    @PostMapping("/send")
    @ApiOperation("发送短信")
    public ResultBean send(@RequestBody ImppSmsContent imppSmsContent) {
        try {
            if (!RedisCacheTool.getSysConfigBooleanVal(CONFIG_SMS_SWITCH)) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                        .setErrorDetail("短信开关未启用")
                        .setErrorSolution("在系统配置中启用短信通知")
                        .build();
            }
            ValidatorBean.beginValid(imppSmsContent)
                    .notNull("receivePhoneNumber",imppSmsContent.getReceivePhoneNumber())
                    .notNull("smsSignCode",imppSmsContent.getSmsSignCode())
                    .notNull("templateCode",imppSmsContent.getTemplateCode());

            SysSmsSign sysSmsSign = sysSmsSignService.getSysSmsSignByCode(imppSmsContent.getSmsSignCode());
            ValidatorBean.checkNotNull(sysSmsSign, "短信签名不存在！");
            SysSmsTemplate smsTemplate = sysSmsTemplateService.getSysSmsTemplateByCode(imppSmsContent.getTemplateCode());
            ValidatorBean.checkNotNull(smsTemplate, "短信模板不存在！");

            // 发送短信内容
            SmsContent smsContent = new SmsContent();
            smsContent.setSmsPhoneNumber(imppSmsContent.getReceivePhoneNumber());
            smsContent.setSmsTemplateCode(smsTemplate.getSupplierTemplateCode());
            smsContent.setSmsSignName(sysSmsSign.getSignName());
            smsContent.setSmsParam(imppSmsContent.getParamMap());

            SmsResult<SmsContent> smsResult = SmsUtilFactory.getSmsUtil().sendSms(smsContent);

            // 持久化发送记录
            SysSmsSendRecord sysSmsSendRecord = new SysSmsSendRecord();
            sysSmsSendRecord.setReceivePhoneNumber(imppSmsContent.getReceivePhoneNumber());
            sysSmsSendRecord.setTemplateCode(smsTemplate.getTemplateCode());
            sysSmsSendRecord.setSmsSignCode(sysSmsSign.getSignCode());
            sysSmsSendRecord.setSmsSign(sysSmsSign.getSignName());
            sysSmsSendRecord.setSmsParam(JsonUtilTool.encode(imppSmsContent.getParamMap()));
            sysSmsSendRecord.setSmsSendStatus(ImppEnumUtil.SMS_SEND_STATUS.WAIT_RECEIPT.getValue());
            sysSmsSendRecord.setSendDateTime(TimeTool.getNowTime(true));
            sysSmsSendRecord.setBizId(smsResult.getResultData().getBizId());

            sysSmsSendRecordService.insert(sysSmsSendRecord);

            if (!smsResult.isSuccess()) {
                sysSmsSendRecord.setSendErrMsg(smsResult.getErrMsg());
                sysSmsSendRecord.setSmsSendStatus(ImppEnumUtil.SMS_SEND_STATUS.WAIT_RECEIPT.getValue());
                sysSmsSendRecordService.update(sysSmsSendRecord);
                return ResultBean.fail().setErrorMsg(smsResult.getMsg()).setResultObject(smsResult);
            }

            // 推送redis延时队列，处理持久化发送记录
            resRedis.pushDelayQueue("SMS_STATUS_SYNC", JsonUtilTool.encode(sysSmsSendRecord),10L);
            return ResultBean.success().setResultObject(smsResult);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 同步
     */
    public static class SyncSmsInfo implements Runnable {

        private ISysSmsSendRecordService sysSmsSendRecordService;
        private ImppRedis redisRes;

        public SyncSmsInfo() {
        }

        public SyncSmsInfo(ISysSmsSendRecordService sysSmsSendRecordService, ImppRedis redisRes) {
            this.sysSmsSendRecordService = sysSmsSendRecordService;
            this.redisRes = redisRes;
        }

        @Override
        public void run() {
            while (true) {
                try {
                    Set<Object> dataSet =
                            redisRes.pullAndRemoveDelayQueueData("SMS_STATUS_SYNC", System.currentTimeMillis());
                    if (dataSet.isEmpty()) {
                        TimeUnit.MINUTES.sleep(3);
                        continue;
                    }

                    LOGGER.info("【SMS】 短信同步触发");
                    for (Object obj : dataSet) {
                        boolean isRetry = true;
                        SysSmsSendRecord smsSendRecord = null;
                        try {
                            smsSendRecord = JsonUtilTool.decode(String.valueOf(obj), SysSmsSendRecord.class);
                            SmsContent smsContent = new SmsContent();
                            smsContent.setSmsPhoneNumber(smsSendRecord.getReceivePhoneNumber());
                            smsContent.setSendDate(smsSendRecord.getSendDateTime());
                            smsContent.setBizId(smsSendRecord.getBizId());

                            // 查询并同步供应商的状态信息
                            SmsResult<SmsContent> smsResult =
                                    SmsUtilFactory.getSmsUtil().getSendSmsDetailByCondition(smsContent);
                            smsSendRecord.setSendErrMsg(smsResult.getErrMsg());
                            if (smsResult.isSuccess() && smsResult.getResultData() != null) {
                                smsContent = smsResult.getResultData();
                                smsSendRecord.setSmsSendStatus(smsContent.getSmsStatus());
                                smsSendRecord.setReceiveDateTime(smsContent.getReceiveDate());
                                smsSendRecord.setSmsContent(smsContent.getSmsContent());
                            }

                            if (smsSendRecord.getSmsSendStatus() != ImppEnumUtil.SMS_SEND_STATUS.WAIT_RECEIPT.getValue()) {
                                smsSendRecord.setSendErrMsg(StringUtils.EMPTY);
                                isRetry = false;
                            }
                        } catch (Exception e) {
                            LOGGER.error("【SMS】 短信状态同步异常", e);
                            if (smsSendRecord != null) {
                                smsSendRecord.setSendErrMsg(ExceptionUtils.getStackTrace(e));
                            }
                        }

                        if (smsSendRecord != null) {
                            sysSmsSendRecordService.update(smsSendRecord);
                        }

                        if (isRetry) {
                            redisRes.pushDelayQueue("SMS_STATUS_SYNC", JsonUtilTool.encode(smsSendRecord), 30L);
                        }
                    }
                } catch (Exception e) {
                    LOGGER.error("【SMS】 短信状态同步异常", e);
                } finally {
                    try {
                        TimeUnit.SECONDS.sleep(5);
                    } catch (InterruptedException ignored) {
                    }
                }
            }
        }
    }
}

