package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsSignService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsSign;
import cn.estsh.impp.framework.base.controller.CrudBaseController;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @Description : 短信签名
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-12-13 16:08
 * @Modify:
 **/
@RestController
@Api(tags = "短信签名")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-sms-sign")
public class SysSmsSignController extends CrudBaseController<SysSmsSign> {

    @Autowired
    private ISysSmsSignService sysSmsSignService;

    /**
     * 新增数据校验
     *
     * @param bean 待新增数据
     * @return 待新增数据
     */
    @Override
    public SysSmsSign validatorInsertBean(SysSmsSign bean) throws Exception {
        ValidatorBean.beginValid(bean)
                .notNull("signCode", bean.getSignCode())
                .notNull("signName", bean.getSignName());

        if (!sysSmsSignService.checkSysSignOnly(bean)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                    .setErrorDetail("签名代码已存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        return bean;
    }
}
