package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTableTemplateService;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysTableTemplate;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @author ns
 * @create 2022/3/17 0017 上午 11:39
 */
@RestController
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-table-template")
@Api(tags = "前端表格模板")
public class SysTableTemplateController {

    private ISysTableTemplateService iSysTableTemplateService;

    public SysTableTemplateController(ISysTableTemplateService sysTableTemplateService) {
        this.iSysTableTemplateService = sysTableTemplateService;
    }

    @ApiOperation("新增系统模板")
    @PostMapping("/insert")
    public ResultBean insertSysTableTemplate(@RequestBody SysTableTemplate sysTableTemplate) throws NoSuchFieldException {
        //条件验证
        ValidatorBean.beginValid(sysTableTemplate)
                .notNull("softType", sysTableTemplate.getSoftType())
                .notNull("templateData", sysTableTemplate.getTemplateData())
                .notNull("page", sysTableTemplate.getPage())
                .notNull("templateType", sysTableTemplate.getTemplateType());
        if (CommonEnumUtil.USER_TYPE.ADMIN.equals(AuthUtil.getUserType())) {
            if (iSysTableTemplateService.isExistSysTableTemplate(sysTableTemplate)) {
                return ResultBean.fail("同一个软件类型和page下不能有相同的模板类型");
            }
            return ResultBean.success().setResultObject(iSysTableTemplateService.insertSysTableTemplate((sysTableTemplate)));
        }
        return ResultBean.fail("当前用户不是管理员，不能新增");

    }

    @ApiOperation("修改系统模板")
    @PutMapping("/update")
    public ResultBean updateSysTableTemplate(@RequestBody SysTableTemplate sysTableTemplate) throws NoSuchFieldException {
        //条件验证
        ValidatorBean.beginValid(sysTableTemplate)
                .notNull("id", sysTableTemplate.getId());
        if (CommonEnumUtil.USER_TYPE.ADMIN.equals(AuthUtil.getUserType())) {
            return ResultBean.success().setResultObject(iSysTableTemplateService.updateSysTableTemplate((sysTableTemplate)));
        }
        return ResultBean.fail("当前用户不是管理员，不能修改");
    }

    @ApiOperation("修改系统模板,如果不存在则新增模板")
    @PutMapping("/putIfAbsent")
    public ResultBean putIfSysTableTemplateAbsent(@RequestBody SysTableTemplate sysTableTemplate) throws NoSuchFieldException {
        //条件验证

        if (!CommonEnumUtil.USER_TYPE.ADMIN.equals(AuthUtil.getUserType())) {
            return ResultBean.fail("当前用户不是管理员，不能修改");
        }
        ValidatorBean validatorBean = ValidatorBean.beginValid(sysTableTemplate);
        if (ObjectUtils.isEmpty(sysTableTemplate.getId())) {
            validatorBean
                    .notNull("softType", sysTableTemplate.getSoftType())
                    .notNull("templateData", sysTableTemplate.getTemplateData())
                    .notNull("page", sysTableTemplate.getPage())
                    .notNull("templateType", sysTableTemplate.getTemplateType());
            if (iSysTableTemplateService.isExistSysTableTemplate(sysTableTemplate)) {
                return ResultBean.fail("同一个软件类型和page下不能有相同的模板类型");
            }
            return ResultBean.success().setResultObject(iSysTableTemplateService.insertSysTableTemplate((sysTableTemplate)));
        }
        if (!iSysTableTemplateService.isExistSysTableTemplateById(sysTableTemplate.getId())) {
            return ResultBean.fail("模板不存在");
        }
        return ResultBean.success().setResultObject(iSysTableTemplateService.updateSysTableTemplate((sysTableTemplate)));
    }


    @ApiOperation("条件查询表格模板")
    @GetMapping("/find")
    public ResultBean findSysTableTemplate( SysTableTemplate sysTableTemplate) {
        if (ObjectUtils.isEmpty(sysTableTemplate)) {
            return ResultBean.fail("查询条件不能为空");
        }
        if (ObjectUtils.isEmpty(sysTableTemplate.getId())
                && ObjectUtils.isEmpty(sysTableTemplate.getPage())
                && ObjectUtils.isEmpty(sysTableTemplate.getSoftType())
                && ObjectUtils.isEmpty(sysTableTemplate.getTemplateType())) {
            return ResultBean.fail("查询条件不能为空");
        }
        return ResultBean.success().setResultList(iSysTableTemplateService.findSysTableTemplate((sysTableTemplate)));

    }

    @ApiOperation("根据id查询表格模板")
    @GetMapping("/getSysTableTemplate")
    public ResultBean getSysTableTemplateById(Long id) throws NoSuchFieldException {
        if (ObjectUtils.isEmpty(id)) {
            return ResultBean.fail("id不能为空");
        }
        return ResultBean.success().setResultObject(iSysTableTemplateService.getSysTableTemplateById(id));

    }


    @ApiOperation("分页查询表格模板")
    @GetMapping("/findBypage")
    public ResultBean findSysTableTemplateByPage(@ApiIgnore SysTableTemplate sysTableTemplate, Pager pager) throws NoSuchFieldException {
        if (ObjectUtils.isEmpty(pager)) {
            pager = Pager.defaultPager();
        }
        return ResultBean.success().setListPager(iSysTableTemplateService.findSysTableTemplateByPage(sysTableTemplate, pager));

    }

    @ApiOperation("根据id删除模板")
    @DeleteMapping("/delete")
    public ResultBean findSysTableTemplate(Long[] ids) {
        if (ObjectUtils.isEmpty(ids)) {
            return ResultBean.fail("id为空");
        }
        return ResultBean.success().setResultObject(iSysTableTemplateService.deleteSysTableTemplateByIds(ids));

    }

}
