package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.*;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.ExcelTool;
import cn.estsh.i3plus.platform.common.tool.ReflexTool;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.FileContentTypeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.UserDetailModel;
import cn.estsh.i3plus.pojo.model.platform.UserExportModel;
import cn.estsh.i3plus.pojo.model.platform.UserInfoImportModel;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.LocaleUtils;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.redisson.api.RLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.core.ReflectUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.ReflectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.attribute.PosixFileAttributes;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description : 用户信息 服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-11-08 19:57
 * @Modify :
 **/
@RestController
@Api(tags = "系统用户服务")
@RequestMapping(PlatformConstWords.BASE_URL + "/sys-user-info")
public class SysUserInfoController extends CoreBaseController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserInfoController.class);

    @Autowired
    private ILicenseClickService licenseClickService;

    @Autowired
    private ISysUserInfoService sysUserInfoService;

    @Autowired
    private ISysDepartmentService departmentService;

    @Autowired
    private ISysPositionService positionService;

    @Autowired
    private EntityManager entityManager;

    @Autowired
    private ICoreMemTreeService memTreeService;

    @Autowired
    private IPersonnelService personnelService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private ISysOrganizeService sysOrganizeService;

    @Autowired
    private PersonnelController personnelController;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    /**
     * 添加用户信息
     *
     * @param userInfo 用户信息
     * @return 处理结果
     */
    @PostMapping(value = "/insert")
    @ApiOperation(value = "添加用户信息", notes = "返回内容添加用户信息")
    public ResultBean insertSysUserInfo(SysUserInfo userInfo) {
        try {
            startMultiService();
            licenseClickService.checkLicenseNumberUser();

            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            checkSysUserInfo(userInfo, false);

            ConvertBean.modelInitialize(userInfo, user);
            userInfo.setUserInfoStatus(CommonEnumUtil.USER_STATUS.ENABLE.getValue());
            userInfo.setUserLoginNum(0);

            sysUserInfoService.checkSysUserInfoOnly(userInfo);
            SysUserInfo info = sysUserInfoService.insertSysUserInfo(userInfo);
            sysUserInfoService.refreshRefUserOrganize(info.getId(), userInfo.getDepartment().getOrganizeId(), user.getUserName());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(info);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据用户信息id修改信息
     *
     * @param userInfo 用户信息
     * @return 处理结果
     */
    @PutMapping(value = "/update")
    @ApiOperation(value = "修改用户信息", notes = "根据用户信息id修改信息")
    public ResultBean updateSysUserInfoById(SysUserInfo userInfo) {
        try {
            startMultiService();

            //登陆用户
            SessionUser user = AuthUtil.getSessionUser();

            checkSysUserInfo(userInfo, true);

            //修改初始化
            ConvertBean.modelUpdate(userInfo, user);
            sysUserInfoService.checkSysUserInfoOnly(userInfo);

            sysUserInfoService.updateSysUserInfo(userInfo);

            sysUserInfoService.refreshRefUserOrganize(userInfo.getId(), userInfo.getDepartment().getOrganizeId(), user.getUserName());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 删除用户信息
     *
     * @param id 用户信息id
     * @return 处理结果
     */
    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除用户信息", notes = "根据用户信息id删除信息")
    public ResultBean deleteSysUserInfo(@PathVariable("id") String id) {
        try {
            //条件验证
            ValidatorBean.checkNotNull(id, "id为空");

            sysUserInfoService.deleteSysUserInfoById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询所有用户信息
     *
     * @return 处理结果
     */
    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有用户信息", notes = "查询所有用户信息")
    public ResultBean findSysUserInfoAll() {
        try {
            List<SysUserInfo> SysUserInfoList = sysUserInfoService.listSysUserInfo();
            return ResultBean.success("操作成功").setResultList(SysUserInfoList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询用户信息
     *
     * @param userInfo 查询条件
     * @param pager    分页信息
     * @return 处理结果
     */
    @GetMapping(value = "/query")
    @ApiOperation(value = "查询用户信息", notes = "复杂查询用户信息带分页")
    public ResultBean querySysUserInfo(SysUserInfo userInfo, Pager pager) {
        try {
            ListPager<SysUserInfo> result = sysUserInfoService.querySysUserInfo(userInfo, packUserInfo(userInfo), pager);
            return ResultBean.success("操作成功").setListPager(result).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id查询用户信息
     *
     * @param id 用户信息id
     * @return 处理结果
     */
    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "根据id查询用户信息", notes = "根据id用户信息")
    public ResultBean getSysUserInfoById(@PathVariable("id") String id) {
        try {
            ValidatorBean.checkIsNumber(id, "id 不能为空");

            SysUserInfo userInfo = sysUserInfoService.getSysUserInfoById(Long.parseLong(id));
            ValidatorBean.beginValid(userInfo).notNull("id", userInfo.getId());

            return ResultBean.success("操作成功").setResultObject(userInfo).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 根据id批量删除
     *
     * @param ids 用户id数组
     * @return 处理结果
     */
    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteSysUserInfoByIds(String[] ids) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids, true);
            if (ids.length == 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户信息。")
                        .build();
            }
            List<SysUserInfo> userInfoList = personnelService.findSysUserInfoByIds(StringTool.getArrayLong(ids));
            if (ObjectUtils.isEmpty(userInfoList)) {
                return ResultBean.fail("数据不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
            Long[] userinfoIds = new Long[userInfoList.size()],
                    userIds = new Long[userInfoList.size()];
            for (int i = 0; i < userInfoList.size(); i++) {
                userinfoIds[i] = userInfoList.get(i).getId();
                userIds[i] = userInfoList.get(i).getUserId();
            }
            personnelService.deleteSysUserInfo(userinfoIds, getSessionUser().getUserName());
            personnelService.deleteSysUser(userIds, getSessionUser().getUserName());
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param userId 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-department/{userId}")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean findRefDepartment(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefUserDepartment> list = sysUserInfoService.findRefUserDepartmentByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询角色的所有角色权限关系
     *
     * @param model 用户明细模型
     * @return 处理结果
     */
    @PostMapping(value = "/query-ref-department")
    @ApiOperation(value = "查询角色的所有角色权限关系", notes = "查询角色的所有角色权限关系")
    public ResultBean queryRefDepartment(UserDetailModel model) {
        try {
            List<Long> idList = new ArrayList<>();

            List<SysRefUserDepartment> refs = sysUserInfoService.querySysRefUserDepartment(model);
            if (refs != null && refs.size() > 0) {
                for (SysRefUserDepartment ref : refs) {
                    idList.add(ref.getUserId());
                }
            }

            List<SysUserInfo> result = sysUserInfoService.querySysUserInfoList(model, idList);

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(result);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 查询用户的所有角色权限关系
     *
     * @param userId 用户id
     * @return 处理结果
     */
    @GetMapping(value = "/find-ref-position/{userId}")
    @ApiOperation(value = "查询用户的所有角色权限关系", notes = "查询用户的所有角色权限关系")
    public ResultBean findRefPosition(@PathVariable("userId") String userId) {
        try {
            ValidatorBean.checkNotNull(userId, "用户 id 不能为空");
            List<SysRefUserPosition> list = sysUserInfoService.findSysRefUserPositionByUserId(Long.parseLong(userId));
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * 用户唯一信息校验
     *
     * @param userInfo 用户信息
     * @param isId     是否检查ID 存在
     */
    private void checkSysUserInfo(SysUserInfo userInfo, boolean isId) {

        try {
            //条件验证
            ValidatorBean.beginValid(userInfo)
                    .notNull("name", userInfo.getName())
                    .notNull("userEmpNo", userInfo.getUserEmpNo())
                    .checkNotZero("departmentId", userInfo.getDepartmentId())
//                    .checkNotZero("departmentIdList",userInfo.getDepartmentIdList())
                    .checkNotZero("positionId", userInfo.getPositionId());
            if (isId) {
                ValidatorBean.beginValid(userInfo)
                        .notNull("id", userInfo.getId());
            }

            SysDepartment department = departmentService.get(userInfo.getDepartmentId());
            if (department == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("主部门信息不存在。")
                        .build();
            }

            SysPosition position = positionService.get(userInfo.getPositionId());
            if (position == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("主岗位信息不存在。")
                        .build();
            }

            sysUserInfoService.checkSysUserInfoOnly(userInfo);

            userInfo.setDepartmentNameRdd(department.getName());
            userInfo.setPositionNameRdd(position.getName());
            userInfo.setDepartment(department);
            userInfo.setPosition(position);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
        } catch (Exception e) {
            LOGGER.error(LocaleUtils.getEnumLocaleResDesc(ImppExceptionEnum.SYSTEM_EXCEPTION,
                    ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription()) + "：{}", e.getMessage(), e);
        }
    }

    @GetMapping("/export")
    @ApiOperation(value = "导出用户信息", notes = "导出用户信息")
    public void exportSysUserInfo(SysUserInfo userInfo, Pager pager, HttpServletResponse response) throws Exception {
        File file = null;
        if (userInfo == null) {
            userInfo = new SysUserInfo();
        }
        try {
            ListPager<SysUserInfo> result = null;
            List<SysUserInfo> list = null;
            if (ObjectUtils.isEmpty(pager)|| pager.getPageSize() == 0) {
                list = sysUserInfoService.querySysUserInfoList(userInfo, packUserInfo(userInfo));
            } else {
                result = sysUserInfoService.querySysUserInfo(userInfo, packUserInfo(userInfo), pager);
                list = result.getObjectList();
            }

            Map<Long, SysUser> sysUserMap = sysUserService.list().stream()
                    .collect(Collectors.toMap(SysUser::getId, Function.identity()));
            List<UserExportModel> exportModelList = new ArrayList<>();


            UserExportModel userExportModel;
            List<SysRefUserRole> sysRefUserRoleList;
            StringBuilder roleName;
            for (SysUserInfo userInfoItem : list) {
                roleName = new StringBuilder();
                // 关联角色信息
                sysRefUserRoleList = sysUserService.findSysRefUserRoleByUserId(userInfoItem.getUserId());
                for (SysRefUserRole sysRefUserRole : sysRefUserRoleList) {
                    roleName.append(sysRefUserRole.getRoleNameRdd()).append(",");
                }
                if (roleName.length() != 0) {
                    roleName.setLength(roleName.length() - 1);
                }

                // 封装model
                userExportModel = new UserExportModel(sysUserMap.get(userInfoItem.getUserId()), userInfoItem);
                userExportModel.setRoleNameRdd(roleName.toString());

                exportModelList.add(userExportModel);
            }

            ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
            String fileName = "user-info-" + System.currentTimeMillis() + ".xls";
            file = new File(fileName);
            file.createNewFile();
            File excle = excelTool.exportData(file, exportModelList, UserExportModel.class
                    , (LinkedHashMap<String, String>) ExcelTool.getColName(UserExportModel.class));

            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType(StringTool.getStringFileSuffix(fileName, true)));

            // 设置文件名
            try (BufferedInputStream bis = new BufferedInputStream(new DataInputStream(new FileInputStream(excle)))) {
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("用户信息导出异常", e);
            }
        } finally {
            FileUtils.deleteQuietly(file);
        }
    }

    @GetMapping("/importTemplate")
    @ApiOperation(value = "下载导入模板", notes = "下载导入模板")
    public void getImportTemplate(HttpServletResponse response) {
        ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
        try {
            String filename = "userImportTemplate.xls";
//            SysPosition positionBean = new SysPosition();
//            positionBean.setOrganizeCode(AuthUtil.getOrganizeCode());
            List<SysPosition> positionList = positionService.findAllByBaseBean();
            Map<String, String> positionMap = positionList.stream().collect(Collectors.toMap(position -> position.getPositionCode(), position -> position.getName(), (oldKey, newkey) -> oldKey));

            Map<String, Map<String, String>> relatefildMap = new HashMap<>();
            relatefildMap.put("岗位", positionMap);
            byte[] templateBytes = excelTool.getImportTemplate(UserInfoImportModel.class, relatefildMap);
            response.setContentType("application/force-download"); // 设置强制下载不打开
            response.addHeader("Content-Disposition", "attachment;fileName=" + filename); // 设置文件名
            response.addHeader("Content-type", FileContentTypeTool.getContentType("xls"));
            // 设置文件名
            try {
                InputStream inputStream = new ByteArrayInputStream(templateBytes);
                BufferedInputStream bis = new BufferedInputStream(inputStream);
                OutputStream os = response.getOutputStream();
                byte[] buffer = new byte[1024];
                int i = bis.read(buffer);
                while (i != -1) {
                    os.write(buffer, 0, i);
                    i = bis.read(buffer);
                }
            } catch (Exception e) {
                LOGGER.error("用户信息导入模板下载异常", e);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    @PostMapping("/import")
    @ApiOperation(value = "导用入户信息", notes = "导入用户信息")
    public ResultBean importUserInfo(@RequestParam MultipartFile file) {
        String lockKey = "REDISLOCK:IMPP:IMPORT_USER:" + AuthUtil.getSessionUser().getUserId();
        RLock rLock = (RLock) redisRes.getLock(lockKey);
        try {
            if (rLock.tryLock(0, TimeUnit.MINUTES)) {
                ExcelTool excelTool = new ExcelTool(entityManager, redisRes);
                //1.读取数据
                List<UserInfoImportModel> importModelList = excelTool.importDataAndConver(file.getOriginalFilename(), file.getInputStream(), UserInfoImportModel.class, UserInfoImportModel.class.getSimpleName());
                List<ResultBean> faildList = new ArrayList<>();
                LOGGER.debug("导入用户信息: 共读取数据{}条", importModelList.size());
                if (ObjectUtils.isEmpty(importModelList)) {
                    return ResultBean.success();
                }
                //2.数据格式校验并去重
                List<UserInfoImportModel> userInfoList = validateAndDistinc(importModelList, faildList);

                //3.某些数据字段转换
//                SysPosition positionBean = new SysPosition();
//                String organizationCode = AuthUtil.getOrganizeCode();
//                positionBean.setOrganizeCode(organizationCode);
                List<SysPosition> positionList = positionService.findAllByBaseBean();

                Map<String, String> positionMap = positionList.stream().collect(Collectors.toMap(position -> position.getPositionCode(), position -> String.valueOf(position.getId()), (oldKey, newkey) -> oldKey));

//                SysDepartment departmentBean = new SysDepartment();
//                departmentBean.setOrganizeCode(organizationCode);
//                List<SysDepartment> departmentList = departmentService.findAllByBaseBean(departmentBean);
                List<SysDepartment> departmentList = departmentService.findAllByBaseBean();
                //部门树
                List<SysDepartment> departmentTree = new ArrayList<>();
                //部门id的映射
                Map<Long, SysDepartment> parentMap = departmentList.stream().collect(Collectors.toMap(el -> el.getId(), Function.identity()));
                //构造全部部门的部门树
                departmentList.forEach(el -> {
                    if (!ObjectUtils.isEmpty(el.getParentId())) {
                        if (CommonEnumUtil.PARENT.DEFAULT.getValue().longValue() == el.getParentId().longValue()) {
                            departmentTree.add(el);
                        } else {
                            SysDepartment parentSys = parentMap.get(el.getParentId());
                            if (!ObjectUtils.isEmpty(parentSys)) {
                                if (null == parentSys.getChildList()) {
                                    parentSys.setChildList(new ArrayList<>());
                                }
                                parentSys.getChildList().add(el);
                            }
                        }
                    }
                });
//            SysRole roleBean = new SysRole();
//            roleBean.setOrganizeCode(organizationCode);
                List<SysRole> roleList = sysRoleService.findAll();
                Map<String, String> roleMap = roleList.stream().collect(Collectors.toMap(sysRole -> sysRole.getName(), sysRole -> String.valueOf(sysRole.getId()), (oldKey, newkey) -> oldKey));

                List<ResultBean> successList = new ArrayList<>();
                //4.插入数据
                for (int i = 0; i < userInfoList.size(); i++) {
                    UserInfoImportModel el = userInfoList.get(i);
                    ResultBean<UserDetailModel> resultBean = converImportData(el, departmentTree, positionMap, roleMap);
                    if (ObjectUtils.isEmpty(resultBean) || !resultBean.success) {
                        faildList.add(resultBean);
                        continue;
                    }
                    resultBean = personnelController.insertUserDetailModel(resultBean.getResultObject());
                    if (ObjectUtils.isEmpty(resultBean) || !resultBean.success) {
                        faildList.add(resultBean);
                        continue;
                    }
                    successList.add(resultBean);
                }
                Map<String, Object> resultMap = new HashMap<>();
                resultMap.put("total", importModelList.size());
                resultMap.put("success", successList.size());
                resultMap.put("fail", faildList.size());
//            resultMap.put("successList", successList);
                resultMap.put("failList", faildList);
                return new ResultBean().setSuccess(true).setResultMap(resultMap).build();
            }
            return ResultBean.fail("当前账号有正在上传的任务进行,请等待上传完毕");

        } catch (InterruptedException e) {
            LOGGER.error("导入人员数据出错", e);
            return ResultBean.fail(e);
        } catch (ImppBusiException busExcep) {
            LOGGER.error("导入人员数据出错", busExcep);
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            LOGGER.error("导入人员数据出错", e);
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            if (rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }


//        try {
//            //1.读取数据
//            List<UserInfoImportModel> importModelList = excelTool.importDataAndConver(file.getOriginalFilename(), file.getInputStream(), UserInfoImportModel.class, UserInfoImportModel.class.getSimpleName());
//            List<ResultBean> faildList = new ArrayList<>();
//            LOGGER.debug("导入用户信息: 共读取数据{}条", importModelList.size());
//            if (ObjectUtils.isEmpty(importModelList)) {
//                return ResultBean.success();
//            }
//            //2.数据格式校验并去重
//            List<UserInfoImportModel> userInfoList = validateAndDistinc(importModelList, faildList);
//
//            //3.某些数据字段转换
//            SysPosition positionBean = new SysPosition();
//            String organizationCode = AuthUtil.getOrganizeCode();
//            positionBean.setOrganizeCode(organizationCode);
//            List<SysPosition> positionList = positionService.findAllByBaseBean(positionBean);
//
//            Map<String, String> positionMap = positionList.stream().collect(Collectors.toMap(position -> position.getPositionCode(), position -> String.valueOf(position.getId()), (oldKey, newkey) -> oldKey));
//
//            SysDepartment departmentBean = new SysDepartment();
//            departmentBean.setOrganizeCode(organizationCode);
//            List<SysDepartment> departmentList = departmentService.findAllByBaseBean(departmentBean);
//            //部门树
//            List<SysDepartment> departmentTree = new ArrayList<>();
//            //部门id的映射
//            Map<Long, SysDepartment> parentMap = departmentList.stream().collect(Collectors.toMap(el -> el.getId(), Function.identity()));
//            //构造全部部门的部门树
//            departmentList.forEach(el -> {
//                if (!ObjectUtils.isEmpty(el.getParentId())) {
//                    if (CommonEnumUtil.PARENT.DEFAULT.getValue().longValue() == el.getParentId().longValue()) {
//                        departmentTree.add(el);
//                    } else {
//                        SysDepartment parentSys = parentMap.get(el.getParentId());
//                        if (!ObjectUtils.isEmpty(parentSys)) {
//                            if (null == parentSys.getChildList()) {
//                                parentSys.setChildList(new ArrayList<>());
//                            }
//                            parentSys.getChildList().add(el);
//                        }
//                    }
//                }
//            });
////            SysRole roleBean = new SysRole();
////            roleBean.setOrganizeCode(organizationCode);
//            List<SysRole> roleList = sysRoleService.findAll();
//            Map<String, String> roleMap = roleList.stream().collect(Collectors.toMap(sysRole -> sysRole.getName(), sysRole -> String.valueOf(sysRole.getId()), (oldKey, newkey) -> oldKey));
//
//            List<ResultBean> successList = new ArrayList<>();
//            //4.插入数据
//            for (int i = 0; i < userInfoList.size(); i++) {
//                UserInfoImportModel el = userInfoList.get(i);
//                ResultBean<UserDetailModel> resultBean = converImportData(el, departmentTree, positionMap, roleMap);
//                if (ObjectUtils.isEmpty(resultBean) || !resultBean.success) {
//                    faildList.add(resultBean);
//                    continue;
//                }
//                resultBean = personnelController.insertUserDetailModel(resultBean.getResultObject());
//                if (ObjectUtils.isEmpty(resultBean) || !resultBean.success) {
//                    faildList.add(resultBean);
//                    continue;
//                }
//                successList.add(resultBean);
//            }
//            Map<String, Object> resultMap = new HashMap<>();
//            resultMap.put("total", importModelList.size());
//            resultMap.put("success", successList.size());
//            resultMap.put("fail", faildList.size());
////            resultMap.put("successList", successList);
//            resultMap.put("failList", faildList);
//            return new ResultBean().setSuccess(true).setResultMap(resultMap).build();
//        } catch (ImppBusiException busExcep) {
//            return ResultBean.fail(busExcep);
//        } catch (Exception e) {
//            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
//        }
    }

    @GetMapping("/find-position-ref-user")
    @ApiOperation(value = "根据职位名称查询用户信息", notes = "根据职位名称查询用户信息")
    public BaseResultBean<SysUserInfo> findUserByPositionName(String positionName) {
        try {
            List<SysUserInfo> resultList = sysUserInfoService.findSysRefUserPositionByPositionName(positionName);
            return new ResultBean().setSuccess(true).setResultList(resultList).build();
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    /**
     * @param userInfoList 导入的全部数据
     * @param faildList    数据重复的
     * @return 不重复的数据
     */
    private List<UserInfoImportModel> validateAndDistinc(List<UserInfoImportModel> userInfoList, List<ResultBean> faildList) {
        //登录名
        Set<String> loginNameSet = new HashSet<>();
        //工号
        Set<String> empNoSet = new HashSet<>();
        List<UserInfoImportModel> successList = new ArrayList<>();
        for (UserInfoImportModel el : userInfoList) {
            if (loginNameSet.contains(el.getUserLoginName())) {
                faildList.add(ResultBean.fail().setErrorMsg("登录名重复").setResultObject(el).build());
                continue;
            }
            if (empNoSet.contains(el.getInfoEmpNo())) {
                faildList.add(ResultBean.fail().setErrorMsg("工号重复").setResultObject(el).build());
                continue;
            }
            loginNameSet.add(el.getUserLoginName());
            empNoSet.add(el.getInfoEmpNo());
            successList.add(el);
        }
        return successList;
    }

    /**
     * 将导入的数据转换UserDetailModel
     *
     * @param userInfoImportModel 用户导入数据的model
     * @param root                部门树
     * @param positionMap         职位映射map
     * @param roleMap             角色映射map
     * @return
     */
    public ResultBean<UserDetailModel> converImportData(UserInfoImportModel userInfoImportModel, List<SysDepartment> root, Map<String, String> positionMap, Map<String, String> roleMap) {

        UserDetailModel userDetailModel = new UserDetailModel();
        userDetailModel.setInfoCardType(userInfoImportModel.getInfoCardType());
        userDetailModel.setInfoCardNumber(userInfoImportModel.getInfoCardNumber());
        userDetailModel.setInfoEmployeeType(userInfoImportModel.getInfoEmployeeType());
        userDetailModel.setInfoBornDate(userInfoImportModel.getInfoBornDate());
        userDetailModel.setInfoEmergencyPhone(userInfoImportModel.getInfoEmergencyPhone());
        userDetailModel.setInfoEmergencyContact(userInfoImportModel.getInfoEmergencyContact());
        userDetailModel.setInfoBirthplace(userInfoImportModel.getInfoBirthplace());
        userDetailModel.setInfoEmpNo(userInfoImportModel.getInfoEmpNo());
        userDetailModel.setInfoFamilyAddress(userInfoImportModel.getInfoFamilyAddress());
        userDetailModel.setInfoGrade(userInfoImportModel.getInfoGrade());
        userDetailModel.setInfoHobby(userInfoImportModel.getInfoHobby());
        userDetailModel.setInfoJoinDate(userInfoImportModel.getInfoJoinDate());
        userDetailModel.setInfoRace(userInfoImportModel.getInfoRace());
        userDetailModel.setInfoName(userInfoImportModel.getInfoName());
        userDetailModel.setInfoMarriage(userInfoImportModel.getInfoMarriage());
        userDetailModel.setInfoSchool(userInfoImportModel.getInfoSchool());
        userDetailModel.setInfoSex(userInfoImportModel.getInfoSex());
        userDetailModel.setInfoSchoolEducation(userInfoImportModel.getInfoSchoolEducation());

        userDetailModel.setUserEmail(userInfoImportModel.getUserEmail());
        userDetailModel.setUserLoginName(userInfoImportModel.getUserLoginName());
        userDetailModel.setUserLanguageCode(userInfoImportModel.getLanguageCode());
        userDetailModel.setUserPhone(userInfoImportModel.getUserPhone());

        //设置密码为默认值
        userDetailModel.setUserLoginPassword(CommonConstWords.CONFIG_PWD_DEFAULT_PASSWORD);
        userDetailModel.setPassword(CommonConstWords.CONFIG_PWD_DEFAULT_PASSWORD);
        SysOrganize sysOrganize = new SysOrganize();
        sysOrganize.setOrganizeCode(AuthUtil.getOrganizeCode());
//        SysOrganize result = sysOrganizeService.get(sysOrganize);
//        if (ObjectUtils.isEmpty(result)) {
//
//            return ResultBean.fail().setErrorMsg("组织不存在").setResultObject(userInfoImportModel).build();
//        }
        //转换部门id
        //todo 可能有多个组织,导入时需要用“ ， ”分割
//        String[] infoDeparments = userInfoImportModel.getUserDepartmentNameRdds().trim().split(",");

        try {
            if (!StringUtils.hasText(userInfoImportModel.getInfoDepartmentNameRdds())) {

                return ResultBean.fail().setErrorMsg("部门不能为空").setResultObject(userInfoImportModel).build();
            }
            String[] infoDeparments = userInfoImportModel.getInfoDepartmentNameRdds().trim().split("/");

            if (ObjectUtils.isEmpty(infoDeparments)) {

                return ResultBean.fail().setErrorMsg("部门不能为空").setResultObject(userInfoImportModel).build();
            }
            int mainDepartmentIndex = -1;
            for (int i = 0; i < infoDeparments.length; i++) {
                infoDeparments[i] = infoDeparments[i].trim();
                if (infoDeparments[i].equals(userInfoImportModel.getInfoDepartmentNameRdd())) {
                    mainDepartmentIndex = i;
                }
            }
            if (mainDepartmentIndex == -1) {

                return ResultBean.fail().setErrorMsg("主部门不在部门列表中").setResultObject(userInfoImportModel).build();
            }
            Long[] infoDeparmentIds = findDeparmentIds(root, infoDeparments);
            if (ObjectUtils.isEmpty(infoDeparmentIds) || infoDeparmentIds.length != infoDeparments.length) {
                return ResultBean.fail().setErrorMsg("部门格式错误").setResultObject(userInfoImportModel).build();

            }
            userDetailModel.setInfoDepartmentId(String.valueOf(infoDeparmentIds[mainDepartmentIndex]));
            userDetailModel.setInfoDepartmentIdList(Arrays.stream(infoDeparmentIds).map(el -> String.valueOf(el)).collect(Collectors.toList()));

            if (!StringUtils.hasText(userInfoImportModel.getUserDepartmentNameRdds())) {
                return ResultBean.fail().setErrorMsg("操作部门列表不能为空").setResultObject(userInfoImportModel).build();
            }
            //转换操作部门id
            String[] userDeparments = userInfoImportModel.getUserDepartmentNameRdds().trim().split("/");

            if (ObjectUtils.isEmpty(userDeparments)) {
                return ResultBean.fail().setErrorMsg("操作部门不能为空").setResultObject(userInfoImportModel).build();
            }
            //主部门索引
            mainDepartmentIndex = -1;

            for (int i = 0; i < userDeparments.length; i++) {
                userDeparments[i] = userDeparments[i].trim();
                if (userDeparments[i].equals(userInfoImportModel.getUserDepartmentNameRdd())) {
                    mainDepartmentIndex = i;
                }
            }
            if (mainDepartmentIndex == -1) {
                return ResultBean.fail().setErrorMsg("主部门不在部门列表中").setResultObject(userInfoImportModel).build();
            }
            Long[] userDeparmentIds = findDeparmentIds(root, userDeparments);
            if (ObjectUtils.isEmpty(userDeparmentIds) || userDeparmentIds.length != userDeparments.length) {
                return ResultBean.fail().setErrorMsg("部门格式错误").setResultObject(userInfoImportModel).build();
            }
            userDetailModel.setUserDepartmentId(String.valueOf(userDeparmentIds[mainDepartmentIndex]));
            userDetailModel.setUserDepartmentIdList(Arrays.stream(userDeparmentIds).map(el -> String.valueOf(el)).collect(Collectors.toList()));

            //转换角色id
            if (!StringUtils.hasText(userInfoImportModel.getRoleNameRdds())) {
                return ResultBean.fail().setErrorMsg("角色不能为空").setResultObject(userInfoImportModel).build();
            }
            String[] roles = userInfoImportModel.getRoleNameRdds().trim().split("/");
            if (!ObjectUtils.isEmpty(roles)) {
                userDetailModel.setUserRoleIdList(
                        Arrays.stream(roles).map(el -> roleMap.get(el.trim())).collect(Collectors.toList())
                );
            }
            //转换职位id
            if (!StringUtils.hasText(userInfoImportModel.getPositionNameRdds())) {
                return ResultBean.fail().setErrorMsg("职位不能为空").setResultObject(userInfoImportModel).build();
            }
            String[] positions = userInfoImportModel.getPositionNameRdds().trim().split("/");
            if (!ObjectUtils.isEmpty(positions)) {
                userDetailModel.setInfoPositionIdList(
                        Arrays.stream(positions).map(el -> positionMap.get(el.trim())).collect(Collectors.toList())
                );
            }
            if (!StringUtils.hasText(userInfoImportModel.getPositionNameRdd())) {
                return ResultBean.fail().setErrorMsg("主职位不能为空").setResultObject(userInfoImportModel).build();
            }
            userDetailModel.setInfoPositionId(positionMap.get(userInfoImportModel.getPositionNameRdd()));

            return ResultBean.success().setResultObject(userDetailModel).build();
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error("导入人员数据转换出错", e);
            return ResultBean.fail(e);
        }
    }

    /**
     * 根据部门树查询部门id,返回部门数组
     *
     * @param root            部门树
     * @param departmentNames 部门名称组成的数组
     * @return
     */
    private Long[] findDeparmentIds(List<SysDepartment> root, String[] departmentNames) {
        if (ObjectUtils.isEmpty(root) || ObjectUtils.isEmpty(departmentNames)) {
            return null;
        }
        Long[] deparmentIds = new Long[departmentNames.length];
        for (int i = 0; i < departmentNames.length; i++) {
            int finalI = i;
            SysDepartment sysDepartment = root.stream().filter(el -> departmentNames[finalI].equals(el.getName())).findFirst().orElse(null);
            if (ObjectUtils.isEmpty(sysDepartment)) {
                return deparmentIds;
            }
            deparmentIds[i] = sysDepartment.getId();
            root = sysDepartment.getChildList();
            if (ObjectUtils.isEmpty(root)) {
                return deparmentIds;
            }
        }
        return deparmentIds;
    }


    private ArrayList<Long> packUserInfo(SysUserInfo userInfo) {
        ArrayList<Long> idList = new ArrayList<>();
        if (userInfo.getOrganizeId() == null || userInfo.getOrganizeId() <= 0) {
            if (AuthUtil.getUserType().equals(CommonEnumUtil.USER_TYPE.ADMIN)) {
                userInfo.setOrganizeCode(null);
                userInfo.setOrganizeId(null);
            } else {
                userInfo.setOrganizeId(AuthUtil.getOrganize().getId());
                userInfo.setOrganizeCode(AuthUtil.getOrganize().getOrganizeCode());
            }
        }

        List<Long> departmentIdList = memTreeService.findDepartmentIdListByOrganizeIdAndDepartmentId(
                userInfo.getOrganizeId(), userInfo.getDepartmentId()
        );
        if (departmentIdList != null && departmentIdList.size() > 0) {
            List<SysRefUserInfoDepartment> refList = personnelService.findSysRefUserInfoDepartmentByDepartmentIdList(departmentIdList);
            if (refList != null && refList.size() > 0) {
                refList.forEach(ref -> idList.add(ref.getUserId()));
            }
        }

        if (userInfo.getPositionId() != null && userInfo.getPositionId() > 0) {
            List<Long> refList = memTreeService.findSysPositionChildIdList(userInfo.getPositionId());
            if (refList != null && refList.size() > 0) {
                List<SysRefUserPosition> refUserPositionList = personnelService.findSysRefUserPositionByIdList(refList);
                if (refUserPositionList != null && refUserPositionList.size() > 0) {
                    refUserPositionList.forEach(ref -> idList.add(ref.getUserId()));
                }
            }
        }
        return idList;
    }


}
