package cn.estsh.i3plus.core.apiservice.controller.cloud;

import cn.estsh.i3plus.core.apiservice.controller.base.BaseMessageController;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.MessagePagerModel;
import cn.estsh.i3plus.pojo.model.platform.RefUserMessagePagerModel;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @Description : SWEB消息管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 19:03
 * @Modify :
 **/
@RestController
@RequestMapping(PlatformConstWords.BASE_URL +"/sweb")
@Api(tags = "消息管理服务")
public class SwebMessageController extends CoreBaseController {

    @Autowired
    private BaseMessageController baseSysMessage;


    /*************************************************** SWEB系统消息 **************************************************/

    /**
     * SWEB - 新增消息
     * @param sysMessage 消息
     * @return 处理结果
     */
    @PostMapping(value = "/sys-message/insert")
    @ApiOperation(value = "新增消息",notes = "新增消息")
    public ResultBean<SysMessage> insertSysMessage(@RequestBody SysMessage sysMessage){
        sysMessage = sysMessage == null ? new SysMessage() : sysMessage;
        sysMessage.setMessageSoftType(CommonEnumUtil.SOFT_TYPE.SWEB.getValue());

        return baseSysMessage.basaeInsertSysMessage(sysMessage);
    }

    /**
     * SWEB - 删除消息
     * @param id 消息id
     * @return 处理结果
     */
    @DeleteMapping("/sys-message/delete/{id}")
    @ApiOperation(value = "根据id删除消息",notes = "根据id删除消息")
    public ResultBean deleteSysMessageById(@PathVariable("id") String id){
        return baseSysMessage.baseDeleteSysMessageById(id);
    }

    /**
     * SWEB - 修改消息
     * @param sysMessage 消息主体
     * @return 处理结果
     */
    @PutMapping("/sys-message/update")
    @ApiOperation(value = "修改信息",notes = "修改信息")
    public ResultBean updateSysMessage(SysMessage sysMessage){
        sysMessage = sysMessage == null ? new SysMessage() : sysMessage;
        sysMessage.setMessageSoftType(CommonEnumUtil.SOFT_TYPE.SWEB.getValue());
        return baseSysMessage.baseUpdateSysMessage(sysMessage);
    }

    /**
     * SWEB - 根据id查询消息
     * @param idStr 消息id
     * @return 处理结果
     */
    @GetMapping(value = "/sys-message/get/{id}")
    @ApiOperation(value = "根据id查询消息",notes = "根据id查询消息")
    public ResultBean<SysMessage> getSysMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseGetSysMessageById(idStr);
    }

    /**
     * SWEB - 分页带条件查询
     * @param messagePagerModel 分页以及条件信息
     * @return 处理结果
     */
    @PostMapping(value = "/sys-message/query")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysMessage> querySysMessageByPager(@RequestBody MessagePagerModel messagePagerModel){
        return baseSysMessage.baseQuerySysMessageByPager(messagePagerModel);
    }

    /**
     * SWEB - 分页带条件查询
     * @param messagePagerModel 分页以及条件信息
     * @return 处理结果
     */
    @PostMapping(value = "/sys-message/querySx")
    @ApiOperation(value = "松下用-消息复杂查询，分页，排序",notes = "松下用-消息复杂查询，分页，排序")
    public ResultBean<SysMessage> querySysMessageByPagerSx(@RequestBody MessagePagerModel messagePagerModel){
        return baseSysMessage.baseQuerySysMessageByPagerSx(messagePagerModel);
    }

    /**
     * SWEB - 批量删除
     * @param ids ids数组
     * @return 处理结果
     */
    @DeleteMapping("/sys-message/batch-delete")
    @ApiOperation(value = "批量删除消息",notes = "批量删除消息")
    public ResultBean deleteSysMessageByIds(String[] ids){
        return baseSysMessage.baseDeleteSysMessageByIds(ids);
    }

    /*********************************************** SWEB用户系统消息 *********************************************/

    /**
     * SWEB - 根据用户消息id查询消息
     * @param idStr
     * @return 处理结果
     */
    @GetMapping(value = "/user-message/get/{id}")
    @ApiOperation(value = "根据用户消息id查询消息",notes = "根据用户消息id查询消息")
    public ResultBean<SysMessage> getUserMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseGetUserMessageById(idStr);
    }

    /**
     * SWEB - 批量切换消息状态
     * @param ids ids数组
     * @param status 状态
     * @return 处理结果
     */
    @PutMapping("/user-message/batch-status")
    @ApiOperation(value = "批量切换消息状态",notes = "批量切换消息状态")
    public ResultBean updateSysMessageStatusByIds(String[] ids,Integer status){
        return baseSysMessage.baseUpdateSysMessageStatusByIds(ids, status);
    }

    /**
     * 用户消息复杂查询，分页，排序
     * @param refUserMessagePagerModel 查询条件及分页信息
     * @return 处理结果
     */
    @PostMapping(value = "/user-message/query")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysRefUserMessage> querySysRefUserMessageByPager(@RequestBody RefUserMessagePagerModel refUserMessagePagerModel){
        return baseSysMessage.baseQuerySysRefUserMessageByPager(refUserMessagePagerModel);
    }

    /**
     * 用户消息删除
     * @param idStr 用户消息id
     * @return 处理结果
     */
    @DeleteMapping(value = "/user-message/delete/{id}")
    @ApiOperation(value = "消息复杂查询，分页，排序",notes = "消息复杂查询，分页，排序")
    public ResultBean<SysRefUserMessage> deleteSysRefUserMessageById(@PathVariable("id") String idStr){
        return baseSysMessage.baseDeleteSysRefUserMessageById(idStr);
    }

}
