package cn.estsh.i3plus.core.apiservice.filter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @Description: 解决 点击劫持：X-Frame-Options未配置
 * @Param:
 * @return:
 * @Date:
 */
@Component
public class CookieFrameFilter implements Filter {
    public static final Logger LOGGER = LoggerFactory.getLogger(CookieFrameFilter.class);

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        LOGGER.info("CookieFrameFilter Init...");
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse resp = (HttpServletResponse) response;
        // 点击劫持问题
        resp.addHeader("X-Frame-Options", "SAMEORIGIN");
        resp.addHeader("Content-Security-Policy", "frame-ancestors 'self'");
        // 防止基于 MIME 类型混淆的攻击
        resp.addHeader("X-Content-Type-Options", "nosniff");
        // xss 保护
        resp.addHeader("X-XSS-Protection", "1; mode=block");
        chain.doFilter(req, resp);
    }

    @Override
    public void destroy() {
        LOGGER.info("CookieFrameFilter Destory.");
    }
}
