package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.apiservice.serviceimpl.base.DataSeparatorServiceImpl;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.jpa.daoimpl.BaseRepositoryImpl;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorMessage;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2021/9/23 13:19
 * @Modify:
 **/
@Configuration
@ConditionalOnExpression(" '${impp.aspect.data.separation:false}' == 'true' ")
public class DataSeparatorQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataSeparatorQueueReceiver.class);

    private static final Map<String, BaseRepositoryImpl> REPOSITORY_HASH_MAP = new HashMap<>();

    @Resource
    private DataSeparatorServiceImpl dataSeparatorService;

    /**
     * QUEUE_SWEB_NOTICE 队列
     *
     * @return 队列
     * @throws Exception
     */
    @Bean
    public Queue getDataSeparatorQueue() {
        return new Queue(PlatformConstWords.QUEUE_IMPP_DATA_SEPARATOR);
    }

    @RabbitListener(queues = PlatformConstWords.QUEUE_IMPP_DATA_SEPARATOR,concurrency = "5")
    public Boolean consumer(String data, Channel channel, Message message) {
        DataSeparatorMessage msg = JsonUtilTool.decode(data, DataSeparatorMessage.class);
//        String refClass = msg.getRefClass();
//        BaseRepository repository;
        //找到项目中bean的repository
//        if (REPOSITORY_HASH_MAP.containsKey(refClass)) {
//            repository = REPOSITORY_HASH_MAP.get(refClass);
//        } else {
//
//            Map<String, BaseRepository> beansRepositories = SpringContextsUtil.getBeansOfType(BaseRepository.class);
//            List<BaseRepository> collect = beansRepositories.values().stream().filter(item -> {
//                String clazz = null;
//                try {
//                    //第一层代理
//                    Field h = item.getClass().getSuperclass().getDeclaredField("h");
//                    h.setAccessible(true);
//                    AopProxy o = (AopProxy) h.get(item);
//                    Field advised = o.getClass().getDeclaredField("advised");
//                    advised.setAccessible(true);
//                    Object target = ((AdvisedSupport) advised.get(o)).getTargetSource().getTarget();
//
//                    //第二层代理
//                    Field h1 = target.getClass().getSuperclass().getDeclaredField("h");
//                    h1.setAccessible(true);
//                    AopProxy aopProxy = (AopProxy) h1.get(target);
//                    Field advised1 = aopProxy.getClass().getDeclaredField("advised");
//                    advised1.setAccessible(true);
//                    Object target1 = ((AdvisedSupport) advised1.get(aopProxy)).getTargetSource().getTarget();
//
//                    Field persistentClass = target1.getClass().getDeclaredField("persistentClass");
//                    persistentClass.setAccessible(true);
//                    clazz = persistentClass.get(target1).toString();
//                } catch (NoSuchFieldException | IllegalAccessException e) {
//                    return false;
//                } catch (Exception e) {
//                    LOGGER.error("通用数据分离,获取repository异常 :{}", e.getMessage());
//                }
//
//                return clazz.substring(clazz.lastIndexOf(".") + 1).equals(refClass.substring(refClass.lastIndexOf(".") + 1));
//
//            }).collect(Collectors.toList());
//
//            if (!collect.isEmpty()) {
//                repository = collect.get(0);
//            } else {
//                LOGGER.error("{}:找不到对应的RepositoryBean!", refClass);
//                return false;
//            }
//        }
//
//        try {
//            dataSeparatorService.doSeparate(repository, msg);
//        } catch (Exception e) {
//            LOGGER.error("通用冷热数据分离失败，失败原因:{}", e.getMessage());
//        } finally {
//            try {
//                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
//            } catch (IOException e) {
//                LOGGER.error("通用冷热数据分离失败，返回ACK失败:{}", e.getMessage());
//            }
//        }
        try {
            if (!Objects.isNull(msg)){
                dataSeparatorService.doSeparate(msg);
            }
        } catch (Exception e) {
            LOGGER.error("通用冷热数据分离失败，失败原因:{}", e.getMessage());
        } finally {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException e) {
                LOGGER.error("通用冷热数据分离失败，返回ACK失败:{}", e.getMessage());
            }
        }
        return true;
    }
}