package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysRefUserExternalService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserExternal;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.sdk.webchat.work.enumutil.WorkWeChatEnumUtil;
import cn.estsh.i3plus.sdk.webchat.work.pojo.MsgResponse;
import cn.estsh.i3plus.sdk.webchat.work.pojo.MsgTaskCardEvent;
import cn.estsh.i3plus.sdk.webchat.work.util.WorkWeChatTool;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import com.alibaba.fastjson.JSON;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 企业微信
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-09-03 11:15
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.work.we-chat:true}' == 'true'")
@Configuration
public class MessageWorkWeChatQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(MessageWorkWeChatQueueReceiver.class);

    @Autowired
    private ISysMessageService sysMessageService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRefUserExternalService sysRefUserExternalService;

    @Bean
    public Queue getQueueImppWorkWeChatMsg(){
        return new Queue(PlatformConstWords.QUEUE_IMPP_WORK_WECHAT_MSG);
    }

    @Bean
    public Queue getQueueImppWorkWeChatMsgCallback(){
        return new Queue(PlatformConstWords.QUEUE_IMPP_WORK_WECHAT_MSG_CALLBACK);
    }

    /**
     * 系统邮件处理队列
     *
     * @param data    系统消息信息
     * @param channel 通道
     * @param message 消息
     */
    @RabbitListener(queues = PlatformConstWords.QUEUE_IMPP_WORK_WECHAT_MSG)
    public Boolean processWorkWeChatMsg(String data, Channel channel, Message message) {
        SysMessage msg = JsonUtilTool.decode(data, SysMessage.class);
        Long startTime = System.currentTimeMillis();
        try {
            LOGGER.debug("【MQ-QUEUE_IMPP_WORK_WECHAT_MSG】数据接收成功：{}", msg);
            if (msg.getMessageSendTime() == null) {
                msg.setMessageSendTime(TimeTool.getNowTime(true));
            }

            // 持久化消息化消息并初始化状态
            ConvertBean.serviceModelInitialize(msg, msg.getMessageSenderNameRdd());
            msg.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SENDING.getValue());
            msg = sysMessageService.insertSysMessage(msg);

            // 默认发送成功
            msg.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SEND_SUCCESS.getValue());

            // 收件人信息
            List<Long> userIdList = new ArrayList<>();
            SysRefUserMessage refUserMessage;
            SysUser sysUser;
            Long userId;
            for (String userIdStr : msg.getMessageReceiversId().split(",")) {
                userId =Long.valueOf(userIdStr);
                sysUser = sysUserService.getSysUserById(userId);

                refUserMessage = new SysRefUserMessage();
                refUserMessage.setMessageId(msg.getId());
                refUserMessage.setMessageSoftType(msg.getMessageSoftType());
                refUserMessage.setMessageTitleRdd(msg.getMessageTitle());
                refUserMessage.setMessageTypeRdd(msg.getMessageType());
                refUserMessage.setMessageSenderNameRdd(msg.getMessageSenderNameRdd());
                refUserMessage.setReceiverId(sysUser.getId());
                refUserMessage.setReceiverNameRdd(sysUser.getUserName());
                refUserMessage.setMessageStatus(ImppEnumUtil.MESSAGE_STATUS.UNREAD.getValue());
                refUserMessage.setReceiverTime(TimeTool.getNowTime(true));
                refUserMessage.setIsUrgent(msg.getIsUrgent());

                sysMessageService.insertSysRefUserMessage(refUserMessage);

                userIdList.add(userId);
            }

            // 查询到关联的用户企业微信账号并发送
            List<String> weChatUserIdList = sysRefUserExternalService.findExternalValueByRefAndUserId(
                    ImppEnumUtil.USER_EXTERNAL_REF.WORK_WECHAT.getValue(), userIdList);

            WorkWeChatTool workWeChatTool = new WorkWeChatTool();
            MsgResponse msgResponse = null;
            try {
                msgResponse = workWeChatTool.sendTaskCardMsg(weChatUserIdList, msg.getMessageTitle(),
                        msg.getMessageContent(), msg.getId().toString());
                LOGGER.info("企业微信信息发送结果{}", JsonUtilTool.encode(msgResponse));
            } catch (Exception e) {
                msg.setSendStatus(ImppEnumUtil.MESSAGE_SEND_STATUS.SEND_FAIL.getValue());
                msg.setErrorMessage(e.getMessage());

            }

            sysMessageService.updateSysMessage(msg);

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            return true;
        } catch (Exception e) {
            LOGGER.error("【MQ-QUEUE_IMPP_WORK_WECHAT_MSG】处理出错：{}", e.getMessage(), e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        } finally {
            LOGGER.info("[Core Work WeChat] Message Id:{} , 耗时{}, Message Info :{}", msg.getId(),
                    System.currentTimeMillis() - startTime, JSON.toJSONString(msg));
        }
        return false;
    }

    /**
     * 系统邮件处理队列
     *
     * @param data    系统消息信息
     * @param channel 通道
     * @param message 消息
     */
    @RabbitListener(queues = PlatformConstWords.QUEUE_IMPP_WORK_WECHAT_MSG_CALLBACK)
    public Boolean processWorkWeChatMsgCallback(String data, Channel channel, Message message) {
        MsgTaskCardEvent msgTaskCardEvent = JsonUtilTool.decode(data, MsgTaskCardEvent.class);
        Long startTime = System.currentTimeMillis();
        try {
            if (!WorkWeChatEnumUtil.MSG_EVENT.TASK_CARD_CLICK.getValue().equals(msgTaskCardEvent.getEvent())) {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                return true;
            }

            Long msgId = Long.parseLong(msgTaskCardEvent.getTaskId());
            SysRefUserExternal sysRefUserExternal = sysRefUserExternalService.getSysRefUserExternalByRefAndValue(
                    ImppEnumUtil.USER_EXTERNAL_REF.WORK_WECHAT.getValue(), msgTaskCardEvent.getFromUserName()
            );

            if(sysRefUserExternal == null ){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("用户关系不存在")
                        .build();
            }
            SysRefUserMessage sysRefUserMessage = sysMessageService.getSysRefUserMessageByMsgIdAndUserId(msgId,
                    sysRefUserExternal.getUserId());

            sysMessageService.updateSysRefUserMessageStatusByIds(new Long[]{sysRefUserMessage.getId()},
                    ImppEnumUtil.MESSAGE_STATUS.READ.getValue());

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            LOGGER.error("【MQ-QUEUE_IMPP_WORK_WECHAT_MSG_CALLBACK】处理出错：{}", e.getMessage(), e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
//                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        } finally {
            LOGGER.info("[Core Work WeChat] Email Id:{} , 耗时{}, Message Info :{}", msgTaskCardEvent.getTaskId(),
                    System.currentTimeMillis() - startTime, JSON.toJSONString(msgTaskCardEvent));
        }
        return true;
    }
}
