package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSoftUpdateRecordService;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysSoftUpdateRecord;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.version.ImppSoftVersionService;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.io.IOException;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.SOFT_VERSION;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-09-14 14:13
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.soft-update-record:true}' == 'true'")
@Configuration
public class SoftUpdateRecordQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(SoftUpdateRecordQueueReceiver.class);

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Resource
    private ISysSoftUpdateRecordService sysSoftUpdateRecordService;

    @Bean
    public Queue queueImppSoftVersion() {
        return new Queue(PlatformConstWords.QUEUE_IMPP_SOFT_VERSION);
    }

    /**
     * 定时任务通道处理
     *
     * @param data    系统消息信息
     * @param channel 通道
     * @param message 消息
     */
    @RabbitListener(queues = PlatformConstWords.QUEUE_IMPP_SOFT_VERSION)
    public Boolean processUpdateRecord(String data, Channel channel, Message message) {
        try {
            LOGGER.debug("【MQ-QUEUE_IMPP_SOFT_VERSION】数据接收成功：{}", data);

            SysSoftUpdateRecord sysSoftUpdateRecord = JsonUtilTool.decode(data, SysSoftUpdateRecord.class);

            if (sysSoftUpdateRecordService.checkSysSoftUpdateRecordOnly(sysSoftUpdateRecord)) {
                sysSoftUpdateRecord.transformUpdateDetail();
                sysSoftUpdateRecordService.insert(sysSoftUpdateRecord);
            }

            String softVersionKey = SOFT_VERSION + ":" + CommonEnumUtil.SOFT_TYPE.valueOfCode(sysSoftUpdateRecord.getSystemSoftType());
            String currentVerStr = (String) redisRes.getObject(softVersionKey);
            if (ImppSoftVersionService.isUpdateVersion(currentVerStr, sysSoftUpdateRecord.getSystemVersion())) {
                redisRes.putObject(softVersionKey, sysSoftUpdateRecord.getSystemVersion(), -1);
            }

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            return true;
        } catch (IOException e) {
            LOGGER.error("【MQ-QUEUE_IMPP_SOFT_VERSION】处理出错：{}", e.getMessage(), e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
        return false;
    }

}
