package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysOrderNoRecordService;
import cn.estsh.i3plus.platform.common.tool.JsonUtilTool;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRecord;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRule;
import com.fasterxml.jackson.core.type.TypeReference;
import com.rabbitmq.client.Channel;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.QUEUE_IMPP_ORDER_NO_RECORD;

/**
 * @Description : 单号生成记录
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-09-03 22:15
 * @Modify:
 **/
@ConditionalOnExpression("'${impp.mq.queue.order-record:true}' == 'true'")
@Component
public class SysOrderNoRecordQueueReceiver {

    private static final Logger LOGGER = LoggerFactory.getLogger(SysOrderNoRecordQueueReceiver.class);

    @Autowired
    private ISysOrderNoRecordService orderNoRecordService;

    @Bean
    public Queue getQueueImppOrderNoRecord(){
        return new Queue(PlatformConstWords.QUEUE_IMPP_ORDER_NO_RECORD);
    }

    @RabbitListener(queues = QUEUE_IMPP_ORDER_NO_RECORD)
    public Boolean processImppMessage(String orderNoListStr, Channel channel, Message message) {
        try {
            List<SysOrderNoRule> orderNoRuleList = JsonUtilTool.decode(orderNoListStr, new TypeReference<List<SysOrderNoRule>>() {
            });
            if (CollectionUtils.isNotEmpty(orderNoRuleList)) {
                List<SysOrderNoRecord> orderNoRecordList = new ArrayList<>();
                orderNoRuleList.forEach(ds -> orderNoRecordList.add(new SysOrderNoRecord(ds.getOrderNoRuleCode(), ds.getOrderNo())));
                orderNoRecordService.saveBatch(orderNoRecordList);
            }
        } catch (Exception e) {
            LOGGER.error("单号生成记录异常", e);
        } finally {
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                LOGGER.info("[MQ ACK] Tag :{} Completed successfully",message.getMessageProperties().getDeliveryTag());
            } catch (IOException e) {
                LOGGER.error("[MQ ERROR] MQ Ack Error Message:{}",e.getMessage());
                e.printStackTrace();
            }
        }
        return true;
    }

}
