package cn.estsh.i3plus.core.apiservice.schedulejob;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsTemplateService;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsTemplate;
import cn.estsh.i3plus.sdk.sms.base.ISmsTemplateUtil;
import cn.estsh.i3plus.sdk.sms.base.SmsUtilFactory;
import cn.estsh.i3plus.sdk.sms.base.pojo.SmsResult;
import cn.estsh.impp.framework.base.schedule.BaseImppScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2021-01-04 13:47
 * @Modify:
 **/
@DisallowConcurrentExecution
public class SysSmsTemplateSyncJob extends BaseImppScheduleJob {

    public SysSmsTemplateSyncJob(Class cls, String jobDesc) {
        super(SysSmsTemplateSyncJob.class,"短信模板同步job");
    }

   @Autowired
   private ISysSmsTemplateService sysSmsTemplateService;

    @Override
    public void executeImppJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            ISmsTemplateUtil smsTemplateUtil = SmsUtilFactory.getSmsTemplateUtil();

            // 第一步，查询出未审批的短信模板
            List<SysSmsTemplate> sysSmsTemplateList =
                    sysSmsTemplateService.findSysSmsTemplateByStatus(ImppEnumUtil.SMS_TEMPLATE_STATUS.UNDER_REVIEW.getValue());
            // 第二步，根据供应商模板代码查询审批状态
            for (SysSmsTemplate smsTemplate : sysSmsTemplateList) {
                SmsResult<SysSmsTemplate> smsResult = smsTemplateUtil.getSmsTemplateByCode(smsTemplate.getSupplierTemplateCode());
                if (smsResult.isSuccess() && smsResult.getResultData() != null) {
                    smsTemplate.setTemplateStatus(smsResult.getResultData().getTemplateStatus());
                    // 第三步，更新模板状态
                    sysSmsTemplateService.update(smsTemplate);
                }else{
                    LOGGER.error("查询供应商短信模板状态异常：{}", smsResult.getErrMsg());
                }
            }
        }catch (Exception e){
            LOGGER.error("短信模板同步job执行异常",e);
        }
    }
}
