package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ICoreDataSourceService;
import cn.estsh.i3plus.core.apiservice.configuration.CoreJdbcTemplateConfig;
import cn.estsh.i3plus.core.apiservice.util.PlatformEnumUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.CoreDataSource;
import cn.estsh.i3plus.pojo.platform.repository.CoreDataSourceRepository;
import cn.estsh.i3plus.pojo.wms.bean.datasource.CusDatasource;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2022/1/7 11:12
 * @Modify:
 **/
@Service
public class CoreDataSourceServiceImpl implements ICoreDataSourceService{


    @Autowired
    private CoreDataSourceRepository coreDataSourceRepository;
    @Autowired
    private CoreJdbcTemplateConfig jdbcConfig;


    @Override
    public CoreDataSource updateDataObject(CoreDataSource dataObject) {
        coreDataSourceRepository.update(dataObject);
        return dataObject;
    }

    @Override
    public CoreDataSource saveDataSource(CoreDataSource source) {

        coreDataSourceRepository.insert(source);
        if (jdbcConfig.checkConnection(source)) {
            source.setSourceStatus(PlatformEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue());
        } else {
            source.setSourceStatus(PlatformEnumUtil.DATA_SOURCE_STATUS.CONN_FAILURE.getValue());
        }
        coreDataSourceRepository.update(source);
        return source;
    }

    @Override
    public CoreDataSource getCoreDatasourceObject(Long id) {
        return coreDataSourceRepository.getById(id);
    }


    @Override
    public void checkDataSourceOnly(CoreDataSource source) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(source.getSourceCode(), "sourceCode", ddlPackBean);
        int count = coreDataSourceRepository.findByHqlWhereCount(ddlPackBean);
        if (count > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("【唯一校验】数据源编码重复")
                    .setErrorSolution("请重新输入数据源编码")
                    .build();
        }
    }

    @Override
    public ListPager<CoreDataSource> queryDataObjectByPager(CoreDataSource coreDataSource, Pager pager) {
        if (coreDataSource == null) {
            pager = PagerHelper.getPager(pager, coreDataSourceRepository.listCount());
            return new ListPager(coreDataSourceRepository.listPager(pager), pager);
        } else {
            DdlPackBean hqlPack = DdlPackBean.getDdlPackBean();
            if (coreDataSource.getSourceCode() != null) {
                DdlPreparedPack.getStringLeftLikerPack(coreDataSource.getSourceCode(), "sourceCode", hqlPack);
            }
            if (coreDataSource.getSourceType() != null) {
                DdlPreparedPack.getNumEqualPack(coreDataSource.getSourceType(), "sourceType", hqlPack);
            }
            if (coreDataSource.getSourceStatus() != null) {
                DdlPreparedPack.getNumEqualPack(coreDataSource.getSourceStatus(), "sourceStatus", hqlPack);
            }
            if (coreDataSource.getSourceDataBaseName() != null) {
                DdlPreparedPack.getStringLeftLikerPack(coreDataSource.getSourceDataBaseName(), "sourceDataBaseName", hqlPack);
            }
            if (coreDataSource.getSourceHost() != null) {
                DdlPreparedPack.getStringLeftLikerPack(coreDataSource.getSourceHost(), "sourceHost", hqlPack);
            }
            if (coreDataSource.getSourceName() != null) {
                DdlPreparedPack.getStringLeftLikerPack(coreDataSource.getSourceName(), "sourceName", hqlPack);
            }
            pager = PagerHelper.getPager(pager, coreDataSourceRepository.findByHqlWhereCount(hqlPack));
            return new ListPager(coreDataSourceRepository.findByHqlWherePage(hqlPack, pager), pager);
        }
    }

    @Override
    public List<CoreDataSource> listAll() {
        return coreDataSourceRepository.list();
    }

    @Override
    public void delete(Long id) {
        CoreDataSource coreDatasource = coreDataSourceRepository.getById(id);
        coreDataSourceRepository.deleteWeaklyById(id, AuthUtil.getSessionUser().getUserName());
        jdbcConfig.removeJdbcTemplateBean(coreDatasource);
    }

    @Override
    public List<CoreDataSource> list() {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(AuthUtil.getOrganizeCode(), "organizeCode", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(PlatformEnumUtil.DATA_SOURCE_STATUS.CONN_SUCCESS.getValue(), "sourceStatus", ddlPackBean);
        return coreDataSourceRepository.findByHqlWhere(ddlPackBean);
    }

    @Override
    public void initCoreDataSource() {
        List<CoreDataSource> coreDataSourceList = coreDataSourceRepository.list();
        coreDataSourceList.forEach(source -> jdbcConfig.checkConnection(source));
    }
}
