package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.IDataSeparatorService;
import cn.estsh.i3plus.core.apiservice.configuration.CoreJdbcTemplate;
import cn.estsh.i3plus.core.apiservice.configuration.CoreJdbcTemplateConfig;
import cn.estsh.i3plus.core.apiservice.util.SeparatorDataUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorMessage;
import cn.estsh.i3plus.pojo.platform.bean.DataSeparatorRule;
import cn.estsh.i3plus.pojo.platform.repository.DataSeparatorRepository;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import com.alibaba.druid.DbType;
import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.ast.SQLStatement;
import com.alibaba.druid.sql.visitor.ExportParameterizedOutputVisitor;
import com.alibaba.druid.util.JdbcConstants;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Strings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2021/12/3 13:44
 * @Modify:
 **/
@Service
@Slf4j
public class DataSeparatorServiceImpl implements IDataSeparatorService {

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Resource
    private DataSeparatorRepository dataSeparatorRepository;

    @Resource
    private CoreJdbcTemplateConfig coreJdbcTemplateConfig;

    @Autowired
    private SeparatorDataUtil separatorDataUtil;

    @Override
    public void doSeparate(DataSeparatorMessage msg) {
        //获取数据来源数据源
        CoreJdbcTemplate srcJdbcTemplate = coreJdbcTemplateConfig.getCoreJdbcTemplate(msg.getSrcDataSourceId());

        //获取目的地的数据源
        CoreJdbcTemplate destJdbcTemplate = coreJdbcTemplateConfig.getCoreJdbcTemplate(msg.getDestDataSourceId());

        //迁移数据
        try {
            execute(msg,srcJdbcTemplate,destJdbcTemplate);
        } catch (Exception e) {
            throw ImppExceptionBuilder.newInstance().setErrorDetail(e.getMessage()+":separate data failed").build();
        }
        //删除原数据
//        baseRepository.deleteById(id);
    }

    @Override
    public boolean addRule(DataSeparatorRule msg) {
        ConvertBean.saveOrUpdate(msg, AuthUtil.getSessionUser().getUserName());
        DataSeparatorRule rule = dataSeparatorRepository.insert(msg);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + ":" + rule.getTableNameSrc().toUpperCase();
        redisRes.putObject(redisKey, rule, -1);
        return true;
    }

    @Override
    public boolean deleteRule(Long id) {
        DataSeparatorRule rule = dataSeparatorRepository.getById(id);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + rule.getTableNameSrc().toUpperCase();
        dataSeparatorRepository.deleteWeaklyById(id, AuthUtil.getSessionUser().getUserName());
        redisRes.deleteKey(redisKey);
        return true;
    }

    @Override
    public boolean update(DataSeparatorRule msg) {
        ConvertBean.saveOrUpdate(msg, AuthUtil.getSessionUser().getUserName());
        dataSeparatorRepository.update(msg);
        String redisKey = CommonConstWords.RES_DATA_SEPARATOR + ":" + msg.getTableNameSrc().toUpperCase();
        redisRes.putObject(redisKey, msg, -1);
        return true;
    }

    @Override
    public ListPager<DataSeparatorRule> query(DataSeparatorRule rule, Pager pager) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(rule.getOrganizeCode());

        if (!Strings.isNullOrEmpty(rule.getTableNameSrc())) {
            DdlPreparedPack.getStringRightLikerPack(rule.getTableNameSrc(), "tableNameSrc", ddlPackBean);
        }
        if (!Strings.isNullOrEmpty(rule.getRuleColumn())) {
            DdlPreparedPack.getStringRightLikerPack(rule.getRuleColumn(), "ruleColumn", ddlPackBean);
        }
        if (!Strings.isNullOrEmpty(rule.getRule())) {
            DdlPreparedPack.getStringRightLikerPack(rule.getRule(), "rule", ddlPackBean);
        }
        if (!Objects.isNull(rule.getDestDataSourceId())) {
            DdlPreparedPack.getNumEqualPack(rule.getDestDataSourceId(), "destDataSourceId", ddlPackBean);
        }
        if (!Objects.isNull(rule.getDestDataSourceId())) {
            DdlPreparedPack.getNumEqualPack(rule.getSrcDataSourceId(), "srcDataSourceId", ddlPackBean);
        }
        if (!Strings.isNullOrEmpty(rule.getTableNameDest())) {
            DdlPreparedPack.getStringRightLikerPack(rule.getTableNameDest(), "tableNameDest", ddlPackBean);
        }
        if (!Strings.isNullOrEmpty(rule.getSeparator())) {
            DdlPreparedPack.getStringEqualPack(rule.getSeparator(), "separator", ddlPackBean);
        }
        pager = PagerHelper.getPager(pager, dataSeparatorRepository.findByHqlWhereCount(ddlPackBean));

        return new ListPager(dataSeparatorRepository.findByHqlPage(ddlPackBean, pager), pager);
    }

    @Override
    public void separatorRedisInit() {
        //启动加载所有的数据到redis中
        List<DataSeparatorRule> list = dataSeparatorRepository.list();
        if (list.size() > 0){
            list.forEach(item -> {
                String refBeanName = item.getTableNameSrc().toUpperCase();
                String redisKey = CommonConstWords.RES_DATA_SEPARATOR + refBeanName;
                redisRes.putObject(redisKey, item, -1);
            });
        }
    }


    private void execute(DataSeparatorMessage msg,CoreJdbcTemplate srcJdbcTemplate,CoreJdbcTemplate descJdbcTemplate) throws Exception {
        //1.查询数据
        String selectSql = "select * from " + msg.getSrcTableName() + " where id =  " + msg.getId();
        HashMap<String, Object> paramMap = new HashMap<>(64);
        Map<String, Object> resultMap = srcJdbcTemplate.getNamedParameterJdbcTemplate().getJdbcTemplate().queryForMap(selectSql);
        paramMap.putAll(resultMap);
        //2.迁移数据
        String insertSql = separatorDataUtil.packInsertSql(paramMap, msg.getDestTableName());
        NamedParameterJdbcTemplate jdbcTemplate = descJdbcTemplate.getNamedParameterJdbcTemplate();
        boolean isSqlServer = isMSSQLSERVER(jdbcTemplate);
        DbType dbType;
        if (isSqlServer) {
            dbType = JdbcConstants.SQL_SERVER;
        } else {
            dbType = JdbcConstants.MYSQL;
        }
        Map<String, Object> tempResult = parseSQL(insertSql, dbType);
        String preparedSql = tempResult.get("preparedSql").toString();
        @SuppressWarnings("unchecked")
        List<Object> parameters = (List<Object>) tempResult.get("parameters");
        try {
            jdbcTemplate.getJdbcTemplate().update(preparedSql,parameters.toArray());
        } catch (DuplicateKeyException e) {
            log.info("迁移数据已经保存到目标库中,数据为:{}", JSON.toJSONString(resultMap));
            //3.删除原数据
            String deleteSql = "delete from " + msg.getSrcTableName() + " where id = :id ";
            srcJdbcTemplate.getNamedParameterJdbcTemplate().update(deleteSql,paramMap);
        }

        //4.删除原数据
        String deleteSql = "delete from " + msg.getSrcTableName() + " where id = :id ";
        srcJdbcTemplate.getNamedParameterJdbcTemplate().update(deleteSql,paramMap);
    }


    /**
     * 解析sql为预编sql和参数
     *
     * @param sql
     * @param dbType
     * @return
     */
    private Map<String, Object> parseSQL(String sql, DbType dbType) {

        List<SQLStatement> sqlStatements = SQLUtils.parseStatements(sql, dbType);

        ExportParameterizedOutputVisitor visitor = new ExportParameterizedOutputVisitor();
        for (SQLStatement statement : sqlStatements) {
            statement.accept(visitor);
        }
        String preparedSql = visitor.getAppender().toString();
        List<Object> parameters = visitor.getParameters();
        HashMap<String, Object> resultMap = new HashMap<>();
        resultMap.put("preparedSql", preparedSql);
        resultMap.put("parameters", parameters);
        return resultMap;
    }

    /**
     * 判断数据库是不是 MSSQLSERVER
     *
     * @return
     * @throws Exception
     */
    private boolean isMSSQLSERVER(NamedParameterJdbcTemplate srcJdbcTemplate) throws Exception {
        return srcJdbcTemplate.getJdbcTemplate().getDataSource().getConnection().getMetaData().getURL().contains("sqlserver");
    }
}
