package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISynchronizedService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysOrderNoRuleService;
import cn.estsh.i3plus.core.apiservice.util.OrderNoMakeUtil;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRule;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.redisson.api.RLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.REDIS_PREFIX_LOCK_GET_ORDER_NO;

/**
 * @Description :
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2019-09-03 14:00
 * @Modify:
 **/
@Service
public class SynchronizedService implements ISynchronizedService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SynchronizedService.class);

    @Autowired
    private ISysOrderNoRuleService sysOrderNoRuleService;

    @Resource(name = CommonConstWords.IMPP_REDIS_RES)
    private ImppRedis redisRes;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public SysOrderNoRule nextOrderNo(String code) {
        List<SysOrderNoRule> list = nextOrderNo(code, NumberUtils.INTEGER_ONE);
        if (CollectionUtils.isEmpty(list)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("[" + code + "] 生成失败")
                    .setErrorSolution("请重新输入规则代码")
                    .build();
        }
        return list.stream().findFirst().get();
    }

    /**
     * 根据单号代码获取单号信息
     *
     * @param code 单号代码
     * @param num
     * @return 单号信息
     */
    @Override
    public synchronized List<SysOrderNoRule> nextOrderNo(String code, int num) {
        //先拿规则
        String lockKey = CommonConstWords.REDIS_PREFIX_LOCK_GET_ORDER_NO + ":" + "LOCK" + ":" + code;
        RLock rLock = (RLock) redisRes.getLock(lockKey);
        rLock.lock();
        try {
            SysOrderNoRule codeRole = sysOrderNoRuleService.getSysOrderNoRuleByCode(code);
            if (codeRole == null) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                        .setErrorDetail("[" + code + "]规则代码不存在存在")
                        .setErrorSolution("请重新输入规则代码")
                        .build();
            } else if (codeRole.getOrderNoRuleStatus() == CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue()) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("[" + code + "]单号规则已禁用")
                        .setErrorSolution("请重新输入规则代码")
                        .build();
            }

            sysOrderNoRuleService.detachSysOrderNoRule(codeRole);

            String orderNoTemplate = OrderNoMakeUtil.getOrderNoTemplate(codeRole);
            String redisKey = REDIS_PREFIX_LOCK_GET_ORDER_NO + ":" + code + ":" + orderNoTemplate;

            long orginSerialNo = codeRole.getSerialNo();
            if (codeRole.getSerialNo() < 0 || OrderNoMakeUtil.isCycleByPrefix(codeRole, orderNoTemplate)) {
                orginSerialNo = 0;
            }
            long incr = codeRole.getSerialNoIncrement() * num;

            //缓存增加
            //先判断redis是否存在，如果不存在，则用默认初始值+步长*数量，如果存在，则添加步长 * 数量
            codeRole.setSerialNo(redisRes.putObjAdditional(redisKey, orginSerialNo, incr) - incr);

            // 设置缓存有效期
            redisRes.expire(redisKey, PlatformConstWords.REDIS_EXPIRE_SECONDS);

            // 生成单号更缓存
            List<SysOrderNoRule> orderNoRuleList = sysOrderNoRuleService.doGetSysOrderNoRuleByNum(codeRole, orderNoTemplate, num);

// FIXME 松下推送量太大并且该记录无意义 先注释
//        try {
//            rabbitTemplate.convertAndSend(PlatformConstWords.QUEUE_IMPP_ORDER_NO_RECORD, JsonUtilTool.encode(orderNoRuleList));
//        } catch (Exception e) {
//            LOGGER.error("单号记录推送异常", e);
//        }

            // 生成单号更缓存
            return orderNoRuleList;
        } catch (Exception e) {
            LOGGER.error("生成单号失败,单号代码:{},num:{}", code, num, e);
        } finally {
            if (rLock.isHeldByCurrentThread()) {
                rLock.unlock();
            }
        }
        return null;
    }
}
