package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.core.apiservice.util.NoSqlPack;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchPack;
import cn.estsh.i3plus.platform.plugin.elasticsearch.ElasticSearchTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.LogFieldModel;
import cn.estsh.i3plus.pojo.model.platform.SysLogModel;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogConsole;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogException;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogOperate;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogSystem;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import cn.estsh.impp.framework.boot.log.ImppLog;
import org.elasticsearch.search.aggregations.Aggregation;
import org.elasticsearch.search.aggregations.bucket.terms.ParsedTerms;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.metrics.ParsedAvg;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-2-13 下午4:16
 * @Modify:
 **/
@Service
public class SysLogElasticSearchService implements ISysLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogElasticSearchService.class);

    @Autowired
    private ImppLog imppLog;

    @Value(value = "${elasticsearch.host:}")
    private String elasticsearchHost;

    @Override
    public ListPager<SysLogException> queryLogExceptionPager(SysLogException log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogException(log), pager, SysLogException.class);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogException(log), pager, SysLogException.class);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogException(log), SysLogException.class);
    }

    @Override
    public SysLogException insertLogException(SysLogException log) {
        imppLog.insertSysLogException(log);
        return log;
    }

    @Override
    public SysLogException getLogException(SysLogException log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(), SysLogException.class);
    }

    @Override
    public void deleteLogException(SysLogException log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogException(log));
    }

    @Override
    public ListPager<SysLogOperate> queryLogOperatePager(SysLogOperate log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogOperate(log), pager, SysLogOperate.class);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogOperate(log), pager, SysLogOperate.class);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogOperate(log), SysLogOperate.class);
    }

    @Override
    public SysLogOperate insertLogOperate(SysLogOperate log) {
        imppLog.insertSysLogOperate(log);
        return log;
    }

    @Override
    public SysLogOperate getLogOperate(SysLogOperate log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(), SysLogOperate.class);
    }

    @Override
    public void deleteLogOperate(SysLogOperate log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogOperate(log));
    }

    @Override
    public ListPager<SysLogSystem> queryLogSystemPager(SysLogSystem log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogSystem(log), pager, SysLogSystem.class);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogSystem(log), pager, SysLogSystem.class);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogSystem(log), SysLogSystem.class);
    }

    @Override
    public SysLogSystem insertLogSystem(SysLogSystem log) {
        imppLog.insertSysLogSystem(log);
        return log;
    }

    @Override
    public SysLogSystem getLogSystem(SysLogSystem log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(), SysLogSystem.class);
    }

    @Override
    public void deleteLogSystem(SysLogSystem log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogSystem(log));
    }

    @Override
    public ListPager<SysLogTaskTime> queryLogTaskTimePager(SysLogTaskTime log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogTaskTime(log), pager, SysLogTaskTime.class);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogTaskTime(log), pager, SysLogTaskTime.class);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogTaskTime(log), SysLogTaskTime.class);
    }

    @Override
    public SysLogTaskTime insertLogTaskTime(SysLogTaskTime log) {
        imppLog.insertSysLogTaskTime(log);
        return log;
    }

    @Override
    public SysLogTaskTime getLogTaskTime(SysLogTaskTime log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(), SysLogTaskTime.class);
    }

    @Override
    public void deleteLogTaskTime(SysLogTaskTime log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogTaskTime(log));
    }
    /******************************************************* 定时任务日志 End *********************************************************/


    /******************************************************* 控制台日志 Start *********************************************************/

    @Override
    public ListPager<SysLogConsole> queryLogConsolePager(SysLogConsole log, Pager pager) {
        return ElasticSearchTool.searchPager(NoSqlPack.packEsSysLogConsole(log), pager, SysLogConsole.class);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log, Pager pager) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogConsole(log), pager, SysLogConsole.class);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log) {
        return ElasticSearchTool.searchList(NoSqlPack.packEsSysLogConsole(log), SysLogConsole.class);
    }

    @Override
    public SysLogConsole insertLogConsoleTime(SysLogConsole log) {
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(TimeTool.parseDate(log.getCreateDatetime()), SysLogConsole.class);
        ElasticSearchTool.insert(pack.getIndexName(), log);
        return log;
    }

    @Override
    public SysLogConsole getLogConsoleTime(SysLogConsole log) {
        return ElasticSearchTool.getPrimaryKey(log.getId(), SysLogConsole.class);
    }

    @Override
    public void deleteLogConsoleTime(SysLogConsole log) {
        ElasticSearchTool.deleteQuery(NoSqlPack.packEsSysLogConsole(log));
    }


    /******************************************************* 控制台日志 End *********************************************************/

    @Override
    public void actionCleanLog(String startTime, String endTime) {
        try {
            ElasticSearchTool.deleteIndex(ElasticSearchPack.getIndexName(SysLogSystem.class), startTime, endTime);
        } catch (Exception e) {
            LOGGER.error("清除系统日志异常", e);
        }
        try {
            ElasticSearchTool.deleteIndex(ElasticSearchPack.getIndexName(SysLogOperate.class), startTime, endTime);
        } catch (Exception e) {
            LOGGER.error("清除系统操作日志异常", e);
        }
        try {
            ElasticSearchTool.deleteIndex(ElasticSearchPack.getIndexName(SysLogException.class), startTime, endTime);
        } catch (Exception e) {
            LOGGER.error("清除系统异常日志异常", e);
        }
        try {
            ElasticSearchTool.deleteIndex(ElasticSearchPack.getIndexName(SysLogTaskTime.class), startTime, endTime);
        } catch (Exception e) {
            LOGGER.error("清除定时任务日志异常", e);
        }
    }

    @Override
    public Map<String, Object> querySysLogSystemAvgExecuteTime(String startTime, String endTime) {
        Map resultData = new LinkedHashMap();
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(SysLogSystem.class);
        pack.pager(CommonConstWords.getDefaultPager());
        pack.andStringDateTimeBetween("createDatetime", startTime, endTime);
        pack.group("createDatetime").avg("executeTime").buildAggregation();
        Map<String, Aggregation> aggregationMap = ElasticSearchTool.searchAggregationMap(pack);

        ParsedTerms parsedTerms = (ParsedTerms) aggregationMap.get("group_createDatetime");
        if (Objects.nonNull(parsedTerms)) {
            for (Terms.Bucket bucket : parsedTerms.getBuckets()) {
                try {
                    String key = bucket.getKeyAsString();
                    Double value = ((ParsedAvg) bucket.getAggregations().get("avg_executeTime")).getValue();
                    resultData.put(key, value.intValue());
                } catch (Exception e) {
                }
            }
        }

        return resultData;
    }

    @Override
    public Map<String, Object> querySysLogSystemByLevel(String startTime, String endTime) {
        Map resultData = new LinkedHashMap();
        ElasticSearchPack pack = ElasticSearchPack.imppBuild(SysLogSystem.class);
        pack.pager(CommonConstWords.getDefaultPager());
        pack.andStringDateTimeBetween("createDatetime", startTime, endTime);
        pack.group("logLevel").count("id").buildAggregation();
        Map<String, Aggregation> aggregationMap = ElasticSearchTool.searchAggregationMap(pack);


        ParsedTerms parsedTerms = (ParsedTerms) aggregationMap.get("group_logLevel");
        if (Objects.nonNull(parsedTerms)) {
            for (Terms.Bucket bucket : parsedTerms.getBuckets()) {
                try {
                    String key = bucket.getKeyAsString();
                    resultData.put(ImppEnumUtil.LOG_LEVEL.valueOfName(Integer.parseInt(key)), bucket.getDocCount());
                } catch (Exception e) {
                }
            }
        }

        return resultData;
    }

    @Override
    public List<LogFieldModel> querySysLog(SysLogModel sysLogModel, Pager pager) {
        final String time = "time";
        ElasticSearchTool.init(elasticsearchHost);
        ElasticSearchPack pack = new ElasticSearchPack(sysLogModel.getIndexName());

        pack.andKeywordEquals("level", sysLogModel.getLevel());
        if (!StringUtils.isEmpty(sysLogModel.getHostId())) {
            pack.andKeywordEquals("hostId", sysLogModel.getHostId());
        }
        if (!StringUtils.isEmpty(sysLogModel.getIndexType())) {
            pack.andKeywordEquals("indexType", sysLogModel.getIndexType());
        }
        if (!StringUtils.isEmpty(sysLogModel.getThread())) {
            pack.andKeywordEquals("thread", sysLogModel.getThread());
        }
//        pack.andKeywordDateTimeBetween(time,
//                sysLogModel.getQueryDate().trim() + " " + sysLogModel.getQueryBeginTime(),
//                sysLogModel.getQueryDate().trim() + " " + sysLogModel.getQueryEndTime());
//
//        pack.orderKeywordDesc(time);
        ListPager<LogFieldModel> listPager = ElasticSearchTool.searchPager(pack, pager, LogFieldModel.class);

        return listPager.getObjectList();
    }

    /******************************************************* 系统异常日志 End *********************************************************/


}
