package cn.estsh.i3plus.core.apiservice.serviceimpl.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysLogService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.LogFieldModel;
import cn.estsh.i3plus.pojo.model.platform.SysLogModel;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogConsole;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogException;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogOperate;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogSystem;
import cn.estsh.i3plus.pojo.platform.platbean.SysLogTaskTime;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.log.ImppLog;
import cn.estsh.impp.framework.boot.util.ImppSwitchUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-2-13 下午4:16
 * @Modify:
 **/
@Service
@Primary
public class SysLogFactory implements ISysLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogFactory.class);

    @Autowired
    private SysLogElasticSearchService elasticSearchService;

    @Autowired
    private ImppLog imppLog;

    private ISysLogService getSysLogService() {
        ImppEnumUtil.LOG_STORAGE storage = imppLog.getLogStorageCase();

//        if(ImppEnumUtil.LOG_STORAGE.MONGO.equals(storage)){
//            if(ImppSwitchUtil.isSwitchMongo() && mongoService != null){
//                return mongoService;
//            }else{
//                throw ImppExceptionBuilder.newInstance()
//                        .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
//                        .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
//                        .setErrorDetail("没有开启Mongo储存技术,请联系管理员")
//                        .setErrorSolution("请检查配置文件")
//                        .build();
//            }
//        }else
        if (ImppEnumUtil.LOG_STORAGE.ELASTICSEARCH.equals(storage)) {
            if (ImppSwitchUtil.isElasticSearchSwitch() && elasticSearchService != null) {
                return elasticSearchService;
            } else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                        .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                        .setErrorDetail("没有开启Elasticsearch储存技术,请联系管理员")
                        .setErrorSolution("请检查配置文件")
                        .build();
            }
        }

        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                .setErrorCode(ImppExceptionEnum.LOG_SAVE_STORAGE_ERROR.getCode())
                .setErrorDetail("没有开启日志储存技术,请联系管理员")
                .setErrorSolution("请检查日志配置,或者配置文件")
                .build();
    }

    @Override
    public ListPager<SysLogException> queryLogExceptionPager(SysLogException log, Pager pager) {
        return getSysLogService().queryLogExceptionPager(log, pager);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log, Pager pager) {
        return getSysLogService().findLogExceptionList(log, pager);
    }

    @Override
    public List<SysLogException> findLogExceptionList(SysLogException log) {
        return getSysLogService().findLogExceptionList(log);
    }

    @Override
    public SysLogException insertLogException(SysLogException log) {
        return getSysLogService().insertLogException(log);
    }

    @Override
    public SysLogException getLogException(SysLogException log) {
        return getSysLogService().getLogException(log);
    }

    @Override
    public void deleteLogException(SysLogException log) {
        getSysLogService().deleteLogException(log);
    }

    @Override
    public ListPager<SysLogOperate> queryLogOperatePager(SysLogOperate log, Pager pager) {
        return getSysLogService().queryLogOperatePager(log, pager);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log, Pager pager) {
        return getSysLogService().findLogOperateList(log, pager);
    }

    @Override
    public List<SysLogOperate> findLogOperateList(SysLogOperate log) {
        return getSysLogService().findLogOperateList(log);
    }

    @Override
    public SysLogOperate insertLogOperate(SysLogOperate log) {
        return getSysLogService().insertLogOperate(log);
    }

    @Override
    public SysLogOperate getLogOperate(SysLogOperate log) {
        return getSysLogService().getLogOperate(log);
    }

    @Override
    public void deleteLogOperate(SysLogOperate log) {
        getSysLogService().deleteLogOperate(log);
    }

    @Override
    public ListPager<SysLogSystem> queryLogSystemPager(SysLogSystem log, Pager pager) {
        return getSysLogService().queryLogSystemPager(log, pager);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log, Pager pager) {
        return getSysLogService().findLogSystemList(log, pager);
    }

    @Override
    public List<SysLogSystem> findLogSystemList(SysLogSystem log) {
        return getSysLogService().findLogSystemList(log);
    }

    @Override
    public SysLogSystem insertLogSystem(SysLogSystem log) {
        return getSysLogService().insertLogSystem(log);
    }

    @Override
    public SysLogSystem getLogSystem(SysLogSystem log) {
        return getSysLogService().getLogSystem(log);
    }

    @Override
    public void deleteLogSystem(SysLogSystem log) {
        getSysLogService().deleteLogSystem(log);
    }

    @Override
    public ListPager<SysLogTaskTime> queryLogTaskTimePager(SysLogTaskTime log, Pager pager) {
        return getSysLogService().queryLogTaskTimePager(log, pager);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log, Pager pager) {
        return getSysLogService().findLogTaskTimeList(log, pager);
    }

    @Override
    public List<SysLogTaskTime> findLogTaskTimeList(SysLogTaskTime log) {
        return getSysLogService().findLogTaskTimeList(log);
    }

    @Override
    public SysLogTaskTime insertLogTaskTime(SysLogTaskTime log) {
        return getSysLogService().insertLogTaskTime(log);
    }

    @Override
    public SysLogTaskTime getLogTaskTime(SysLogTaskTime log) {
        return getSysLogService().getLogTaskTime(log);
    }

    @Override
    public void deleteLogTaskTime(SysLogTaskTime log) {
        getSysLogService().deleteLogTaskTime(log);
    }


    @Override
    public ListPager<SysLogConsole> queryLogConsolePager(SysLogConsole log, Pager pager) {
        return getSysLogService().queryLogConsolePager(log, pager);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log, Pager pager) {
        return getSysLogService().findLogConsoleList(log, pager);
    }

    @Override
    public List<SysLogConsole> findLogConsoleList(SysLogConsole log) {
        return getSysLogService().findLogConsoleList(log);
    }

    @Override
    public SysLogConsole insertLogConsoleTime(SysLogConsole log) {
        return getSysLogService().insertLogConsoleTime(log);
    }

    @Override
    public SysLogConsole getLogConsoleTime(SysLogConsole log) {
        return getSysLogService().getLogConsoleTime(log);
    }

    @Override
    public void deleteLogConsoleTime(SysLogConsole log) {
        getSysLogService().deleteLogConsoleTime(log);
    }


    /******************************************************* 定时任务日志 End *********************************************************/

    @Override
    public void actionCleanLog(String startTime, String endTime) {
        getSysLogService().actionCleanLog(startTime, endTime);
    }

    @Override
    public Map<String, Object> querySysLogSystemAvgExecuteTime(String startTime, String endTime) {
        return getSysLogService().querySysLogSystemAvgExecuteTime(startTime, endTime);
    }

    @Override
    public Map<String, Object> querySysLogSystemByLevel(String startTime, String endTime) {
        return getSysLogService().querySysLogSystemByLevel(startTime, endTime);
    }

    @Override
    public List<LogFieldModel> querySysLog(SysLogModel logModel, Pager pager) {
        return getSysLogService().querySysLog(logModel, pager);
    }

    /******************************************************* 系统异常日志 End *********************************************************/


}
