package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleResourceService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.SerializeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleResource;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.repository.SysMenuRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysRefRoleMenuRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.RESOURCE_PREFIX_MENU;

/**
 * @Description : 系统功能业务接口实现
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 15:18:14.969
 * @Modify :
 **/
@Service
public class SysMenuService extends CrudService<SysMenu> implements ISysMenuService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysMenuService.class);

    /**
     * 功能权限国际化
     */
    private final static Map<String, Map<String, String>> MEM_CACHE_LANGAGE = new HashMap<>();

    @Autowired
    private SysMenuRepository sysMenuRDao;

    @Autowired
    private CoreTreeService coreTreeService;

    @Autowired
    private SysRefRoleMenuRepository refRoleMenuRDao;

    @Autowired
    private ISysLocaleResourceService localeResourceService;

    @Override
    public BaseRepository<SysMenu, Long> getRepository() {
        return sysMenuRDao;
    }

    @Override
    public DdlPackBean getFindPagerPackBean(SysMenu bean) {
        return CoreHqlPack.packDdlBeanSysMenu(bean, null);
    }

    @Override
    public SysMenu packBeforeUpdate(SysMenu bean) {
        return super.packBeforeUpdate(bean);
    }

    @Override
    public void checkSysMenuOnly(SysMenu bean) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean);
        DdlPreparedPack.getNumNOEqualPack(bean.getId(), "id", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getMenuCode(), "menuCode", packBean);

        if (sysMenuRDao.findByHqlWhereCount(packBean) > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("权限代码已存在")
                    .setErrorSolution("请重新输入权限代码")
                    .build();
        }
    }

    @Override
    public void refreshMemSysMenuLanguage() {
        MEM_CACHE_LANGAGE.clear();
        SysLocaleResource resource = new SysLocaleResource();
        resource.setResourceType(CommonEnumUtil.SYS_LOCALE_RESOURCE_TYPE.MENU.getValue());
        List<SysLocaleResource> resourceList = localeResourceService.findSysLocaleResource(resource);
        if (CollectionUtils.isNotEmpty(resourceList)) {
            for (SysLocaleResource localeResource : resourceList) {
                Map<String, String> map = MEM_CACHE_LANGAGE.get(localeResource.getLanguageCode());
                if (Objects.isNull(map)) {
                    map = new HashMap<>();
                }
                map.put(localeResource.getResourceKey(), localeResource.getResourceValue());
                MEM_CACHE_LANGAGE.put(localeResource.getLanguageCode(), map);
            }
        }
    }

    @Override
    public List<SysMenu> setLanguageSysMenuName(String languageCode, List<SysMenu> menuList) {
        languageCode = StringUtils.isNotBlank(languageCode) ? languageCode : CommonConstWords.DEFAULT_LANGUAGE;
        Map<String, String> localeResourceMap = MEM_CACHE_LANGAGE.get(languageCode);

        if (menuList != null && localeResourceMap != null) {
            List<SysMenu> resultList = (List<SysMenu>) SerializeTool.copyObject(menuList);
            for (SysMenu menu : resultList) {
                if (menu != null) {
                    if (StringUtils.isNotBlank(menu.getName())) {
                        String menuName = localeResourceMap.get(RESOURCE_PREFIX_MENU + menu.getName());
                        menu.setName(StringUtils.isNotBlank(menuName) ? menuName : menu.getName());
                    }

                    if (StringUtils.isNotBlank(menu.getParentNameRdd())) {
                        String parentNameRdd = localeResourceMap.get(RESOURCE_PREFIX_MENU + menu.getParentNameRdd());
                        menu.setParentNameRdd(StringUtils.isNotBlank(parentNameRdd) ? parentNameRdd : menu.getParentNameRdd());
                    }

                    if(CollectionUtils.isNotEmpty(menu.getChildList())){
                        menu.setChildList(setLanguageSysMenuName(languageCode, menu.getChildList()));
                    }
                }
            }
            return resultList;
        }

        return menuList;
    }

    //    @Override
//    @ApiOperation(value = "更新菜单信息")
//    public void updateSysMenu(SysMenu menu) {
//        LOGGER.info("系统功能 SYS_MENU :{}", menu);
//
//        // 唯一校验 CODE 相同 ID 不同 则视为相同
//        StringBuffer countWhere = new StringBuffer();
//
//        HqlPack.getStringEqualPack( menu.getMenuCode(),"menuCode",countWhere);
//        HqlPack.getNumNOEqualPack(menu.getId().toString(),"id",countWhere);
//
//        int count = sysMenuRDao.findByHqlWhereCount(countWhere.toString());
//        if(count > 0){
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
//                    .setErrorDetail("权限代码已存在")
//                    .setErrorSolution("请重新输入权限代码")
//                    .build();
//        }
//
//        // 父节点检查
//        coreTreeService.doValidatorSysMenuParentId(menu);
//
//        if (menu.getParentId() != null  && menu.getParentId() > 0) {
//            SysMenu parent = sysMenuRDao.getById(menu.getParentId());
//            ValidatorBean.checkNotNull(parent,"上级权限信息不存在");
//            menu.setParentNameRdd(parent.getName());
//        }
//
//        sysMenuRDao.update(menu);
//        // 更新冗余信息
//        updateSysMenuChildRdd(menu);
//    }
//
    @Override
    @ApiOperation(value = "更新菜单信息",notes = "根据ID 更新菜单状态信息")
    public void updateSysMenuStatus(Long id, int status, SessionUser user) {
        LOGGER.info("系统功能 SYS_MENU  id:{} status:{} user:{}", id,status,user);
        SysMenu menu = sysMenuRDao.getById(id);
        if(menu != null){
            menu.setMenuStatus(status);
            menu.setModifyUser(user.getUserName());
            sysMenuRDao.update(menu);
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail("数据不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }
//
//    @Override
//    @ApiOperation(value = "更新菜单信息",notes = "根据ID 批量更新菜单状态信息")
//    public void updateSysMenuStatusByIds(Long[] ids, int status, SessionUser user) {
//        LOGGER.info("系统功能 SYS_MENU  ids:{}  status:{} user:{}", ids, status, user);
//        StringBuffer where = new StringBuffer();
//
//        HqlPack.getInPack(StringUtils.join( ids,","), "id", where);
//        sysMenuRDao.updateByHqlWhere(where.toString(), "menuStatus", status);
//    }
//
//    @Override
//    public void updateSysMenuChildRdd(SysMenu sysMenu) {
//        LOGGER.info("系统功能 SYS_MENU  sysMenu:{}", sysMenu);
//        if(sysMenu != null){
//            sysMenuRDao.updateByProperties(
//                    new String[]{"parentId"}, new Object[]{sysMenu.getId()},
//                    new String[]{"parentNameRdd"}, new Object[]{sysMenu.getName()});
//        }
//    }
//
//    @Override
//    @ApiOperation(value = "删除菜单信息",notes = "根据ID 删除菜单信息")
//    public void deleteSysMenuById(Long id) {
//        LOGGER.info("系统功能 SYS_MENU Key:{}", id);
//        long count = sysMenuRDao.findByPropertyCount("parentId",id);
//        if(count >= 1){
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
//                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
//                    .setErrorDetail("该数据有子集")
//                    .setErrorSolution("请先删除子集在操作")
//                    .build();
//
//        }
//
//        refRoleMenuRDao.deleteById(id);
//        sysMenuRDao.deleteById(id);
//    }
//
//    @Override
//    @ApiOperation(value = "删除菜单信息",notes = "根据ID 批量删除菜单信息")
//    public void deleteSysMenuByIds(Long[] ids) {
//        LOGGER.info("系统功能 SYS_MENU ids :{}", ids);
//        long positionCount = sysMenuRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("parentId",ids));
//        if (positionCount >= 1) {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                    .setErrorDetail("存在相关系统功能信息无法删除！")
//                    .setErrorSolution("请先删除子集信息在操作")
//                    .build();
//        }else {
//            refRoleMenuRDao.deleteByPropertyIn("menuId",ids);
//            sysMenuRDao.deleteByIds(ids);
//        }
//    }
//
//    @Override
//    @ApiOperation(value = "新增菜单信息")
//    public SysMenu insertSysMenu(SysMenu sysMenu) {
//        LOGGER.info("系统功能 SYS_MENU :{}", sysMenu);
//
//        // 代码唯一校验
//        long count = sysMenuRDao.findByPropertyCount("menuCode", sysMenu.getMenuCode());
//        if(count <= 0){
//            if (sysMenu.getParentId() != null  && sysMenu.getParentId() > 0) {
//                SysMenu parent = sysMenuRDao.getById(sysMenu.getParentId());
//                ValidatorBean.checkNotNull(parent,"上级权限信息不存在");
//                sysMenu.setParentNameRdd(parent.getName());
//            }
//            return sysMenuRDao.insert(sysMenu);
//        }else {
//            throw ImppExceptionBuilder.newInstance()
//                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
//                    .setErrorDetail("功能代码已存在")
//                    .setErrorSolution("请重新输入功能代码")
//                    .build();
//        }
//    }
//
//    @Override
//    public List<SysMenu> refreshSysMenu(List<SysMenu> list) {
//        LOGGER.info("系统功能 SYS_MENU list :{}", list);
//
////        sysMenuRDao.deleteAll();
//        return sysMenuRDao.saveAll(list);
//    }
//
//    @Override
//    @ApiOperation(value = "查询菜单信息",notes = "查询所有菜单信息")
//    public List<SysMenu> findSysMenuAll() {
//        LOGGER.info("系统功能 SYS_MENU find All");
//        SysMenu menu = new SysMenu();
//        menu.setMenuStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());
//        menu.setOrderByParam("menuSort");
//        menu.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
//        return sysMenuRDao.findByHqlWhere(CoreHqlPack.packDdlBeanSysMenu(menu,null));
//    }
//
//    @Override
//    public List<SysMenu> findSysMenuByMenuAndInId(SysMenu menu,List<Long> idList) {
//        return sysMenuRDao.findByHqlWhere(CoreHqlPack.packDdlBeanSysMenu(menu,idList));
//    }
//
//    @Override
//    public List<SysMenu> findLanguageMenuByMenuAndInId(SysMenu menu, List<Long> idList) {
//        List<SysMenu> menuList = sysMenuRDao.findByHqlWhere(CoreHqlPack.packDdlBeanSysMenu(menu, idList));
//        if(menuList != null && menuList.size() > 0){
//            menuList = localeResourceService.tranSysMenu(menuList);
//        }
//        return menuList;
//    }
//
//    @Override
//    @ApiOperation(value = "查询菜单信息",notes = "根据ID查询菜单信息")
//    public SysMenu getSysMenuById(Long id) {
//        LOGGER.info("系统功能 SYS_MENU find id:{}", id);
//        return sysMenuRDao.getOne(id);
//    }
//
//    @Override
//    public ListPager<SysMenu> querySysMenu(SysMenu menu, List<Long> idList, Pager pager) {
//        DdlPackBean packBean = CoreHqlPack.packDdlBeanSysMenu(menu, idList);
//        pager = PagerHelper.getPager(pager, sysMenuRDao.findByHqlWhereCount(packBean));
//        return new ListPager(sysMenuRDao.findByHqlWherePage(packBean, pager), pager);
//    }
//
    @Override
    public SysMenu getSysMenuByCode(String code) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(code, "menuCode", packBean);
        return sysMenuRDao.getByProperty(packBean);
    }

    @Override
    public void deleteSysMenuById(Long id) {
        List<Long> idList = new ArrayList<>();
        SysMenu sysMenu = sysMenuRDao.getById(id);
        if(sysMenu != null){
            idList.add(sysMenu.getId());
            List<Long> oneList = getIdList(id,idList);
            if(oneList != null){
                sysMenuRDao.deleteByIds(idList.stream().toArray(Long[]::new));
            }
        }
    }

    private List<Long> getIdList(Long id,List<Long> idList){
        List<SysMenu> sysMenuTwoList = sysMenuRDao.findByProperty("parentId",id);
        if(CollectionUtils.isEmpty(sysMenuTwoList)){
            return null;
        }
        List<Long> twoList = sysMenuTwoList.stream().map(SysMenu::getId).collect(Collectors.toList());

        List<SysMenu> sysMenuThreeList = getSysMenuList(twoList);
        if(CollectionUtils.isEmpty(sysMenuThreeList)){
            return null;
        }
        List<Long> threeList = sysMenuThreeList.stream().map(SysMenu::getId).collect(Collectors.toList());

        List<SysMenu> sysMenuFourList = getSysMenuList(twoList);
        if(CollectionUtils.isEmpty(sysMenuFourList)){
            return null;
        }
        List<Long> fourList = sysMenuFourList.stream().map(SysMenu::getId).collect(Collectors.toList());

        idList.addAll(twoList);
        idList.addAll(threeList);
        idList.addAll(fourList);
        return idList;
    }

    private List<SysMenu> getSysMenuList(List<Long> idList){
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(idList, "parentId", ddlPackBean);
        return sysMenuRDao.findByHqlWhere(ddlPackBean);
    }
}
