package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysOrderNoRuleService;
import cn.estsh.i3plus.platform.common.tool.CloneTool;
import cn.estsh.i3plus.core.apiservice.util.OrderNoMakeUtil;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRule;
import cn.estsh.i3plus.pojo.platform.repository.SysOrderNoRuleRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 系统单号生成规则接口实现
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-21 20:04
 * @Modify:
 **/
@Service
public class SysOrderNoRuleService implements ISysOrderNoRuleService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysOrderNoRuleService.class);

    @Autowired
    private SysOrderNoRuleRepository sysOrderNoRuleRDao;

    @Override
    public SysOrderNoRule insertSysOrderNoRule(SysOrderNoRule sysOrderNoRule) {
        if (sysOrderNoRuleRDao.isExitByProperty("orderNoRuleCode", sysOrderNoRule.getOrderNoRuleCode())) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("规则代码已存在")
                    .setErrorSolution("请重新输入规则代码")
                    .build();
        }
        return sysOrderNoRuleRDao.insert(sysOrderNoRule);
    }

    @Override
    public void updateSysOrderNoRule(SysOrderNoRule sysOrderNoRule) {
        // 校验唯一
        String hqlPack = CoreHqlPack.packHqlSysOrderNoRuleCode(sysOrderNoRule);
        long repeatCount = sysOrderNoRuleRDao.findByHqlWhereCount(hqlPack);
        if (repeatCount > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("规则代码已存在")
                    .setErrorSolution("请重新输入规则代码")
                    .build();
        }
        sysOrderNoRuleRDao.update(sysOrderNoRule);
    }

    @Override
    public void deleteSysOrderNoRuleById(Long id) {
        sysOrderNoRuleRDao.deleteById(id);
    }

    @Override
    public SysOrderNoRule getSysOrderNoRuleById(Long id) {
        return sysOrderNoRuleRDao.getById(id);
    }

    /**
     * 根据code 查询单号
     *
     * @param code 单号code
     * @return 单号信息
     */
    @Override
    public SysOrderNoRule getSysOrderNoRuleByCode(String code) {
        return sysOrderNoRuleRDao.getByProperty("orderNoRuleCode",code);
    }

    @Override
    public ListPager querySysOrderNoRuleByPager(SysOrderNoRule sysOrderNoRule, Pager pager) {
        if (sysOrderNoRule == null) {
            pager = PagerHelper.getPager(pager, sysOrderNoRuleRDao.listCount());
            return new ListPager(sysOrderNoRuleRDao.listPager(pager), pager);
        } else {
            DdlPackBean hqlPack = CoreHqlPack.packHqlSysOrderNoRule(sysOrderNoRule);
            pager = PagerHelper.getPager(pager, sysOrderNoRuleRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysOrderNoRuleRDao.findByHqlWherePage(hqlPack, pager), pager);
        }
    }

    /**
     * 根据单号代码获取单号信息
     *
     * @param sysOrderNoRule 单号代码
     * @param orderNoTemplate
     * @param num
     * @return 单号信息
     */
    @Override
    public synchronized List<SysOrderNoRule> doGetSysOrderNoRuleByNum(SysOrderNoRule sysOrderNoRule, String orderNoTemplate, int num) {
        List<SysOrderNoRule> resultList = new ArrayList<>();
        try {
            for (int i = 0; i < num; i++) {
                sysOrderNoRule = OrderNoMakeUtil.next(CloneTool.clone(sysOrderNoRule), orderNoTemplate);
                resultList.add(sysOrderNoRule);
            }
        }catch (Exception e){
            LOGGER.error("单号生成异常：",e);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION.getCode())
                    .setErrorDetail("单号生成异常 %s",e.getMessage())
                    .build();
        }

        sysOrderNoRuleRDao.update(sysOrderNoRule);
        return resultList;
    }

    @Override
    public void updateSysOrderNoRuleCodeStatusById(Long id, Integer status) {
        sysOrderNoRuleRDao.updateByProperties("id", id, "orderNoRuleStatus", status);
    }

    @Override
    public void deleteSysOrderNoRuleByIds(Long[] ids) {
        sysOrderNoRuleRDao.deleteByIds(ids);
    }

    @Override
    public void updateSysOrderNoRuleByIds(Long[] ids, Integer status) {
        StringBuffer where = new StringBuffer();
        HqlPack.getInPack(String.join(",", StringTool.getArrayString(ids)), "id", where);
        sysOrderNoRuleRDao.updateByHqlWhere(where.toString(), "orderNoRuleStatus", status);
    }

    @Override
    public void insertSysOrderNoRuleList(List sysOrderNoRuleList) {
        sysOrderNoRuleRDao.saveAll(sysOrderNoRuleList);
    }

    /**
     * @param sysOrderNoRule
     */
    @Override
    public void detachSysOrderNoRule(SysOrderNoRule sysOrderNoRule) {
        sysOrderNoRuleRDao.detachObject(sysOrderNoRule);
    }


}
