package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysPositionService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SysPosition;
import cn.estsh.i3plus.pojo.platform.repository.SysPositionRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysUserInfoRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import com.dingtalk.api.request.OapiAlitripBtripApprovalNewRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 岗位服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-19 16:36
 * @Modify :
 **/
@Service
public class SysPositionService extends CrudService<SysPosition> implements ISysPositionService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysPositionService.class);

    @Autowired
    private SysPositionRepository positionRDao;

    @Autowired
    private CoreTreeService coreTreeService;

    @Autowired
    private SysUserInfoRepository userInfoRDao;

    @Override
    public BaseRepository<SysPosition, Long> getRepository() {
        return positionRDao;
    }

    @Override
    public DdlPackBean getFindPagerPackBean(SysPosition bean) {
        return CoreHqlPack.packHqlSysPosition(bean);
    }

    @Override
    public SysPosition packBeforeInsert(SysPosition bean) {
        checkSysPositionOnly(bean);

        if (bean.getParentId() != null && bean.getParentId() > 0) {
            LOGGER.info("岗位信息 POSITION parentId:{}", bean.getParentId());

            SysPosition parentSysPosition = positionRDao.getById(bean.getParentId());
            ValidatorBean.checkNotNull(parentSysPosition, "上级岗位信息不存在");
            bean.setParentNameRdd(parentSysPosition.getName());
        }
        return bean;
    }

    @Override
    public SysPosition packBeforeUpdate(SysPosition bean) {
        checkSysPositionOnly(bean);

        if (bean.getParentId() != null && bean.getParentId() > 0) {
            LOGGER.info("岗位信息 POSITION parentId:{}", bean.getParentId());

            SysPosition parentSysPosition = positionRDao.getById(bean.getParentId());
            ValidatorBean.checkNotNull(parentSysPosition, "上级岗位信息不存在");
            bean.setParentNameRdd(parentSysPosition.getName());
        }

        // 父节点检查
        coreTreeService.doValidatorSysPositionParentId(bean);

        // 更新冗余信息
        positionRDao.updateByProperties(new String[]{"parentId"}, new Object[]{bean.getId()}, new String[]{"parentNameRdd"}, new Object[]{bean.getName()});

        // 更新用户主岗位冗余信息
        userInfoRDao.updateByProperties(new String[]{"positionId"}, new Object[]{bean.getId()}, new String[]{"positionNameRdd"}, new Object[]{bean.getName()});

        return bean;
    }

    @Override
    public void checkSysPositionOnly(SysPosition position) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(position);
        DdlPreparedPack.getNumNOEqualPack(position.getId(), "id", packBean);
        DdlPreparedPack.getStringEqualPack(position.getPositionCode(), "positionCode", packBean);

        if (positionRDao.findByHqlWhereCount(packBean) > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("岗位代码已存在")
                    .setErrorSolution("请重新输入岗位代码")
                    .build();
        }
    }

    @Override
    public ListPager<SysPosition> querySysPosition(SysPosition position, List<Long> idList, Pager pager) {
        DdlPackBean ddlPackBean = CoreHqlPack.packHqlSysPosition(position, idList);
        pager = PagerHelper.getPager(pager, positionRDao.findByHqlWhereCount(ddlPackBean));
        return new ListPager(positionRDao.findByHqlWherePage(ddlPackBean, pager), pager);
    }

    @Override
    public SysPosition findPositionByPositionCodeAndOrganizeId(String positionCode, String organizeCode) {
        String[] columns = {"positionCode", "organizeCode"};
        Object[] values = {positionCode, organizeCode};
        return positionRDao.getByProperty(columns, values);
    }

    @Override
    public List<SysPosition> findAllByBaseBean() {
//        Example example = Example.of(sysPosition);
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        return this.positionRDao.findByHqlWhere(ddlPackBean);
    }

    @Override
    public List<SysPosition> findAllByPositionName(String positionName) {
        List<SysPosition> positionList = new ArrayList();
        if (StringUtils.isEmpty(positionName)) {
            return positionList;
        }
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(positionName, "name", ddlPackBean);
        positionList = this.positionRDao.findByHqlWhere(ddlPackBean);
        return positionList;
    }

}
