package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysSmsTemplateService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysSmsTemplate;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.CrudService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 短信模板
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2020-12-14 20:20
 * @Modify:
 **/
@Service
public class SysSmsTemplateService extends CrudService<SysSmsTemplate> implements ISysSmsTemplateService {

    @Override
    public DdlPackBean getFindPagerPackBean(SysSmsTemplate bean) {
        return CoreHqlPack.packHqlFindSysSmsTemplate(bean);
    }

    /**
     * 检查短信模板是否唯一
     *
     * @param smsTemplate 待检查的短信模板数据
     * @return 检查结果
     */
    @Override
    public boolean checkSysSmsTemplateOnly(SysSmsTemplate smsTemplate) {
        return !baseRepository.isExitByHql(CoreHqlPack.packHqlCheckSysSmsTemplateOnly(smsTemplate));
    }

    /**
     * 根据code短信模板
     *
     * @param code 模板代码
     * @return 查询结果
     */
    @Override
    public SysSmsTemplate getSysSmsTemplateByCode(String code) {
        return baseRepository.getByProperty(
                new String[]{"isValid", "isDeleted", "templateCode"},
                new Object[]{
                        CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                        code
                }
        );
    }

    /**
     * 根据模板状态查询模板信息
     *
     * @param status 模板状态
     * @return 查询结果
     */
    @Override
    public List<SysSmsTemplate> findSysSmsTemplateByStatus(Integer status) {
        return baseRepository.findByProperty(
                new String[]{"isValid", "isDeleted", "templateStatus"},
                new Object[]{
                        CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue(),
                        CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                        status
                }
        );
    }
}
