package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserPasswordService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.util.SysUserPasswordUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.jpa.dao.BaseRepository;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.bean.SysUserPassword;
import cn.estsh.i3plus.pojo.platform.repository.SysUserPasswordRepository;
import cn.estsh.impp.framework.base.service.CrudService;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-6-4 下午3:49
 * @Modify:
 **/
@Primary
@Service
public class SysUserSavePasswordService extends CrudService<SysUserPassword> implements ISysUserPasswordService {

    @Autowired
    private SysUserPasswordRepository userPasswordRDao;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private SysUserPasswordUtil userPasswordUtil;

    @Resource(name = CommonConstWords.IMPP_REDIS_CORE)
    protected ImppRedis redisCore;

    @Override
    public BaseRepository<SysUserPassword, Long> getRepository() {
        return userPasswordRDao;
    }

    @Override
    public void updatePassword(Long userId, String password, String newPwd) {
        SysUser user = userService.getSysUserById(userId);
        ValidatorBean.checkNotNull(user, "不存在的用户信息");

        SysUserPassword userPassword = userPasswordRDao.getById(user.getUserLoginPasswordId());
        ValidatorBean.checkNotNull(userPassword, "不存在的用户密码信息");
        if (!StringUtils.equals(userPassword.getUserPassword(), SysUserPasswordUtil.encoder(password))) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("原始密码错误")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        updatePassword(userId, newPwd);
    }

    @Override
    public void updatePassword(String loginName, String password, String newPwd) {
        SysUser user = userService.getSysUserByLoginName(loginName);
        SysUserPassword userPassword = userPasswordRDao.getById(user.getUserLoginPasswordId());
        ValidatorBean.checkNotNull(userPassword, "不存在的用户密码信息");
        if (StringUtils.equals(userPassword.getUserPassword(), password)) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("原始密码错误")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        updatePasswordResetLastModifyTime(user.getId(), newPwd, true);
    }

    @Override
    public void updatePassword(Long userId, String password) {
        SysUser user = userService.getSysUserById(userId);
        ValidatorBean.checkNotNull(user, "不存在的用户信息");
        user.setUserLoginPassword(password);
        userPasswordUtil.checkPasswordSave(user);

        String frequencyDay = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_DAY);
        String redisKey = CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_DAY+"_"+user.getId()+"_"+user.getUserLoginName();
        //校验密码修改次数
        Object redisfrequencyDay = redisCore.getObject(redisKey);

        if (redisfrequencyDay == null) {
            redisCore.putObject(redisKey, frequencyDay, Integer.parseInt(frequencyDay)*CommonConstWords.REDIS_TIME_DAY_ONE);
            user.setModifyPwdNum(1);
        } else {
            int modifyNum = 0;
            Integer modifyUnlockNum = user.getModifyPwdNum();
            if (null != modifyUnlockNum) {
                modifyNum = modifyUnlockNum;
            }
            user.setModifyPwdNum(modifyNum++);
        }

        SysUserPassword pwd = new SysUserPassword();
        pwd.setUserId(userId);
        pwd.setUserPassword(SysUserPasswordUtil.encoder(password));
        ConvertBean.serviceModelInitialize(pwd, AuthUtil.getSessionUser());
        SysUserPassword save = userPasswordRDao.save(pwd);

        user.setUserLoginPasswordId(save.getId());
        user.setUserPasswordLastModifyTime(TimeTool.getNowTime(true));
        user.setUserLoginPassword(SysUserPasswordUtil.encoder(password));
        ConvertBean.serviceModelUpdate(user, AuthUtil.getSessionUser().getUserName());
        userService.updateSysUser(user);
    }

    @Override
    public void updatePasswordResetLastModifyTime(Long userId, String password, Boolean isCheckPassword) {
        SysUser user = userService.getSysUserById(userId);
        ValidatorBean.checkNotNull(user, "不存在的用户信息");
        user.setUserLoginPassword(password);
        if (isCheckPassword){
            userPasswordUtil.checkPasswordSave(user);
        }

        userPasswordRDao.updateByProperties(
                new String[]{"userId", "isDeleted"},
                new Object[]{user.getId(), CommonEnumUtil.IS_DEAL.NO.getValue()},
                new String[]{"isDeleted", "modifyDatetime", "modifyUser"},
                new Object[]{
                        CommonEnumUtil.IS_DEAL.YES.getValue(), TimeTool.getNowTime(true), AuthUtil.getSessionUser().getUserName()});

        SysUserPassword pwd = new SysUserPassword();
        pwd.setUserId(userId);
        pwd.setUserPassword(SysUserPasswordUtil.encoder(password));
        ConvertBean.serviceModelInitialize(pwd, AuthUtil.getSessionUser());
        SysUserPassword save = userPasswordRDao.save(pwd);

        user.setUserLoginPasswordId(save.getId());
        // 松下需要 重置密码之后 重新登录重新设置密码
        user.setUserPasswordLastModifyTime(TimeTool.getNowTime(true));
        // 重置密码设置重置密码的时间，方便充值后重新技术校验密码重复次数
        user.setResetPasswordTime(TimeTool.getNowTime(true));
        user.setUserLoginPassword(SysUserPasswordUtil.encoder(password));
        ConvertBean.serviceModelUpdate(user, AuthUtil.getSessionUser().getUserName());
        userService.updateSysUser(user);
    }

    @Override
    public void updatePasswordWizoutSession(String loginName, String password, String newPwd) {
        SysUser user = userService.validatePasswordByLoginNameAndReturnUser(loginName, password);
        user.setUserLoginPassword(newPwd);
        userPasswordUtil.checkPasswordSave(user);

        //供应商,修改密码校验修改次数
        String frequencyDay = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_DAY);
        String redisKey = CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_DAY+"_"+user.getId()+"_"+user.getUserLoginName();
        //校验密码修改次数
        Object redisfrequencyDay = redisCore.getObject(redisKey);

        if (redisfrequencyDay == null) {
            redisCore.putObject(redisKey, frequencyDay, Integer.parseInt(frequencyDay)*CommonConstWords.REDIS_TIME_DAY_ONE);
            user.setModifyPwdNum(1);
        } else {
            Integer modifyUnlockNum = user.getModifyPwdNum();
            int modifyNum = 0;
            if(null != modifyUnlockNum){
                modifyNum = modifyUnlockNum;
            }
            user.setModifyPwdNum(++modifyNum);
        }
        userService.updateSysUser(user);
        //当前方法是不需要session,是在登陆前弹窗修改密码
        //如果是首次登录修改密码，则设置登录次数为1
        if(null == user.getUserLoginNum() || user.getUserLoginNum() == 0){
            userService.updateUserLoginNum(user.getId(),1);
        }
            userPasswordRDao.updateByProperties(
                    new String[]{"userId", "isDeleted"},
                    new Object[]{user.getId(), CommonEnumUtil.IS_DEAL.NO.getValue()},
                    new String[]{"isDeleted", "modifyDatetime", "modifyUser",},
                    new Object[]{
                            CommonEnumUtil.IS_DEAL.YES.getValue(), TimeTool.getNowTime(true), loginName});


        SysUserPassword pwd = new SysUserPassword();
        pwd.setUserId(user.getId());
        pwd.setUserPassword(SysUserPasswordUtil.encoder(newPwd));
        ConvertBean.serviceModelInitialize(pwd, loginName);
        SysUserPassword save = userPasswordRDao.save(pwd);
        user.setUserLoginPasswordId(save.getId());
        user.setUserPasswordLastModifyTime(TimeTool.getNowTime(true));
        user.setUserLoginPassword(SysUserPasswordUtil.encoder(newPwd));
        ConvertBean.serviceModelUpdate(user, loginName);
        userService.updateSysUser(user);
    }

    @Override
    public List<SysUserPassword> findPasswordByPassword(String password,Long userId) {
        List<SysUserPassword> passwordList = new ArrayList<>();
        if(StringUtils.isBlank(password) || userId == null){
            return passwordList;
        }
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(password,"userPassword",ddlPackBean);
        DdlPreparedPack.getNumEqualPack(userId,"userId",ddlPackBean);
        return userPasswordRDao.findByHqlWhere(ddlPackBean);
    }

    @Override
    public List<SysUserPassword> findPasswordByPasswordAndResetPwdTime(String password,Long userId) {
        List<SysUserPassword> passwordList = new ArrayList<>();
        if(StringUtils.isBlank(password) || userId == null){
            return passwordList;
        }
        SysUser user = userService.getSysUserById(userId);
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(password,"userPassword",ddlPackBean);
        DdlPreparedPack.getNumEqualPack(userId,"userId",ddlPackBean);
        //当重置密码时间不为空时，查出在重置密码之后所修改的密码
        if(null != user && user.getResetPasswordTime() != null){
            DdlPreparedPack.getStringBiggerPack(user.getResetPasswordTime(), "createDatetime",ddlPackBean);
        }
        List<SysUserPassword> byHqlWhere = userPasswordRDao.findByHqlWhere(ddlPackBean);
        return byHqlWhere;
    }

    @Override
    public void updatePasswordByPwdIdAndNewPwd(Long pwdId, String newPwd) {
        try {
            SysUserPassword sup = userPasswordRDao.getById(pwdId);
            sup.setUserPassword(SysUserPasswordUtil.encoder(newPwd));
            sup.setModifyDatetime(TimeTool.getNowTime(true));
            userPasswordRDao.update(sup);
        } catch (Exception e) {
            throw ImppExceptionBuilder.newInstance().setErrorDetail(e.getMessage()).build();
        }
    }
}
