package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.util.PlatformConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysOrderNoRule;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;

import static cn.estsh.i3plus.platform.common.util.PlatformConstWords.SERIAL_NO;

/**
 * @Description : 单号生成工具类
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-21 13:26
 * @Modify:
 **/
public class OrderNoMakeUtil {

    private OrderNoMakeUtil() {
    }

    public static final BigDecimal TEN = new BigDecimal(10);

    public static String getOrderNoTemplate(SysOrderNoRule orderNoRule) {
        // 数据校验
        if (orderNoRule == null || StringUtils.isBlank(orderNoRule.getOrderNoRule())) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("单号生成规则不存在")
                    .build();
        }

        String[] dateTime = new SimpleDateFormat("yyyy-yy-MM-dd-HH-mm-ss").format(System.currentTimeMillis()).split("-");
        StringBuilder orderNoTemplate = new StringBuilder(orderNoRule.getOrderNoRule());

        // 单号表达式
        replace(orderNoTemplate, PlatformConstWords.YEAR_FULL, dateTime[0]);
        replace(orderNoTemplate, PlatformConstWords.YEAR, dateTime[1]);
        replace(orderNoTemplate, PlatformConstWords.MONTH, dateTime[2]);
        replace(orderNoTemplate, PlatformConstWords.DAY, dateTime[3]);
        replace(orderNoTemplate, PlatformConstWords.HOUR_OF_DAY, dateTime[4]);
        replace(orderNoTemplate, PlatformConstWords.MINUTE, dateTime[5]);
        replace(orderNoTemplate, PlatformConstWords.SECOND, dateTime[6]);

        return orderNoTemplate.toString();
    }

    /**
     * @param orderNoRule
     * @param orderNoTemplate
     * @return
     */
    public static SysOrderNoRule next(SysOrderNoRule orderNoRule, String orderNoTemplate) {
        int prefixIdx = orderNoTemplate.indexOf(SERIAL_NO);

        // 流水号长度补全表达式
        String serialNoFormatStr = MessageFormat.format(PlatformConstWords.SERIAL_NO_FORMAT, orderNoRule.getSerialNoLength());

        // 判断是否首次生成（SerialNo初始值-1）
        if (orderNoRule.getSerialNo() > 0 && prefixIdx != -1) {
            long serialNo = orderNoRule.getSerialNo() + orderNoRule.getSerialNoIncrement();
            BigDecimal maxSerialNo = TEN.pow(orderNoRule.getSerialNoLength()).subtract(BigDecimal.ONE);
            if (CommonEnumUtil.TRUE_OR_FALSE.valueOfBoolean(orderNoRule.getIsFixedLength())
                    && serialNo > maxSerialNo.longValue()) {
                serialNo = new BigDecimal(serialNo).remainder(maxSerialNo).longValue();
                orderNoRule.setSerialNo(serialNo == 0 ? maxSerialNo.longValue() : serialNo);
            }else{
                orderNoRule.setSerialNo(serialNo);
            }
        } else {
            orderNoRule.setSerialNo(orderNoRule.getSerialNoSeed() * orderNoRule.getSerialNoIncrement());
        }

        orderNoRule.setOrderNo(orderNoTemplate.replace(SERIAL_NO, String.format(serialNoFormatStr, orderNoRule.getSerialNo())));
        return orderNoRule;
    }

    /**
     *
     * @param orderNoRule
     * @param orderNoTemplate
     * @return
     */
    public static boolean isCycleByPrefix(SysOrderNoRule orderNoRule, String orderNoTemplate){
        int prefixIdx = orderNoTemplate.indexOf(SERIAL_NO);
        String newNo = orderNoTemplate.substring(0, prefixIdx);
        String oldNo = orderNoRule.getOrderNo().substring(0, prefixIdx);
        return !newNo.equals(oldNo) && CommonEnumUtil.TRUE_OR_FALSE.valueOfBoolean(orderNoRule.getIsCycle());
    }

    /**
     * 替换字符串
     *
     * @param no
     * @param constant
     * @param param
     */
    private static void replace(StringBuilder no, String constant, Object param) {
        int day = no.indexOf(constant);
        if (day != -1) {
            no.replace(day, day + constant.length(), param.toString());
        }
    }
}
