package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.platform.plugin.sqltool.IBaseSqlService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.persistence.Column;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2022/1/7 14:19
 * @Modify:
 **/
@Component
@Slf4j
public class SeparatorDataUtil {
    @Autowired
    private IBaseSqlService baseSqlService;

    /**
     * 包装 插入sql
     *
     * @param bean
     * @param destTableName
     * @return
     */
    public String packInsertSql(HashMap<String, Object> objMap, String destTableName) {
        String insertSQL = baseSqlService.packInsertSQL(destTableName, objMap);
        return insertSQL;
    }

    /**
     * 获取封装的k-v值
     *
     * @param bean
     * @return
     */
    private HashMap<String, Object> getObjMap(Object bean) {
        Field[] declaredFields = bean.getClass().getDeclaredFields();
        Field[] parentFields = bean.getClass().getSuperclass().getDeclaredFields();
        HashMap<String, Object> objMap = new HashMap<>(declaredFields.length);
        List<Field[]> list = new ArrayList<>();
        list.add(declaredFields);
        list.add(parentFields);
        list.forEach(item -> {
            for (Field field : item) {
                field.setAccessible(true);
                if (null != field.getAnnotation(Column.class)) {
                    Column annotation = field.getAnnotation(Column.class);
                    String columnName = annotation.name();
                    try {
                        Object value = field.get(bean);
                        objMap.put(columnName, value);
                    } catch (IllegalAccessException e) {
                        log.error("冷热分离Mysql封装数据错误:{}!", columnName);
                    }
                }
            }
        });
        return objMap;
    }
}
