package cn.estsh.i3plus.core.apiservice.util;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserPasswordService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysConfig;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.bean.SysUserPassword;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import javax.naming.NamingException;
import javax.naming.ldap.LdapContext;
import javax.persistence.EntityManager;
import java.io.UnsupportedEncodingException;
import java.security.NoSuchAlgorithmException;
import java.text.ParseException;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 20-6-8 上午10:10
 * @Modify:
 **/
@Component
public class SysUserPasswordUtil {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserPasswordUtil.class);

    @Autowired
    private ISysUserPasswordService userPasswordService;

    @Lazy
    @Autowired
    private ISysUserService userService;

    @Resource(name = CommonConstWords.IMPP_REDIS_CORE)
    protected ImppRedis redisCore;

    /**
     * 大写字符char数组
     */
    public static char[] upperChars = new char[]{'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'};
    /**
     * 小写字符char数组
     */
    public static char[] lowerChars = new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
    /**
     * 数字字符char数字
     */
    public static char[] numChars = new char[]{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
    //char[] specialChars = new char[]{' ', '!', '"', '#', '$', '%', '&', '\'', '(', ')', '*', '+', ',', '-', '.', '/', ':', ';', '<', '=', '>', '?', '@', '[', '\\', ']', '^', '_', '`', '{', '|', '}', '~'};
    /**
     * 特殊字符char数组
     */
    //去除一些特殊字符
    public static char[] specialChars = new char[]{'!', '#', '$', '%', '&', '*', '+', '-', '?', '@', '^', '_', '~'};

    /**
     * 大写字符char数组的名称
     */
    public static String UPPER_CHAR_ARRAY_NAME = "UPPER";
    /**
     * 小写字符char数组的名称
     */
    public static String LOWER_CHAR_ARRAY_NAME = "LOWER";
    /**
     * 数字字符char数组的名称
     */
    public static String NUM_CHAR_ARRAY_NAME = "NUM";
    /**
     * 特殊字符char数组的名称
     */
    public static String SPECIAL_CHAR_ARRAY_NAME = "SPECIAL";

    /**
     * 密码加密  返回加密信息
     *
     * @param password
     * @return
     */
    public static final String encoder(String password) {
        try {
            return EncryptTool.hexMD5(password);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        throw ImppExceptionBuilder.newInstance()
                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                .setErrorDetail("密码加密失败")
                .setErrorSolution("请重新操作")
                .build();
    }

    /**
     * 登录时校验的密码安全策略
     * @param user
     */
    public void checkPasswordLogin(SysUser user) {
        //
        SysConfig sysConfig = RedisCacheTool.getSysConfigByConfigCode(CommonConstWords.CONFIG_PWD_LENGTH);
        if(StringUtils.isEmpty(user.getUserPasswordLastModifyTime()) || user.getUserPasswordLastModifyTime().compareTo(sysConfig.getModifyDatetime()) >= 0) {
            /* 密码 长度校验 */
            checkPasswordLength(user.getUserLoginPassword());
            /* 密码 大写字母校验 */
            checkPasswordUppercaseEnglish(user.getUserLoginPassword());
            /* 密码 小写字母校验 */
            checkPasswordlowerEnglish(user.getUserLoginPassword());
            /* 密码 数字校验 */
            checkPasswordNumber(user.getUserLoginPassword());
            /* 密码 特殊字符校验 */
            checkPasswordSpecial(user.getUserLoginPassword());
            /* 密码 密码正则校验 */
            checkPasswordRepeat(user.getUserLoginPassword());
        }
    }

    /******************************************** 用户保存密码检查 ********************************************/

    public void checkPasswordSave(SysUser user) {
        /* 密码 长度校验 */
        checkPasswordLength(user.getUserLoginPassword());
        /* 密码 大写字母校验 */
        checkPasswordUppercaseEnglish(user.getUserLoginPassword());
        /* 密码 小写字母校验 */
        checkPasswordlowerEnglish(user.getUserLoginPassword());
        /* 密码 数字校验 */
        checkPasswordNumber(user.getUserLoginPassword());
        /* 密码 特殊字符校验 */
        checkPasswordSpecial(user.getUserLoginPassword());
        /* 密码 重复使用校验 */
        checkPasswordRepeatDay(user);
        /* 密码 密码正则校验 */
        checkPasswordRepeat(user.getUserLoginPassword());
        /* 密码 用户修改密码频次校验 */
        checkPasswordFrequency(user);
    }

    /**
     * 密码长度检查
     *
     * @param content
     */
    private void checkPasswordLength(String content) {
        int length = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_PWD_LENGTH, CommonConstWords.CONFIG_PWD_LENGTH_DEFAULT);

        if (StringUtils.isBlank(content) || StringUtils.length(content) < length) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("密码长度不够" + length + "位")
                    .setErrorSolution("请重新输入")
                    .build();
        }
    }

    /**
     * 密码 大写字母检查
     *
     * @param content
     */
    private void checkPasswordUppercaseEnglish(String content) {
        boolean isUppercaseEnglish = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXIST_UPPERCASE_ENGLISH, CommonEnumUtil.TRUE_OR_FALSE.FALSE);

        if (StringUtils.isNotBlank(content) && isUppercaseEnglish) {
            char[] chars = content.toCharArray();
            int num = 0;

            for (char ch : chars) {
                // Chart ASCLL 编码比对 A-Z
                if (ch >= 65 && ch <= 90) {
                    ++num;
                }
            }

            if (num <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码必须包含大写英文")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }
    }

    /**
     * 密码 小写字母检查
     *
     * @param content
     */
    private void checkPasswordlowerEnglish(String content) {
        boolean isUppercaseEnglish = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXIST_LOWER_ENGLISH, CommonEnumUtil.TRUE_OR_FALSE.FALSE);

        if (StringUtils.isNotBlank(content) && isUppercaseEnglish) {
            char[] chars = content.toCharArray();
            int num = 0;

            for (char ch : chars) {
                // Chart ASCLL 编码比对 a-z
                if (ch >= 97 && ch <= 122) {
                    ++num;
                }
            }

            if (num <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码必须包含小写英文")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }
    }

    /**
     * 密码 数字检查
     *
     * @param content
     */
    private void checkPasswordNumber(String content) {
        boolean isUppercaseEnglish = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXIST_NUMBER, CommonEnumUtil.TRUE_OR_FALSE.FALSE);

        if (StringUtils.isNotBlank(content) && isUppercaseEnglish) {
            char[] chars = content.toCharArray();
            int num = 0;

            for (char ch : chars) {
                // Chart ASCLL 编码比对 0-9
                if (ch >= 48 && ch <= 57) {
                    ++num;
                }
            }

            if (num <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码必须包含数字")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }
    }

    /**
     * 密码 特殊字符检查
     *
     * @param content
     */
    private void checkPasswordSpecial(String content) {
        boolean isUppercaseEnglish = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXIST_SPECIAL_CHAR, CommonEnumUtil.TRUE_OR_FALSE.FALSE);

        if (StringUtils.isNotBlank(content) && isUppercaseEnglish) {
            char[] chars = content.toCharArray();
            int num = 0;

            for (char ch : chars) {
                // Chart ASCLL 编码比对 特殊字符
                if ((ch >= 32 && ch <= 47) || (ch >= 58 && ch <= 64) ||
                        (ch >= 91 && ch <= 96) || (ch >= 123 && ch <= 126)) {
                    ++num;
                }
            }

            if (num <= 0) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码必须包含特殊字符")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }
    }

    /**
     * 密码 中文字符检查
     *
     * @param content
     */
    private void checkPasswordSpecialCn(String content) {
        boolean isUppercaseEnglish = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXIST_SPECIAL_CHAR, CommonEnumUtil.TRUE_OR_FALSE.FALSE);

        if (StringUtils.isNotBlank(content) && isUppercaseEnglish) {
            if (!Pattern.compile("[\u4e00-\u9fa5]").matcher(content).find()) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码必须包含中文字符")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }
    }

    /**
     * 近期密码重复检查
     *
     * @param user
     */
    private void checkPasswordRepeatDay(SysUser user) {
        int num = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_PWD_REPEAT_DAY, CommonConstWords.CONFIG_PWD_REPEAT_DAY_DEFAULT);
        if (num > 0) {
            try {
//                SysUserPassword userPassword = new SysUserPassword();
//                userPassword.setUserId(user.getId());
//                userPassword.setUserPassword(EncryptTool.hexMD5(user.getUserLoginPassword()));

//                List<SysUserPassword> passwords = userPasswordService.findAllByBean(userPassword);

                //修改为查询这个账号最近的密码是否重复
                List<SysUserPassword> passwords = userPasswordService.findPasswordByPasswordAndResetPwdTime(EncryptTool.hexMD5(user.getUserLoginPassword()),user.getId());

                int lastDay = Integer.MAX_VALUE;
                if (CollectionUtils.isNotEmpty(passwords)) {
                    for (SysUserPassword password : passwords) {
                        //防止日期转换错误
                        if (StringUtils.isNotBlank(password.getCreateDatetime())) {
                            int day = TimeTool.getSecoundsBetweenTime(4, password.getCreateDatetime(), TimeTool.getNowTime(true));
                            if (day < lastDay) {
                                lastDay = day;
                            }
                        }
                    }
                    if (num > lastDay) {
                        throw ImppExceptionBuilder.newInstance()
                                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                                .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                                .setErrorDetail(String.format("最近%s次内,请勿使用重复密码",num))
                                .setErrorSolution("请重新输入")
                                .build();
                    }
                }

//                if (lastDay > num) {
//                    throw ImppExceptionBuilder.newInstance()
//                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                            .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
//                            .setErrorDetail("请勿使用重复密码")
//                            .setErrorSolution("请重新输入")
//                            .build();
//                }
            } catch (ImppBusiException e) {
                throw e;
            } catch (Exception e) {
                e.printStackTrace();
                LOGGER.error("密码加密错误，异常信息：{}", e.getMessage());
            }
        }
    }

    /**
     * 密码 正则表达式教研
     *
     * @param content
     */
    private void checkPasswordRepeat(String content) {
        String regulars = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_PWD_REPEAT_REGULARS);

        if (StringUtils.isNotBlank(regulars) && regulars.contains(",")) {
            String[] split = regulars.split(",");

            try {
                for (String compile : split) {
                    if (StringUtils.isNotBlank(compile)) {
                        if (!Pattern.compile(compile).matcher(content).find()) {
                            throw ImppExceptionBuilder.newInstance()
                                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                                    .setErrorDetail("正则【" + compile + "】密码校验不通过")
                                    .setErrorSolution("请重新输入")
                                    .build();
                        }
                    }
                }
            } catch (ImppBusiException e) {
                // 业务异常 抛出
                throw e;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 密码 用户修改密码频次校验
     *
     * @param user
     */
    private void checkPasswordFrequency(SysUser user) {
        String frequencyDay = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_DAY);
        String frequencyNum = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_MODIFY_PWD_FREQUENCY_NUM);

        if (StringUtils.isNotBlank(frequencyDay) && StringUtils.isNotBlank(frequencyNum)) {
            //获取到修改的次数count
            Integer modifyUnlockNum = user.getModifyPwdNum();
            if(null!= modifyUnlockNum && modifyUnlockNum >= Integer.parseInt(frequencyNum)){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("密码【%s】天只能修改【%s】次",frequencyDay,frequencyNum)
                        .setErrorSolution("修改次数过多")
                        .build();
            }
        }
    }

    /**
     * 校验正确登录是否在自动锁定的时间
     *
     * @param user
     */
    private ResultBean checkAuthUnlockTime(SysUser user) {
        //校验自动解锁次数是否达到三次
        if (null != user.getAutoUnlockNum() && user.getAutoUnlockNum() >= 3){
            //锁定账号并设定锁定类型
            userService.doLockAndStatusSysUserById(user.getId(),CommonEnumUtil.LOCK_TYPE.LOGGER_TOO_MANY_FAILURES.getValue());
            return ResultBean.fail()
                    .setCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorMsg(LocaleUtils.getLocaleRes("用户名或密码多次输入错误。用户已被锁定请联系管理员!"));
        }
        String unlockRedisKey = CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUM_UNLOCK_TIME + "_" + user.getId();
        String unlockTime = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUM_UNLOCK_TIME);
        Object redisUnlockTime = redisCore.getObject(unlockRedisKey);
        if(null != redisUnlockTime){
            try {
                int secoundsBetweenTime = TimeTool.getSecoundsBetweenTime(1, redisUnlockTime.toString(), TimeTool.getNowTime(null));
                secoundsBetweenTime = Integer.parseInt(unlockTime) * 60 - secoundsBetweenTime;
                return ResultBean.fail()
                        .setCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorMsg(LocaleUtils.getLocaleRes("登录失败次数过多。请【" + secoundsBetweenTime/60 + "】分【" + secoundsBetweenTime%60 + "】秒后再试"));
            } catch (ParseException parseException) {
                parseException.printStackTrace();
                LOGGER.error("时间计算异常", parseException);
            }
        }

        return ResultBean.success();
    }


    /******************************************** 用户登录密码检查 ********************************************/
    public ResultBean checkSysUserLogin(SysUser user) {
        if (ImppSwitchUtil.isLoginActiveDirectory()) {
            // 使用AD域账号登录
            try {
                checkActiveDirectory(user);
            } catch (ImppBusiException e) {
                e.printStackTrace();
                return ResultBean.fail(e);
            } catch (Exception e) {
                return ResultBean.fail(e);
            }
        } else {
            /* 检查密码有效期 */
            ResultBean result = checkLoginPasswordTimeOut(user);
            if (!result.isSuccess()) {
                return result;
            }
            /* 登录 长时间未登录锁定 */
            result = checkLoginTimeOut(user);
            if (!result.isSuccess()) {
                return result;
            }
            /* 判断是否第一次登录  */
            result = checkFirstLoginChangePwd(user);
            if (!result.isSuccess()) {
                return result;
            }
            /* 判断自动锁定时间  */
            result = checkAuthUnlockTime(user);
            if (!result.isSuccess()) {
                return result;
            }
            //登录时是否检查用户密码是否符合安全规则
            try {
              int isLoginCheckPassword = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_LOGIN_CHECK_PASSWORD,CommonConstWords.CONFIG_LOGIN_CHECK_PASSWORD_DEFAULT);
                if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == isLoginCheckPassword){
                    checkPasswordLogin(user);
                }
                result = ResultBean.success();
            } catch (Exception e) {
                LOGGER.error("检查用户密码是否符合安全规则发生异常:",e);
                result = ResultBean.fail()
                        .setCode(ImppExceptionEnum.LOGIN_EXCEPTION_PWD_RULE_CHANGE.getCode())
                        .setErrorMsg(ImppExceptionEnum.LOGIN_EXCEPTION_PWD_RULE_CHANGE.getDescription());;
            }
            if (!result.isSuccess()) {
                return result;
            }
        }
        return ResultBean.success();
    }

    /**
     * 账号 ActiveDirectory 登录集成
     *
     * @param user
     */
    public void checkActiveDirectory(SysUser user) {
        if (ImppSwitchUtil.isLoginActiveDirectory()) {
            String activeDirectoryUrl = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_PWD_ACTIVE_DIRECTORY_URL);

            if (StringUtils.isNotBlank(activeDirectoryUrl)) {
                try {
                    LdapContext context = ImppLdapUtils.getLdapContext(activeDirectoryUrl, user.getUserLoginName(), user.getUserLoginPassword());
                    // 释放链接资源
                    context.close();
                } catch (ImppBusiException e) {
                    throw e;
                } catch (NamingException e) {
                    e.printStackTrace();
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                            .setErrorDetail("ActiveDirectory 登录失败请重新登录")
                            .setErrorSolution("请重新操作")
                            .build();
                } catch (Exception e) {
                    e.printStackTrace();
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                            .setErrorDetail("ActiveDirectory 登录失败请重新登录")
                            .setErrorSolution("请重新操作")
                            .build();
                }
            } else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                        .setErrorDetail("ActiveDirectory 未配置链接地址")
                        .setErrorSolution("请联系管理员")
                        .build();
            }
        }
    }

    /**
     * 登录 错误次数检查
     */
    public void checkLoginErrorNumber(String loginName, ResultBean resultBean, Exception e) {
        SysUser user = userService.getSysUserByLoginName(loginName);
        if (!user.getUserLoginName().equals(loginName)){
            user = null;
        }
        if (Objects.nonNull(user)) {
            checkLoginErrorNumber(user.getId(), resultBean, e);
        }
    }

    /**
     * 登录 失败次数检查
     *
     * @param userId
     * @param resultBean
     * @param e
     */
    public void checkLoginErrorNumber(Long userId, ResultBean resultBean, Exception e) {
        int loginErrorNumberMax = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUMBER, CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUMBER_DEFAULT);
        String today = TimeTool.getToday();

        int loginErrorNumber = 0;
        String redisKey = CommonConstWords.USER_LOGIN_ERROR + ":" + today + ":" + userId;
        String redisAuthUnlockKey = CommonConstWords.USER_LOGIN_ERROR + ":" + "authUnlockKey" + ":" + today + ":" + userId;

        try {
            Object redisValue = redisCore.getObject(redisKey);
            if (Objects.nonNull(redisValue)) {
                loginErrorNumber = (Integer) redisValue;
            }
        } catch (Exception exception) {
            LOGGER.error("获取登录异常次数错误，错误信息：{}", exception.getMessage());
        }

        Object authUnlockNum = redisCore.getObject(redisAuthUnlockKey);

        if (Objects.equals(loginErrorNumberMax, loginErrorNumber) && authUnlockNum != null && Integer.parseInt(authUnlockNum.toString())>=3) {
            // 锁定账号信息
            userService.doLockSysUserById(userId);

            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("账号已被锁定")
                    .setErrorSolution("请联系系统管理员")
                    .build();
        } else if (loginErrorNumber > loginErrorNumberMax && authUnlockNum != null && Integer.parseInt(authUnlockNum.toString())>=3) {
            // 锁定账号信息
            userService.doLockSysUserById(userId);
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                    .setErrorDetail("账号已锁定")
                    .setErrorSolution("请联系系统管理员")
                    .build();
        }

        // 登录异常 更新登录失败次数
        if (Objects.nonNull(e)) {
            ++loginErrorNumber;

            if (Objects.nonNull(resultBean)) {
                int num = loginErrorNumberMax - loginErrorNumber;
                if (num <= 0) {
                    int autoUnlockNum = 0;
                    String unlockTime = RedisCacheTool.getSysConfigStrVal(CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUM_UNLOCK_TIME);
                    String unlockRedisKey = CommonConstWords.CONFIG_USER_LOGIN_ERROR_NUM_UNLOCK_TIME + "_" + userId;

                    if (null != authUnlockNum) {
                        autoUnlockNum = Integer.parseInt(authUnlockNum.toString());
                    }
                    Object redisUnlockTime = redisCore.getObject(unlockRedisKey);

                    if (null == redisUnlockTime) {
                        redisCore.putObject(unlockRedisKey, TimeTool.getNowTime(null), Long.parseLong(unlockTime) * 60L);
                        redisCore.putObject(redisAuthUnlockKey, autoUnlockNum++, CommonConstWords.REDIS_TIME_DAY_ONE);
                        if(autoUnlockNum >= 3 ){
                            userService.doLockAndStatusSysUserById(userId,CommonEnumUtil.LOCK_TYPE.LOGGER_TOO_MANY_FAILURES.getValue());
                            redisCore.deleteKey(unlockRedisKey);
                            throw ImppExceptionBuilder.newInstance()
                                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                                    .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                                    .setErrorDetail("登录失败次数过多")
                                    .setErrorSolution("请联系系统管理员")
                                    .build();
                        }
                        resultBean.setErrorMsg(LocaleUtils.getLocaleRes("用户名或密码输入错误。请【" + unlockTime + "】分钟后再试"));
                    } else {
                        try {
                            int secoundsBetweenTime = TimeTool.getSecoundsBetweenTime(1, redisUnlockTime.toString(), TimeTool.getNowTime(null));
                            secoundsBetweenTime = Integer.parseInt(unlockTime) * 60 - secoundsBetweenTime;
                            resultBean.setErrorMsg(LocaleUtils.getLocaleRes("登录失败次数过多。请【" + secoundsBetweenTime/60 + "】分【" + secoundsBetweenTime%60 + "】秒后再试"));
                        } catch (ParseException parseException) {
                            parseException.printStackTrace();
                            LOGGER.error("时间计算异常", parseException);
                        }
                    }
                } else {
                    resultBean.setErrorMsg(String.format(LocaleUtils.getLocaleRes("用户名或密码输入错误。还有 %s 次机会"), num));
                }
            }
            redisCore.putObject(redisKey, loginErrorNumber, CommonConstWords.REDIS_TIME_DAY_ONE);
        }
    }

    /**
     * 登录 长时间未登录锁定
     */
    private ResultBean checkLoginTimeOut(SysUser user) {
        try {
            if (StringUtils.isBlank(user.getUserLoginLastDateTime())) {
                return ResultBean.fail("上次登录时间为空");
            }
            boolean longTimenNotLogin = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_USER_UNLOGIN_LONG_TIME_LOCK, CommonEnumUtil.TRUE_OR_FALSE.FALSE);
            int loginDayMax = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_USER_LOGIN_DAY_OUT, CommonConstWords.CONFIG_USER_LOGIN_DAY_OUT_DEFAULT);
            if (longTimenNotLogin && loginDayMax > 0) {

                int day = TimeTool.getSecoundsBetweenTime(4, user.getUserLoginLastDateTime(), TimeTool.getNowTime(true));
                if (day > loginDayMax) {
                    // 锁定账号信息
                    //todo: 账户没有被锁定,由于异常事务回滚,导致账户没有被锁定,这个更新操作就是多余的
                    userService.doLockSysUserById(user.getId());

//                    throw ImppExceptionBuilder.newInstance()
//                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                            .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
//                            .setErrorDetail("长时间未登录账号已被锁定")
//                            .setErrorSolution("请联系系统管理员")
//                            .build();
                    ImppBusiException exception = ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
                            .setErrorDetail("长时间未登录账号已被锁定")
                            .setErrorSolution("请联系系统管理员")
                            .build();
                    return ResultBean.fail(exception);

                }
            }
        } catch (ImppBusiException e) {
//            throw e;
            return ResultBean.fail(e);
        } catch (Exception e) {
            LOGGER.error("长时间未登录处理异常，异常信息：{}", e.getMessage());
            return ResultBean.fail(e);
        }
        return ResultBean.success();
    }

    /**
     * 登录 检查密码有效期
     */
    private ResultBean checkLoginPasswordTimeOut(SysUser user) {
        try {
            boolean pwdExpireSwitch = RedisCacheTool.getSysConfigBooleanVal(CommonConstWords.CONFIG_PWD_EXPIRE_SWITCH, CommonEnumUtil.TRUE_OR_FALSE.FALSE);
//            int passwordDayMax = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_PWD_VALID_DAY, CommonConstWords.CONFIG_PWD_VALID_DAY_DEFAULT);
            Integer userPasswordOverdueValue = user.getUserPasswordOverdue();

            if (pwdExpireSwitch && userPasswordOverdueValue == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) {
                SysConfig sysConfigByConfigCode = RedisCacheTool.getSysConfigByConfigCode(CommonConstWords.CONFIG_PWD_EXPIRE_DAY_TIME);
                String configValue = sysConfigByConfigCode.getConfigValue();
                SysUserPassword password = userPasswordService.get(user.getUserLoginPasswordId());
                if (Objects.nonNull(password) && StringUtils.isNotBlank(password.getCreateDatetime())) {
                    int day = TimeTool.getSecoundsBetweenTime(4, password.getCreateDatetime(), TimeTool.getNowTime(true));
                    if (day > Integer.parseInt(configValue)) {
                        // 锁定账号信息
//                        userService.doLockSysUserById(user.getId());

//                        throw ImppExceptionBuilder.newInstance()
//                                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
//                                .setErrorCode(ImppExceptionEnum.NOT_CONFIG_EXCEPTION.getCode())
//                                .setErrorDetail("账号密码已过期")
//                                .setErrorSolution("请联系系统管理员")
//                                .build();
                        ImppBusiException exception = ImppExceptionBuilder.newInstance()
                                .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                                .setErrorCode(ImppExceptionEnum.LOGIN_EXCEPTION_PASSWORD_EXPIRED.getCode())
                                .setErrorDetail("账号密码已过期")
                                .setErrorSolution("请重新修改密码")
                                .build();
                        return ResultBean.fail(exception);
                    }
                }
            }
        } catch (ImppBusiException e) {
//            throw e;
            e.printStackTrace();
            return ResultBean.fail(e);
        } catch (Exception e) {
            LOGGER.error("密码有效期处理异常，异常信息：{}", e.getMessage());
            e.printStackTrace();
            return ResultBean.fail(e);
        }
        return ResultBean.success();
    }


    /**
     * 第一次登录是否修改密码
     */
    private ResultBean checkFirstLoginChangePwd(SysUser user) {

        try {
            int firstLoginChangePwd = RedisCacheTool.getSysConfigIntVal(CommonConstWords.FIRST_LOGIN_CHANGE_PWD_SWITCH_KEY, CommonConstWords.FIRST_LOGIN_CHANGE_PWD_SWITCH_DEFAULT_VALUE);
            if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == firstLoginChangePwd) {
                if (null == user.getUserLoginNum() || user.getUserLoginNum() <= 0) {
                    return ResultBean.fail()
                            .setCode(ImppExceptionEnum.LOGIN_EXCEPTION_CHANGE_PWD.getCode())
                            .setErrorMsg(ImppExceptionEnum.LOGIN_EXCEPTION_CHANGE_PWD.getDescription());
                }
            }
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            LOGGER.error("首次登录验证出错，异常信息：{}", e.getMessage());
            return ResultBean.fail(e);
        }
        return ResultBean.success();
    }


    /**
     * 根据密码安全策略里面的规则生成随机密码,重复性检验和正则表达式检验不进行校验
     * 密码保存规则里面有正则校验,正则校验可能会与其他规则冲突,导致生成的密码无法通过规则校验
     * 目前重置密码不会进行密码规则的校验
     *
     * @return string
     */
    public static String generateRandomPassword() {
        int length = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_PWD_LENGTH, CommonConstWords.CONFIG_PWD_LENGTH_DEFAULT);
        //todo: 修改密码长度的接口要做校验,长度不能超过最大值
        if (length < CommonConstWords.CONFIG_PWD_LENGTH_DEFAULT) {
            length = CommonConstWords.CONFIG_PWD_LENGTH_DEFAULT;
        }
        //四种类型的字符一定会有,所以先随机占用4个位置
        Random random = new Random();
        String[] typeArray = new String[]{UPPER_CHAR_ARRAY_NAME, LOWER_CHAR_ARRAY_NAME, NUM_CHAR_ARRAY_NAME, SPECIAL_CHAR_ARRAY_NAME};
        char[] randomCharArray = new char[length];
        int top = length;
        int[] indexArray = new int[length];
        for (int i = 0; i < indexArray.length; i++) {
            indexArray[i] = i;
        }
        Map<Integer, String> indexMap = new HashMap<>(length);
        //确定四个必须要的字符的位置,剩下的位置的字符随机从四种中选择
        for (int i = 0; i < 4; i++) {
            int index = random.nextInt(top);
            indexMap.put(indexArray[index], typeArray[i]);
            indexArray[index] = indexArray[top - 1];
            top--;
        }
        for (int i = 0; i < randomCharArray.length; i++) {
            //判断对应字符的位置是否已经被占用
            if (!ObjectUtils.isEmpty(indexMap.get(i))) {
                randomCharArray[i] = getRandomChar(indexMap.get(i), random);
            } else {
                //todo:这里可以把所有char合并一个数组做减少typeArray的random,直接从所有可选字符合并的大数组里面选一个字符
                randomCharArray[i] = getRandomChar(typeArray[random.nextInt(typeArray.length)], random);
            }
        }
        return String.valueOf(randomCharArray);
    }

    /**
     * 根据同的字符类型生成对应类型的随机字符
     * @param key
     * @param random
     * @return
     */
    public static char getRandomChar(String key, Random random) {
        if (UPPER_CHAR_ARRAY_NAME.equals(key)) {
            return upperChars[random.nextInt(upperChars.length)];
        } else if (LOWER_CHAR_ARRAY_NAME.equals(key)) {
            return lowerChars[random.nextInt(lowerChars.length)];
        } else if (NUM_CHAR_ARRAY_NAME.equals(key)) {
            return numChars[random.nextInt(numChars.length)];
        } else {
            return specialChars[random.nextInt(specialChars.length)];
        }
    }

}
